<?php
/**
 * Download a file
 *
 * URL format:
 * RewriteRule ^(dl|downloads)\/dlToken([0-9]+)(\/resId[a-zA-Z0-9]+){0,1}\/(.+)  /ext/download.php?dlToken=$2&file=$4&resId=$3 [L]
 * resId: resource id in current session, optional, skipped for direct downloads
 * dlToken: id of current download, used for download resume. See efTokensWrite / efTokensRead functions.
 * 			When JS generated, concatenation of ascii values of path+filename chars
 * file: rawurlencoded filename, includi
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */


/**
 * @desc display error page
 *
 * @param integer $errorCode : HTTP error code
 */
function  displayError($errorCode){
	require_once(INCLUDE_DIR.'outputFunctions.php');

	switch ($errorCode){
		case 403;
			$message='403 Forbidden';
			break;
		case 404:
		default:
			$message='404 Not Found';
			break;
	}
	header('HTTP/1.1 '.$message);
	cfLog($message .' - '.$_SERVER["REQUEST_URI"],LOG_DBG);
	wSession_write_close();
	outDisplayErrorPage($message .' - '.$_SERVER["REQUEST_URI"]);
	exit();
}

/**
 * @desc If it comes from a short link and it is a redirect frow weezo.net site, 1st display a download page so the top (weezo.net) frame doesn't display a "account cannot be contacted" error message
 *
 */
function downloadShowDownloadPage(){
	// If no referer is passed, don't display download page
	if(!($host=cfArrayItem(parse_url(WHeadersIn::get('referer')),'host'))) return;

	// If download page has already been displayed, proceed to download
	if(cfCmpLeft($_SERVER['REQUEST_URI'],'/shortLinkDownload/')) return;

	// Remove account name from host name (if present)
	str_replace('.','.',$host,$count);
	while($count>1){
		$host=substr($host,strpos($host,'.')+1);
		str_replace('.','.',$host,$count);
	}
	// If it's not a known host, proceed to download
	if(!in_array($host,$_ENV['W_ALT_HOSTS'])) return;

	// Set new link
	$link='/shortLinkDownload'.$_SERVER['REQUEST_URI'];

	require_once(INCLUDE_DIR.'outputFunctions.php');

	/**
	 * Display download page
	 */
	cfInsertHEAD();
	?>
<body>
<div style="width:100%;height:100%;overflow:hidden">
	<div style="position:absolute;top:50%;left:50%;">
		<div class="frame1 shade" style="width:200px;text-align:center;position:relative;left:-50%;top:-120px">
			<img src="<?php echo outIcon('appIcon')?>" alt="logo" style="width:48px;height:48px"><br/><br/>
			<div style="font-size:180%;text-decoration:none;color:#333"><?php echo @$_GET['file']?></div>
			<br>
			<span><?php echo cfCaption('downloadStarted'); ?></span>
		</div>
	</div>
</div>
<a style="text-decoration:none;color:#333;position:absolute;left:1em;bottom:1em" href="http://<?php echo APP_SITE;?>">weezo.net</a>
<script type="text/javascript">
wl.goURL("<?php echo $link;?>");
</script>
</body>
</html>
	<?php
	exit;
}

require_once(INCLUDE_DIR.'explorerFunctions.php');

// Save GET info as it might be erased by XSRF protection procedure (chrome download drag use top URL as referer so GET & POST are erased on session opening)
$tmpGET=$_GET;

// If it comes from a short link and it is a redirect frow weezo.net site, 1st display a download page so the top (weezo.net) frame doesn't display a "account cannot be contacted" error message
if(isset($_GET['shortLink'])) downloadShowDownloadPage();

// Start session and restore GET
wSession_start();
$_GET=$tmpGET;

$headers=getallheaders();

// Check IE download resume request
if(isset($headers['If-Modified-Since']) && preg_replace('/;.*$/', '', $headers['If-Modified-Since'])){
	header("HTTP/1.1 304 Not Modified");
	if(!isset($_SESSION['userLogged'])) wSession_write_close();
	exit;
}

// Centralized authentication token existence check by central server
if(isset($_GET['checkExistence'])){
	// Get download token
	if(!isset($_GET['dlToken'])) die();
	$dlToken=$_GET['dlToken'];	if(substr($dlToken,0,7)=='dlToken') $dlToken=substr($dlToken,7);
	// Load stored tokens and Check token existence
	$tokenList=efTokensRead();
	if(isset($tokenList[$dlToken])) die('ok'); else die('');
}

// If no session, only rely on download token
if(!isset($_SESSION['userLogged']) || strpos($_SERVER['REQUEST_URI'],'resId')===false) {
	if(!isset($_GET['dlToken'])) die();
	if(!isset($_SESSION['userLogged'])) wSession_destroy();
	efResumeDownload($_GET['dlToken']);
	exit;
}

// Get file name
$completeFilename=str_replace('*weezoPlus*','+',str_replace('*weezoSharp*','#',str_replace('&amp;','&',str_replace('*weezoAmp*','&',str_replace('*weezoPercent*','%',$_GET['file'])))));
$completeFilename=stripcslashes($completeFilename);

// Verify resource existence
if(substr($_GET['resId'],0,6)=='/resId') $_GET['resId']=substr($_GET['resId'],6);
if(!isset($_SESSION['res'][$_GET['resId']])) displayError(404);

// Switch to download's resource
$_SESSION['activeResourceId']=$_GET['resId'];

// verify IP address
if (!isset($_SESSION['userId']) || ($_SESSION['accountIP'] != $_SERVER['REMOTE_ADDR'] && cfGGetVar('disableClientSessionIPControl')!=true && !isset($_SERVER['HTTPS']))) displayError(403);

// verify that resource has a "path" property
if(!cfRGetVar('path')) displayError(403);

// Inject data into POST
$_POST['data1']=cfDirName(stripslashes($completeFilename));
$_POST['data2']=basename($completeFilename);
$_POST['data3']='download';
$_POST['data5']=$_GET['dlToken'];
$_GET ['data5']='dlToken'.$_GET['dlToken'];

// Proceed to POST processing
efProcessPOST();
?>