<?php
/**
 * Videos viewer script
 *
 * Included by viewFunctions.php
 * parameters :
 * - $completeFilename : complete filename of image to view
 * - $command : command
 * - $extraData : [image outer width]x[image outer height]
 *
 * Main resource config parameters used :
 * "reencodedVideo": true to allow flv real-time or pre-encoding
 * "flashStream":  true for real-time encoding, false for preencoding *** not used anymore since V210Q as preencoding is no more allowed => set to true
 * "flashOutputSize": flash (pre or real-time) encoding width (160, 320, 400, 640, 900, 1280)
 * "flashOutputQuality": flash (pre or real-time) encoding quality ('original' | 'high' | 'medium' | 'low')
 *
 * "allowOriginalVideo": allow direct viewing of source video (with any allowed player)
 * "useQTVideo": true to allow qt/mp4 preencoding when compatible device found
 * "qtOutputSize": qt/mp4 preencoding width (default value set to 480 = iPhone width)
 * "qtOutputQuality": qt/mp4 preencoding quality ('high' | 'medium' | 'low')

 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.1.0
 */

// Video players, from prefered to least wanted
$_ENV['preferedPlayers']=array(
	'flashWii'=>'LGPL',		// Flash player for Wii, FLV/VP6/mp3
	'flashF4v'=>'GPL',		// Flash player (F4V/h264/mp3)
	'html5Webm'=>'LGPL',	// HTML5 player (WebM/VP8/Vorbis)
	'html5Ogv'=>'LGPL',		// HTML5 player (Ogv/Theora/Vorbis)
	'html5Auto'=>'LGPL',	// HTML5 player (Ogv/Theora/Vorbis or WebM or H264, depending on browser)
	'flashFlv'=>'LGPL',		// Flash player (Flv/vp6/mp3)
	'divx'=>'raw',			// Raw file (divX web player)
	'wmp'=>'raw',			// Raw file (WMP)
	'qt'=>'raw',			// Raw file (Quicktime)
	'qtReenc'=>'GPL',		// iPhone player (mp4/h264/mp3)
	'm3u'=>'raw',			// m3u downloable file (pointing on raw file)
	'vlc'=>'raw'			// Raw file (VLC plugin)
);

// Mobile players, from prefered to least wanted
$_ENV['preferedMobilePlayers']=array(
	'flashWii'=>'LGPL',		// Flash player for Wii, FLV/VP6/mp3
	'flashF4v'=>'GPL',		// Flash player (F4V/h264/mp3)
	'flashFlv'=>'LGPL',		// Flash player (Flv/vp6/mp3)
	'html5Auto'=>'LGPL',	// HTML5 player (Ogv/Theora/Vorbis or WebM or H264, depending on browser)
	'html5Ogv'=>'LGPL',		// HTML5 player (Ogv/Theora/Vorbis)
	'html5Webm'=>'LGPL',	// HTML5 player (WebM/VP8/Vorbis)
	'divx'=>'raw',			// Raw file (divX web player)
	'wmp'=>'raw',			// Raw file (WMP)
	'qt'=>'raw',			// Raw file (Quicktime)
	'qtReenc'=>'GPL',		// iPhone player (mp4/h264/mp3)
	'm3u'=>'raw',			// m3u downloable file (pointing on raw file)
	'vlc'=>'raw'			// Raw file (VLC plugin)
);

// Is player stretchable
$_ENV['stretchablePlayer']=array(
	'flashWii'=>0,
	'flashFlv'=>1,
	'flashF4v'=>1,
	'html5Webm'=>1,
	'html5Ogv'=>1,
	'html5Auto'=>1,
	'divx'=>0,
	'flash'=>1,
	'wmp'=>0,
	'qt'=>0,
	'qtReenc'=>0,
	'm3u'=>1,
	'vlc'=>1
);
// Player controls height
$_ENV['playerControlsHeight']=array(
	'flashWii'=>0,
	'flashFlv'=>0,
	'flashF4v'=>0,
	'html5Webm'=>0,
	'html5Ogv'=>0,
	'html5Auto'=>0,
	'divx'=>26,
	'flash'=>0,
	'wmp'=>69,
	'qt'=>15,
	'qtReenc'=>15,
	'm3u'=>0,
	'vlc'=>0
);

// Supported files from extensions
$_ENV['supportedExt']=array(
	'flashWii'=>array('*'=>1),
	'flashFlv'=>array('*'=>1),
	'flashF4v'=>array('*'=>1),
	'html5Auto'=>array('*'=>1),
	'html5Webm'=>array('*'=>1),
	'html5Ogv'=>array('*'=>1),
	'divx'=>array('divx'=>1,'avi'=>1,'mp4'=>1),
	'flash'=>array('flv'=>1,'mp4'=>1,'mov'=>1),
	'wmp'=>array('*'=>1),
	'qt'=>array('mov'=>1,'mp4'=>1,'3gp'=>1,'3gp2'=>1,'mpg'=>1,'mpeg'=>1),
	'qtReenc'=>array('*'=>1),
	'm3u'=>array('*'=>1),
	'vlc'=>array('*'=>1)
);

if(!defined('W_EXPLORERFUNCTIONS_LOADED')) require(INCLUDE_DIR.'explorerFunctions.php');


/**
 * Class used to pass global parameters to video functions
 *
 */
class VVOptions{
	static private $options;
	static private $forbiddenPlayers=array();
	static private $extraPlayers=false;
	static private $dropDownMenu=false;
	static private $videoWindowAutoAdapt=false;
	static private $mobileTopControlsExtra=false;
	static private $desktopTopControlsExtra=false;

	function __construct(){
		self::$options=array();
	}

	/**
	 * @desc Forbid ratio modification by user
	 *
	 */
	static function setNoRatio(){
		self::$options['noRatio']=true;
	}
	/**
	 * @desc Return true if user may not modify video width/height ratio
	 *
	 * @return boolean
	 */
	static function noRatio(){
		return (self::$options['noRatio']===true)?true:false;
	}

	/**
	 * @desc set an array of forbidden video players (used by webcam resource to limit incompatible players)
	 *
	 * @param unknown_type $forbiddenPlayers
	 */
	static function setForbiddenPlayers($forbiddenPlayers){
		self::$forbiddenPlayers=$forbiddenPlayers;
	}
	/**
	 * @desc set an array of forbidden video players (used by webcam resource to limit incompatible players)
	 *
	 * @param unknown_type $forbiddenPlayers
	 */
	static function getForbiddenPlayers(){
		return self::$forbiddenPlayers;
	}

	/**
	 * @desc indicate Window must be automatically adapted to video size
	 *
	 */
	static function setWindowAutoAdapt(){
		self::$videoWindowAutoAdapt=1;
	}

	/**
	 * @desc Echo script for automatically resizing window to video size
	 *
	 * @param int $w
	 * @param int $h
	 */
	static function windowSizeAutoAdaptScript($w,$h){
		if(!self::$videoWindowAutoAdapt) return;
		echo 'winMe.resizeInnerTo('.$w.'+20,'.$h.'+60,1);';
	}

	/**
	 * @Desc Set $doUse to true to use drop-down menu instead of icon list for player
	 *
	 * @param string $doUse
	 */
	static function useDropDownMenuForPlayerSelection($doUse=null){
		if($doUse!==null) return self::$dropDownMenu=$doUse;
		return self::$dropDownMenu;
	}

	/**
	 * @desc Echo script for automatically resizing window to video size
	 *
	 * @param int $w
	 * @param int $h
	 */
	static function setExtraPlayers($list){
		self::$extraPlayers=$list;
	}

	/**
	 * @Desc Set $doUse to true to use drop-down menu instead of icon list for player
	 *
	 * @param string $doUse
	 */
	static function getExtraPlayers(){
		return self::$extraPlayers;
	}

	/**
	 * @desc Set extra top controls for mobile player
	 *
	 * @param string $extra
	 */
	static function setMobileTopControlsExtra($extra){
		self::$mobileTopControlsExtra=$extra;
	}

	/**
	 * @desc Get extra top controls for mobile player
	 *
	 * @return string
	 */
	static function getMobileTopControlsExtra(){
		return self::$mobileTopControlsExtra;
	}
	/**
	 * @desc Set extra top controls for desktop player
	 *
	 * @param string $extra
	 */
	static function setDesktopTopControlsExtra($extra){
		self::$desktopTopControlsExtra=$extra;
	}

	/**
	 * @desc Get extra top controls for desktop player
	 *
	 * @return string
	 */
	static function getDesktopTopControlsExtra(){
		return self::$desktopTopControlsExtra;
	}
}


/**
 * @desc Return allowed players according to
 * 		 - browser support
 * 		 - resource configuration
 * 		 - video file extension (optional)
 *
 * @param string $ext : video file extension (optional)
 * @return array: allowed players, in:
 * 			- flashWii (real-time or pre) encoded flash, wii version
 * 			- flashFlv: flash FLV/VP6
 * 			- flashFlv: flash F4V/H264
 * 			- html5Webm: google WebM
 * 			- html5Ogv: Ogv Theora
 * 			- divx (from source file)
 * 			- flash (from source file)
 * 			- qt (from source file)
 * 			- vlc (from source file)
 * 			- wmp (from source file)
 *
 */
function vvfAllowedPlayers($ext=false){
	$p=array();

	// Client-side detected plugins
	$cc=cfBGetVar('clientCapabilities');

	// Check what players can be used by resource / browser - Warning : $p order is important, it defines default chosen browser (1st)
	if(cfIsWII()) return array('flashWii'=>'Flash');

	if(cfRGetVar('reencodedVideo')) {

		// Flash
		if(cfRGetVar('flvAllowed') && $cc && @$cc['flash']){
			$p['flashFlv']='Flash Player';
			if(cfFfmpegVersion()=='GPL') $p['flashF4v']='Flash Player (H264)';
		}
		// HTML5 / WebM
		if(cfRGetVar('webmAllowed') && $cc && @$cc['html5VideoWebm']) $p['html5Webm']='HTML5 / WebM';

		// HTML5 / Ogg Video
		if(cfRGetVar('ogvAllowed') && $cc && @$cc['html5VideoOgv']) $p['html5Ogv']='HTML5 / Ogg Theora';

		// HTML5 / Auto
		if(!$cc && (cfRGetVar('webmAllowed') || cfRGetVar('ogvAllowed'))) $p['html5Auto']='HTML5 / Auto';
	}

	if(cfRGetVar('allowOriginalVideo')){
		if(($cc && @$cc['divx']) || (!$cc && cfBGetVar('divx')))	$p['divx']='DivX Web player';
		if(($cc && @$cc['flash']) || (!$cc && cfBGetVar('flash')))	$p['flash']='Flash';
		if(($cc && @$cc['qt']) || (!$cc && cfBGetVar('qt')))		$p['qt']='Quicktime';
		if(($cc && @$cc['vlc']) || (!$cc && cfBGetVar('vlc')))		$p['vlc']='Videolan';
		if(($cc && @$cc['wmp']) || (!$cc && cfBGetVar('wmp')))		$p['wmp']='Windows Media Player';
		if(cfBGetVar('playlistFormat')=='m3u')						$p['m3u']=cfCaption('musicCfgExternal');
		if(cfBGetVar('playlistFormat')=='asx')						$p['asx']=cfCaption('musicCfgExternal');
	}
	if(/*cfRGetVar('useQTVideo') && */cfBGetVar('qtReenc') && cfRGetVar('reencodedVideo'))	{
		if(cfFfmpegVersion()=='GPL') $p['qtReenc']='Quicktime';
	}
	//$p['qtReenc']='Quicktime'; // XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	// Check if file extension is supported by browsers
	if($ext){
		foreach ($p as $key=>$value) {

			if(!isset($_ENV['supportedExt'][$key][$ext]) && !isset($_ENV['supportedExt'][$key]['*'])) unset($p[$key]);
		}
		if(isset($p['flashWii']) && $ext!='flv' && !cfRGetVar('reencodedVideo')) unset($p['flashWii']);
	}

	if(!count($p)) return false;

	foreach (VVOptions::getForbiddenPlayers() as $pid) if(isset($p[$pid])) unset($p[$pid]);

	return $p;
}

function vvfAllowedPlayersScripts($vvfAllowedPlayers){
?>
<script type="text/javascript">
var videoPlayer,vvfAllowedPlayers=<?php echo json_encode($allowedPlayers);?>;

function vvfAllowedPlayersInit(){
	// Ensure selected player is supported, and hide unsupported players
	for(e in vvfAllowedPlayers){
		if((e=='flashFlv' && !wl.flashVersion()) ||
			(e=='flashF4v' && !(wl.flashVersion()>9)) ||
			(e=='html5Webm' && !wl.html5Video('webm')) ||
			(e=='html5Ogv' && !wl.html5Video('ogv'))){
				$('bt_'+e).style.display='none';
				vvfAllowedPlayers[e]=0
		}
		else if(!defaultPlayer) defaultPlayer=e;
	}

	btActivate('bt_'+videoPlayer);

	// If selected player is not supported by browser, switch to 1st supported one
	if(!vvfAllowedPlayers[videoPlayer] && defaultPlayer) return $('bt_'+defaultPlayer).click();
}
</script>
<?php
}

/**
 * @desc return player icon name (without path and extension)
 *
 * @param string $vp: video player name (qt, wmp, vlc,...)
 * @return string
 */
function playerIcon($vp){return ($vp=='no')?'noPlayer':(($vp=='flashWii')?'flash':$vp);}

/**
 * @desc Preencode a video in Flash FLV/VP6/MP3 (deprecated)
 *
 * @param unknown_type $completeFilename
 * @param unknown_type $extraData
 * @param unknown_type $width
 * @param unknown_type $height
 */
function viewVideoPreencodeVideo($completeFilename,$extraData,$width,$height){
	// Quicktime
	if($extraData=='qt' && !isEncoding($completeFilename,'mp4') && !isEncoded($completeFilename,'mp4')){
		// Set output size
		$maxWidth=cfRGetVar('qtOutputSize');
		if($maxWidth=='original') {$maxWidth=$width; $maxHeight=$height;}
		else{$maxHeight=$maxWidth/1.5;}

		list($w,$h)=efGetOutputImageDimensions($width,$height,$maxWidth,$maxHeight); $w+=($w%2); $h+=($h%2);
		$w=16*floor($w/16);$h=16*floor($h/16);

		// Set quality settings
		switch (cfRGetVar('qtOutputQuality')) {
			case 'high':$vbr=600; $af=44100; $ac=2; $abr=128; break;
			case 'low':$vbr=180; $af=11025; $ac=1; $abr=32; break;
			case 'medium':
			default:$vbr=300; $af=11025; $ac=1; $abr=32; break;
		}

		// Rencode video to Quicktime mp4/h264/AAC

		/* $cl='-i "'.$completeFilename.'" -acodec aac -ac '.$ac.' -ar '.$af.' -ab '.$abr.'k -s '.$w.'x'.$h;
		$cl.=' -vcodec h264 -b '.$vbr.'k -flags +loop -cmp +chroma -partitions +parti4x4+partp8x8+partb8x8 -me umh -subq 5';
		$cl.=' -trellis 1 -refs 1 -coder 0 -me_range 16 -g '.$vbr.' -keyint_min 25 -sc_threshold 40 -i_qfactor 0.71';
		$cl.=' -bt '.$vbr.'k -maxrate '.$vbr.'k -bufsize 300k -qcomp 0.6 -qmin 15 -qmax 51 -qdiff 4 -level 30';
		$cl.=' -aspect '.$w.':'.$h.' "'.reencodedCompleteFilename($completeFilename,'mp4').'"'; */

		// New ffmpeg version
		$cl='-i "'.$completeFilename.'" -acodec libfaac -ac '.$ac.' -ar '.$af.' -ab '.$abr.'k -s '.$w.'x'.$h;
		$cl.=' -vcodec libx264 -b '.$vbr.'k -flags +loop -cmp +chroma -partitions +parti4x4+partp8x8+partb8x8 -subq 5';
		$cl.=' -trellis 1 -refs 1 -coder 0 -me_range 16 -g '.$vbr.' -keyint_min 25 -sc_threshold 40 -i_qfactor 0.71';
		$cl.=' -bt '.$vbr.'k -maxrate '.$vbr.'k -bufsize 300k -qcomp 0.6 -qmin 15 -qmax 51 -qdiff 4 -level 30';
		$cl.=' -aspect '.$w.':'.$h.' "'.reencodedCompleteFilename($completeFilename,'mp4').'"';

		cfFfmpegLog($cl,'iPhone/iPad video pre-encoding (deprecated)');

		$pid=cfExecFork(cfFfmpegExe(),'SW_HIDE',$cl,1);
		if($pid) setEncoding($completeFilename,'mp4',$pid);
		echo cfAsyncXMLJSaction('monitorEncoding()');
	}
	// Flash
	if($extraData=='flv' && !isEncoding($completeFilename,'flv') && !isEncoded($completeFilename,'flv')){
		// Set output size
		$maxWidth=cfRGetVar('flashOutputSize');
		if($maxWidth=='original') {$maxWidth=$width; $maxHeight=$height;}
		else{$maxHeight=$maxWidth/1.5;}

		list($w,$h)=efGetOutputImageDimensions($width,$height,$maxWidth,$maxHeight); $w+=($w%2); $h+=($h%2);

		// Set quality settings

		// Compute audio/video bitrate
		switch (cfRGetVar('flashOutputSize')) {
			case "original":
			case 1280:
			case 900:
				if($w>640) $br=$w;
				elseif($w>320) $br=100;
				else $br=25;
				break;
			case 640:$br=400;break;
			case 400:$br=300;break;
			case 320:$br=100;break;
			case 160:$br=25;	break;
			default:$br=100;break;
		}
		switch (cfRGetVar('flashOutputQuality')) {
			case 'original':
				$outputQuality='-sameq';$vbr=8*cfFileSize($completeFilename)/(1024*vvfVideoDuration($completeFilename));
				$af=44100; $abr=64;
				break;
			case 'high':
				$outputQuality='-b '.($vbr=3*$br).'k';
				$af=22050; $abr=64;
				break;
			case 'medium':
				$outputQuality='-b '.($vbr=2*$br).'k';
				$af=11025; $abr=32;
				break;
			default:
				$outputQuality='-b '.($vbr=$br).'k';
				$af=11025; $abr=32;
				break;
		}
		// FFMPEG command line for creating preencoded FLV video
		$cl='-i "'.$completeFilename.'" '.$outputQuality.' -acodec mp3 -ar '.$af.' -ab '.$abr.'k -ac 1 -f flv -s '.$w.'x'.$h.' -y "'.reencodedCompleteFilename($completeFilename,'flv').'"';
		cfFfmpegLog($cl,'Flash video pre-encoding');

		$pid=cfExecFork(cfFfmpegExe(),'SW_HIDE',$cl,1);

		if($pid) setEncoding($completeFilename,'flv',$pid);
		echo cfAsyncXMLJSaction('monitorEncoding()');
	}
}

/**
 * @desc return video length using ffmpeg('s stderr)
 *
 * @param string $completeFilename : path and filename of video
 * @return integer : video length or 0 if not found
 */
function vvfVideoDuration($completeFilename){
	if(isset($_SESSION) && cfRGetVar('videosLengthArray',$completeFilename)!==false) return cfRGetVar('videosLengthArray',$completeFilename);
	if(!@file_exists($completeFilename)) return 0;

	if(cfCmpLeft($completeFilename,'webcam:')) return 9999;

	$l=0;

	// Get ffmpeg stdout to get duration.
	// Note / workaround: stop getting output when a chr(10) is detected at end of streaming buffer (sometime hangs...)
	$streamOptions=array();
	//if(WEnv::host()->isWindows()) $streamOptions=array('haltOnSeq'=>chr(10));
	$cl='"'.cfFfmpegExe().'" -i "'.$completeFilename.'"';
	$stderr=cfStreamProc($cl,array('stderr'=>'return'),$streamOptions);

	if(strpos($stderr, 'Duration:')){
		$stderr=explode(':',substr(ltrim(substr($stderr,strpos($stderr, 'Duration:')+9)),0,8));
		if(isset($stderr[2])) $l = 3600*$stderr[0]+60*$stderr[1]+$stderr[2];
	}

	isset($_SESSION) && cfRSetVar('videosLengthArray',$completeFilename, $l);
	return $l;
}

/**
 * @desc get video width an height using ffmpeg
 *
 * @param string $completeFilename : path and filename of video
 * @param boolean $$returnDefaultOnError: true to return 320x240 if size couldn't be computed
 * @return array : width, height
 */
function vvfVideoDimensions($completeFilename,$returnDefaultOnError=true){
	$default=array(320,240);

	$isStream=strpos($completeFilename,'://');

	// Webcam
	if(cfCmpLeft($completeFilename,'webcam:')){
		require_once(INCLUDE_DIR.'webcamFunctions.php');
		$videoId=substr($completeFilename,7);
		if(strpos($videoId,':')) $videoId=substr($videoId,0,strpos($videoId,':'));
		list($w,$h)=$l=webcamGetResolution($videoId);
		if((!$w ||!$h) && $returnDefaultOnError) $l=$default;
		return $l;
	}

	// Check that file exists
	if(!$isStream && !file_exists($completeFilename)) return 0;

	// Check cached value
	if(isset($_SESSION) && cfRGetVar('videosDimensionsArray',$completeFilename)!==false) return cfRGetVar('videosDimensionsArray',$completeFilename);

	// Get video dimensions by getting dimensions of a generated thumbnail... classe... pas classe :)
	if(file_exists(cfAppTempDir().'/_temp_1.jpg')) @unlink(cfAppTempDir().'/_temp_1.jpg');
	$cl='-ss 0 -i "'.$completeFilename.'" -vframes 1 -r 1 -an "'.cfAppTempDir().'/_temp_%d.jpg"';
	cfFfmpegLog($cl,'Get video width & height');

	// Generate image from video
	cfExecNoFork(cfFfmpegExe(),'SW_HIDE',$cl);

	// And get image dimensions
	if(file_exists(cfAppTempDir().'/_temp_1.jpg')) {
		list($width, $height, $type, $attr)=@getimagesize(cfAppTempDir().'/_temp_1.jpg');
		@unlink(cfAppTempDir().'/_temp_1.jpg');
		$l=array($width,$height);
	}
	elseif($returnDefaultOnError) $l=$default;
	else $l=array(0,0);

	if(isset($_SESSION)) cfRSetVar('videosDimensionsArray',$completeFilename, $l); // cache result
	return $l;
}

/**
 * @desc Return complete filename of cached reencoded video
 *
 * @param string $completeFilename: source video filename
 * @param string $format: reencoded file format ('flv' | 'mp4')
 * @param integer $size: reencoded max width
 * @param string $quality: reencoded quality ('original' | 'high' | 'medium' | 'low')
 * @return string
 */
function reencodedCompleteFilename($completeFilename,$format,$size=0,$quality=0){
	if($format!='flv' && $format!='mp4') return false;

	if(cfRGetVar('path')!='computerRoot') $path=cfRGetVar('resourceDataDir').substr(cfRGetVar('efCurrentDirectory'), strlen(cfCleanPathName(cfRGetVar('path'))));
	else $path=cfRGetVar('resourceDataDir').'/'.str_replace(':/','/',cfRGetVar('efCurrentDirectory'));

	// Create path if not existing
	if(!is_dir($path)) cfMkdir($path);

	if(!$size)		$size=(($format=='mp4')?cfRGetVar('qtOutputSize'):cfRGetVar('flashOutputSize'));
	if(!$quality)	$quality=(($format=='mp4')?cfRGetVar('qtOutputQuality'):cfRGetVar('flashOutputQuality'));
	if($format=='flv') return $path.'/'.$size.$quality.basename($completeFilename).'.flv';
	if($format=='mp4') return $path.'/'.$size.$quality.basename($completeFilename).'.mp4';
}

/**
 * @desc return true is $completeFilename is beeing reencoded
 *
 * @param string $completeFilename: source file to be encoded
 * @param string $format: encoding format ('flv' | 'mp4')
 * @return mixed: false if not encoding, ffmpeg pid if encoding
 */
function isEncoding($completeFilename,$format){
	$encodingVideos=cfMGetVar('encodingVideos');
	if(!isset($encodingVideos[$format][$completeFilename])) return false;
	if($procs=cfProcList())
		foreach ($procs as $proc) if($proc['pid']==$encodingVideos[$format][$completeFilename]) return $proc['pid'];

	// Clean encoding videos list
	unset($encodingVideos[$format][$completeFilename]);
	cfMSetVar('encodingVideos',$encodingVideos);
}

/**
 * @desc Set a file as beeing encoded
 *
 * @param string $completeFilename: encoding filename
 * @param string $format: encoding format ('flv' | 'mp4')
 * @param integer $pid: ffmpeg process id, set to 0 to remove from list
 */
function setEncoding($completeFilename,$format,$pid){
	$encodingVideos=cfMGetVar('encodingVideos');
	if($pid) $encodingVideos[$format][$completeFilename]=$pid; else unset($encodingVideos[$format][$completeFilename]);
	cfMSetVar('encodingVideos',$encodingVideos);
}

/**
 * @desc return true if a file has been completly reencoded
 *
 * @param string $completeFilename: source video filename
 * @param string $format: reencoded file format ('flv' | 'mp4')
 * @param integer $size: reencoded max width
 * @param string $quality: reencoded quality ('original' | 'high' | 'medium' | 'low')
 * @return boolean result
 */
function isEncoded($completeFilename,$format,$size=0,$quality=0){
	$encodedFilename=reencodedCompleteFilename($completeFilename,$format,$size,$quality);
	if(!file_exists($encodedFilename)) return false;
	if(filesize($encodedFilename)>0 && !isEncoding($completeFilename,$format)) return true;
	elseif (filesize($encodedFilename)==0) @unlink($encodedFilename);
	return false;
}

/**
 * @desc return encoding estimated progress
 *
 * @param string $completeFilename: source video filename
 * @param string $format: reencoded file format ('flv' | 'mp4')
 * @param integer $size: reencoded max width
 * @param string $quality: reencoded quality ('original' | 'high' | 'medium' | 'low')
 * @return integer progress (0-100)
 */
function encodingProgress($completeFilename,$format,$size=0,$quality=0){
	if(!$size)		$size=(($format=='mp4')?cfRGetVar('qtOutputSize'):cfRGetVar('flashOutputSize'));
	if(!$quality)	$quality=(($format=='mp4')?cfRGetVar('qtOutputQuality'):cfRGetVar('flashOutputQuality'));

	$encodedSize=@filesize(reencodedCompleteFilename($completeFilename,$format,$size,$quality));
	if(!($videoDuration=vvfVideoDuration($completeFilename))) return 0;

	if($format=='mp4'){
		switch ($quality) {
			case 'high':$vbr=600; $af=44100; $ac=2; $abr=128; break;
			case 'low':$vbr=180; $af=11025; $ac=1; $abr=32; break;
			case 'medium':
			default:$vbr=300; $af=11025; $ac=1; $abr=32; break;
		}
	}
	if($format=='flv'){
		list($w,$h)=vvfVideoDimensions($completeFilename);
		switch($size) {
			case "original":
			case 1280:
			case 900:
				if($w>640) $br=$w;
				elseif($w>320) $br=100;
				else $br=25;
				break;
			case 640:$br=400;break;
			case 400:$br=300;break;
			case 320:$br=100;break;
			case 160:$br=25;	break;
			default:$br=100;break;
		}
		switch ($quality) {
			case 'original':
				$af=44100; $abr=64;
				$vbr=3*(($size=='original')?$w:$size); // Estimated vbr
				break;
			case 'high':
				$vbr=3*$br;
				$af=22050; $abr=64;
				break;
			case 'medium':
				$vbr=2*$br;
				$af=11025; $abr=32;
				break;
			default:
				$vbr=$br;
				$af=11025; $abr=32;
				break;
		}
//cfDbg($size.'/'.$w.' - '.$vbr.' '.$abr.' => '.($videoDuration*($vbr+$abr)/8).' duration='.$videoDuration);
	}
	return max(0,min(100,100*$encodedSize*(8/(1024*($vbr+$abr)))/$videoDuration));
}

/**
 * @desc insert HTML5 player for reencoded WebM and Ogg Video files
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : output video width
 * @param integer $height : output video height
 * @param array options: autoplay=0/1, videoPlayer=html5Webm|html5Ogv
 */
function insertVideoPlayerHtml5Reenc($completeFilename, $videoWidth, $videoHeight, $options){
	// Codec
	if($options['videoPlayer']=='html5Webm') $codec='webm';
	else if($options['videoPlayer']=='html5Ogv') $codec='ogv';
	else if($options['videoPlayer']=='html5Auto') $codec='auto';

	$autoplay=(isset($options['autoplay']))?$options['autoplay']:1;

	// Compute name of cache reencoded file
	$reencodedCompleteFilename=reencodedCompleteFilename($completeFilename,$options['videoPlayer']);

	// Get video duration
	$videoDuration=vvfVideoDuration($completeFilename);

	// Store video file name in session variable, for future use in video.php
	$params=array(
		'q'=>cfRGetVar('flashOutputQuality'),
		'w'=>$videoWidth,
		'h'=>$videoHeight,
		'format'=>$options['videoPlayer'],
		'offset'=>cfRGetVar('flashOutputOffset')
	);

	// Video src
	$src='';
	if($codec=='ogv' || $codec=='auto'){
		$params['format']='html5Ogv';
		$src.='<source src="'.cfExtVideo($completeFilename,$params,'relative').'" type=\'video/ogg; codecs="theora, vorbis"\'>'."\n";
	}

	if($codec=='webm' || $codec=='auto'){
		$params['format']='html5Webm';
		$src.='<source src="'.cfExtVideo($completeFilename,$params,'relative').'" type=\'video/webm; codecs="vp8, vorbis"\'>'."\n";
	}

	if(cfFfmpegVersion()=='GPL' && $codec=='h264' || $codec=='auto'){
		$params['format']='html5H264';
		$src.='<source src="'.cfExtVideo($completeFilename,$params,'relative').'" type=\'video/mp4; codecs="avc1.42E01E, mp4a.40.2"\'>'."\n";
	}

	//$src=cfExtVideo($completeFilename,$params,'relative');
	//$src='/testPlayers.php?webm'

	// Subtitles (unsupported yet)
	//if(cfHGetVar('showSubtitles',true)) cfRSetVar('singleVideoSubtitles',$completeFilename); else cfRSetVar('singleVideoSubtitles',false);

	/**
	 * Insert HTML5 video tag
	 */
	?>
<div id="playerVideoDiv" style="position:relative">
<video id="playerVideo"
	width="<?php echo $videoWidth; ?>"
	height="<?php echo $videoHeight; ?>"
	<?php if($autoplay) echo 'autoplay="true"';?>
	preload="auto"
	onerror="W.html5Error()"
	onplay="html5StartPlaying()"
	ondblclick="html5Play();setFullscreen()"
	oncanplay="W.html5CanPlay=1"
	onended=""
	<?php
	if(cfIsMobile())
		echo 'onclick="html5Play(1);$(this).e.toggleFullscreen(1)"';
	else
		echo 'onmousemove="htmlButtonsFading=2"';
?>
	align="middle" style="background:#000;color:white;z-index:0 !important"
>
<?php echo $src; ?>
HTML5 Video not supported
</video>
<?php

if(cfIsMobile()){
	// Error indicator
	echo outImageIcon('alertBig','id="html5Error"','position:absolute;right:10px;top:0;display:none;z-index:10');

	// Play button
	echo mfButton(false,'javascript:html5Play(1)',outIcon('html5AP/play'),'id="html5PlayBt" style="position:absolute;top:0px;left:5px;cursor:pointer;z-index:10"');

	// Pause button
	echo mfButton(false,'javascript:html5Play(0)',outIcon('html5AP/pause'),'id="html5PauseBt" style="position:absolute;top:0px;left:5px;cursor:pointer;z-index:10;display:none"');

	// Toggle fullscreen button
	echo mfButton(false,'javascript:$(\'playerVideo\').e.toggleFullscreen()',outIcon('fscreen32'),'style="position:absolute;top:0px;right:5px;cursor:pointer;z-index:10"');
}
else{
	// Loading indicator
	echo outImageIcon('loadingBlack','id="html5Loading"','position:absolute;left:50%;top:50%;margin:-8px 0 0 -8px;z-index:10');

	// Error indicator
	echo outImageIcon('alertBig','id="html5Error"','position:absolute;left:50%;top:50%;margin:-24px 0 0 -24px;display:none;z-index:10');

	// Play button
	echo outImage(outIcon('html5PlayBt'),'javascript:html5Play(1)','id="html5PlayBt"','position:absolute;bottom:10px;display:none;cursor:pointer;z-index:10;'.outCssOpacity(60));

	// Pause button
	echo outImage(outIcon('html5PauseBt'),'javascript:html5Play(0)','id="html5PauseBt"','position:absolute;bottom:10px;display:none;cursor:pointer;z-index:10;'.outCssOpacity(60));
}
?>
</div>
<script type="text/javascript">
var htmlButtonsFading=2,html5Playing=0,html5Paused=<?php if($autoplay) echo 0; else echo 1; ?>,html5Offset=<?php echo (int)cfRGetVar('flashOutputOffset');?>,html5CanPlay=false;
function html5Error(){
	if(windowReloading) return;
	setInterval(function(){$('html5Error').style.display=($('html5Error').style.display=='none')?'':'none';},500);
	if(dgi('html5Loading')) $('html5Loading').style.display='none';
}
function html5StartPlaying(){
	if(dgi('html5Loading')) $('html5Loading').style.display='none';
	$('html5PauseBt').style.display='';
	html5Playing=1;
}
function html5Play(play){
	if(play===undefined) play=1-html5Playing;
	html5Paused=1-play;
	html5Playing=play;
	if(play) $('playerVideo').play(); else $('playerVideo').pause();
	$('html5PlayBt').style.display=(play)?'none':'';
	$('html5PauseBt').style.display=(play)?'':'none';
}
// buttons fading
setInterval(function(){
	if(isMobile) return;
	var nf,f=htmlButtonsFading;
	nf=Math.max(0,f-0.025);
	if(f>0 && (f<0.6 || f==2)){
		$('html5PauseBt').e.setAlpha(Math.min(0.6,nf))
		$('html5PlayBt').e.setAlpha(Math.min(0.6,nf))
	}
	htmlButtonsFading=nf;
},50);
// Progress update
setInterval(function(){if(W.setTime) setTime(html5Offset+$('playerVideo').currentTime)},500)
// Play not-starting workaround: if video should be playing bug is not, re-display play button so user can click on it.
setInterval(function(){if(!html5Paused && !isMobile) html5Play(1)} ,100)

// Fullscreen cancel on esc
wl.assignKey(27,function(){if(html5Fullscreen) setFullscreen(0)});
</script>
<?php

	/**
	 * Progress bar (except for preencoding)
	 */
	echo '<div id="extraControls" style="width:100%;margin:0;position:absolute;bottom:0;left:0"><center>';
	if(cfIsWII()) displayProgressBar($completeFilename,$videoDuration,cfRGetVar('flashOutputOffset'), 700);
	else displayProgressBar($completeFilename,$videoDuration,cfRGetVar('flashOutputOffset'), '90%',$videoWidth,$videoHeight);
	echo '</center><br>&nbsp;</div>';
}

/**
 * @desc insert flash FLV player for reencoded F4V or FLV files and start playing (if autostart is set to true) reencoded file
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : output video width
 * @param integer $height : output video height
 * @param array options:
 * 		- 'debug'=> (flash player only)
 * 		- 'chunkLength'=>chunk length in seconds (flash player only)
 * 		- 'videoPlayer'=>'flashF4v' or 'flashFlv';
 * 		- 'streaming'=>true if video is a live streaming feed (web tv)
 */
function insertVideoPlayerFlashReenc($completeFilename, $videoWidth, $videoHeight, $options){
	if(isset($options['debug'])) $debug='&amp;debug=1'; else $debug='';
//$debug='&amp;debug=1'; if(0) cfDbg();

	if(isset($options['chunkLength'])) $chunkLength=$options['chunkLength']; else $chunkLength=(int)cfHGetVar('flashChunkLength');
	if(cfIsMobile() || cfBGetVar('name')=='Wii') $chunkLength=270;
	if(@$options['streaming']) $chunkLength=99999; // No chunking for streaming

	if(@$options['videoPlayer']=='flashF4v') $codec='h264';
	else if(@$options['videoPlayer']=='flashFlv') $codec='vp6';
	else $codec='vp6';

	$autoplay=1;

	// Compute name of cache reencoded file
	$reencodedCompleteFilename=reencodedCompleteFilename($completeFilename,'flv');

	// File already preencoded
	if(!cfRGetVar('flashStream') && file_exists($reencodedCompleteFilename)) {
		// Standalone player: redirect browser to flash player (Wii)
		if(cfBGetVar('standaloneMediaPlayer')) {
			$playerAdress=cfHostName().'/js/vplayer.swf?file='.cfExtVideo($reencodedCompleteFilename).'&w='.$videoWidth.'&h='.$videoHeight.'&offset='.cfRGetVar('flashOutputOffset').'&autoplay='.(($autoplay)?"1":"0").'&noCallback=1&buttonSize=50&buffer='.((cfRGetVar('bufferTime'))?cfRGetVar('bufferTime'):2).'&ping='.max(cfGGetVar('clientPingInterval').'&ffmpegVersion='.cfFfmpegVersion().$debug,10);
			die('<script type="text/javascript">location.replace("'.cfHostName().$playerAdress.'")</script>');
		}
		// Embeded player: insert regular flash player
		else return insertVideoPlayerFlash($reencodedCompleteFilename,$videoWidth, $videoHeight,$autoplay);
	}
	// Get video duration (or set -1 for streaming)
	if(@$options['streaming']) $videoDuration=-1; else $videoDuration=vvfVideoDuration($completeFilename);

	// Store video file name in session variable, for future use in video.php
	$params=array('q'=>cfRGetVar('flashOutputQuality'),'w'=>$videoWidth,'h'=>$videoHeight,'offset'=>cfRGetVar('flashOutputOffset'),'chunkLength'=>$chunkLength); if($debug) $params['debug']=1;
	cfRSetVar('singleVideo', cfExtVideo($completeFilename,$params,'relative'));
	cfRSetVar('singleVideoWidth', $videoWidth);
	cfRSetVar('singleVideoHeight',$videoHeight);

	// Subtitles
	if(cfHGetVar('showSubtitles',true))
		cfRSetVar('singleVideoSubtitles',$completeFilename);
	else
		cfRSetVar('singleVideoSubtitles',false);

	// Set SWF source to *singleVideo*.xml (so /ext/video.php outputs an XML file setting a link to actual FLV file)
	$src=cfExtVideo('*singleVideo*.xml',array('chunkLength'=>$chunkLength));

	/**
	 * Insert Embeded Flash player
	 */
	if((cfRGetVar('flashStream') && !cfIsWII() && !cfBGetVar('standaloneMediaPlayer')) || cfRGetVar('forceInlinePlayer')){
		$movieURL=cfHostName().'/js/vplayer.swf?file='.$src.
			'&amp;w='.$videoWidth.'&amp;h='.$videoHeight.
			'&amp;offset='.cfRGetVar('flashOutputOffset').
			'&amp;autoplay='.(($autoplay)?"1":"0").
			'&amp;buffer='.cfHGetVar('videoBufferTime',3).
			'&amp;browser='.cfGetBrowser().
			'&amp;videoDurationExt='.$videoDuration.
			'&amp;chunkLength='.(int)$chunkLength.
			'&amp;ffmpegVersion='.cfFfmpegVersion().
			'&amp;codec='.$codec.
			'&amp;isMobile='.(int)cfIsMobile().
			$debug;

?>
<div id="playerVideoDiv">
<object id="playerVideo" classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" codebase="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0" width="<?php echo $videoWidth; ?>" height="<?php echo $videoHeight; ?>" align="middle">
	<param name="allowScriptAccess" value="sameDomain" />
	<param name="allowFullScreen" value="true" />
	<param name="fullScreenRect" value="true" />
	<param name="movie" value="<?php echo $movieURL;?>" />
	<param name="quality" value="high" />
	<param name="bgcolor" value="#000000" />
	<embed name="playerVideo" id="playerVideoEmbed"
		src="<?php echo $movieURL;?>"
		swLiveConnect="true"
		quality="high"
		width="<?php echo $videoWidth; ?>"
		height="<?php echo $videoHeight; ?>"
		allowFullScreen="true"
		fullScreenRect="true"
		align="middle"
		allowScriptAccess="sameDomain"
		type="application/x-shockwave-flash"
		pluginspage="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://www.macromedia.com/go/getflashplayer"
	/>
</object>
</div>
<script type="text/javascript" language="Javascript">
function playerVideo_DoFSCommand(commande, args) {if(W.processFlashCommand) processFlashCommand(commande,args)}
function flashCommand(commande,args) {if(W.processFlashCommand) processFlashCommand(commande,args);}
</script>
<script language="VBScript">
On Error Resume Next
Sub playerVideo_FSCommand(ByVal command, ByVal args)
	Call playerVideo_DoFSCommand(command, args)
End Sub
</script>
<?php
	}

	/**
	 * Display still image for preencoding interface or time-jumping preview (standalone flash player)
	 */
	else{
		if(cfRGetVar('flashStream')) $imageOffset=cfRGetVar('flashOutputOffset'); else $imageOffset=3;
		// Generate still image
		if(file_exists(cfAppResourceDir().'/_temp_preview1.jpg')) @unlink(cfAppResourceDir().'/_temp_preview1.jpg');
		list($pw,$ph)=efGetOutputImageDimensions($videoWidth,$videoHeight,cfGGetVar('userScreenWidth'),cfGGetVar('userScreenWidth')/1.5,true);

		$cl='-ss '.$imageOffset.' -i "'.$completeFilename.'" -vframes 1 -an -s '.($pw+$pw%2).'x'.($ph+$ph%2).' "'.cfAppResourceDir().'/_temp_preview%0d.jpg"';
		cfFfmpegLog($cl,'Extract video frame');
		cfFfmpegExec($cl);


		/**
		 * Preencoding interface
		 */
		if (!cfRGetVar('flashStream')){
			// Display image with link
			echo '<div style="border:0px;width:'.$pw.'px;height:'.$ph.'px;position:relative" id="playerVideo">';
			echo outImage(cfExtImage(cfAppResourceDir().'/_temp_preview1.jpg',$pw,$ph,false),false,false,'width:100%;height:100%;position:absolute;left:0');
			echo '<center><div id="preencodeStuff" style="position:relative;top:'.($ph/2-40).'px">';
			$isEncoding=isEncoding($completeFilename,'flv');

			// Encode video button
			echo '<a class="buttonBig" id="encodeVideoBt" href="javascript:encodeVideo(1)"'.(($isEncoding)?' style="display:none"':'').'>'.outImage(outIcon('gearsB')).'<br>'.cfCaption('genEncode').'</a>';

			// Stop encoding button
			echo '<a class="buttonBig" id="encodeVideoCancelBt" href="javascript:encodeVideo(0)"'.(($isEncoding)?'':' style="display:none"').'>'.outImage(outIcon('cancelBig')).'<br>'.cfCaption('genCancel').'</a>';

			// Encoding progressbar
			$progress=(($isEncoding)?encodingProgress($completeFilename,'mp4'):0);
			echo outProgressBar(($progress/100),'90%',floor($progress).' %','progressBar',false,false,'margin-top:1em'.(($isEncoding)?'':';visibility:hidden'));
			echo '</div></center></div>';
			echo '<div id="loadingIconDiv" style="visibility:hidden">'.outThemeImage(outIcon('loading'),false,'id="loadingIcon"',(($isEncoding)?'':'visibility:hidden')).'</div>';
?>
<script type="text/javascript">
D.getElementsByTagName('td')[1].lastChild.innerHTML+=dgi('loadingIconDiv').innerHTML;dgi('loadingIconDiv').innerHTML='';
function encodeVideo(act){
	if(act){
		dgi('loadingIcon').style.visibility='';
		dgi('encodeVideoBt').style.display='none';
		dgi('progressBar').style.visibility='';
		dgi('encodeVideoCancelBt').style.display='';
		submitFrm('encodeVideo','flv');
	}
	else{
		dgi('encodeVideoCancelBt').style.visibility='hidden';
		dgi('progressBar').style.visibility='hidden';
		submitFrm('encodeVideoCancel','flv');
	}
}
function doneCancel(){
	dgi('loadingIcon').style.visibility='hidden';
	dgi('encodeVideoBt').style.display='';
	dgi('encodeVideoCancelBt').style.display='none';
	dgi('encodeVideoCancelBt').style.visibility='';
	monitorEncoding(-1);
}

function monitorEncoding(perc){
	W.setTimeout('submitFrm("monitorEncoding","flv")',1000);
	if(perc!==undefined) {
		if(perc<0) perc=0;
		progressSetPerc('progressBar',perc);
		progressSetText('progressBar',Math.floor(perc)+'%');
	}
}
<?php if($isEncoding) echo 'monitorEncoding()';?>
</script>
<?php
		}

		/**
		 * time-jumping preview interface (for standalone flash player)
		 */
		else{
			// Display image with link
			$playerAdress=cfHostName().'/js/vplayer.swf?file='.$src.'&w='.$videoWidth.'&h='.$videoHeight.'&offset='.cfRGetVar('flashOutputOffset').'&autoplay='.(($autoplay)?"1":"0").'&noCallback=1&buttonSize=50&buffer='.((cfRGetVar('bufferTime'))?cfRGetVar('bufferTime'):2).'&ping='.max(cfGGetVar('clientPingInterval'),1).$debug.'&chunkLength='.$chunkLength.'&browser='.cfBGetVar('name');
			echo '<a href="'.$playerAdress.'" target="_top" id="playerVideo" style="width:'.$pw.'px;height:'.$ph.'px"><img id="playerVideoImage" src="'.cfExtImage(cfAppResourceDir().'/_temp_preview1.jpg',$pw,$ph,false).'"></a>';
			echo '<script type="text/javascript">stretchablePlayer=1;stretchPlayer=1;</script>';
		}
	}

	/**
	 * Progress bar (except for preencoding)
	 */
	if(cfRGetVar('flashStream')){
		echo '<div id="extraControls" style="width:100%;margin:0;position:absolute;bottom:0;left:0"><center>';
		if(cfIsWII()) displayProgressBar($completeFilename,$videoDuration,cfRGetVar('flashOutputOffset'), 700);
		else displayProgressBar($completeFilename,$videoDuration,cfRGetVar('flashOutputOffset'), '90%', $videoWidth, $videoHeight);
		echo '</center><br>&nbsp;</div>';
	}
}

/**
 * @desc insert flash FLV player, and start playing (if autostart is set to true)
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : output FLV video width
 * @param integer $height : output FLV video height
 * @param string $autoplay : true : start playing when loaded, false don't
 */
function insertVideoPlayerFlash($completeFilename, $videoWidth, $videoHeight, $autoplay=1){
	// Set SWF source to *singleVideo*.xml (so /ext/video.php outputs an XML file setting a link to actual FLV file)
	$sourceLink=cfExtVideo($completeFilename,'','relative');

	// Insert Flash player
	?>
<object id="playerVideo" classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" codebase="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0" width="<?php echo $videoWidth; ?>" height="<?php echo $videoHeight; ?>" align="middle">
	<param name="allowScriptAccess" value="sameDomain" />
	<param name="allowFullScreen" value="true" />
	<param name="fullScreenRect" value="true" />
	<param name="movie" value="<?php echo cfHostName();?>/js/vplayer.swf?file=<?php echo $sourceLink; ?>&amp;w=<?php echo $videoWidth; ?>&amp;h=<?php echo $videoHeight; ?>&amp;autoplay=<?php if($autoplay) echo "1"; else echo "0"; ?>&amp;buffer=<?php echo ((cfRGetVar('bufferTime'))?cfRGetVar('bufferTime'):3); ?>&amp;browser=<?php echo cfGetBrowser();?>" />
	<param name="quality" value="high" />
	<param name="bgcolor" value="#000000" />
	<embed name="playerVideo" id="playerVideoEmbed"
		src="<?php echo cfHostName();?>/js/vplayer.swf?file=<?php echo $sourceLink; ?>&amp;w=<?php echo $videoWidth; ?>&amp;h=<?php echo $videoHeight; ?>&amp;autoplay=<?php if($autoplay) echo "1"; else echo "0"; ?>&amp;buffer=<?php echo ((cfRGetVar('bufferTime'))?cfRGetVar('bufferTime'):3); ?>&amp;browser=<?php echo cfGetBrowser();?>"
		swLiveConnect="true"
		quality="high"
		width="<?php echo $videoWidth; ?>"
		height="<?php echo $videoHeight; ?>"
		align="middle"
		allowScriptAccess="sameDomain"
		allowFullScreen="true"
		fullScreenRect="true"
		type="application/x-shockwave-flash"
		pluginspage="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://www.macromedia.com/go/getflashplayer"
	/>
</object>
<script type="text/javascript" language="Javascript">
function playerVideo_DoFSCommand(commande, args) {}
function flashCommand(commande,args) {}
</script>
<script language="VBScript">
On Error Resume Next
Sub playerVideo_FSCommand(ByVal command, ByVal args)
End Sub
</script>

<?php
}


/**
 * @desc insert Quicktime player, and start playing (if autostart is set to true)
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : source video width
 * @param integer $height : source video height
 * @param string $autoplay : true : start playing when loaded, false don't
 */
function insertVideoPlayerQT($completeFilename, $videoWidth, $videoHeight, $autoplay=1){
	$sourceLink=cfExtVideo($completeFilename,'','absolute');
?>
<object id="playerVideo" classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B"
width="<?php echo $videoWidth;?>" height="<?php echo ($videoHeight+15);?>"
codebase="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://www.apple.com/qtactivex/qtplugin.cab#version=6,0,2,0"
align="middle" >
<param name="src" value="<?php echo $sourceLink;?>" />
<param name="autoplay" value="<?php echo $autoplay;?>" />
<embed name="playerVideo" id="playerVideoEmbed" src="<?php echo $sourceLink;?>" width="<?php echo $videoWidth;?>" height="<?php echo ($videoHeight+15);?>"
pluginspage=<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://www.apple.com/quicktime/download/
align="middle" autoplay="<?php echo $autoplay;?>" bgcolor="black"></embed>
</object>
<?php
}


/**
 * @desc insert Windows Media Player, and start playing (if autostart is set to true)
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : source video width
 * @param integer $height : source video height
 * @param string $autoplay : true : start playing when loaded, false don't
 */
function insertVideoPlayerWMP($completeFilename, $videoWidth, $videoHeight, $autoplay=1){
	$sourceLink=cfExtVideo($completeFilename,'','absolute');
?>
<object id="playerVideo" name="NSPlayer" classid="CLSID:22D6F312-B0F6-11D0-94AB-0080C74C7E95" codebase="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=6,1,5,217" standby="<?php  echo cfUTF8Encode('Microsoft Windows Media Player components...');?>" type="application/x-oleobject">
<param name='AutoStart' value='<?php echo $autoplay;?>'>
<param name='filename' value='<?php echo $sourceLink;//IE parameters ?>'>
<param name='EnableContextMenu' value='1'>
<param name='ShowStatusBar' value='1'>
<param name='ShowAudioControls' value='1'>
<param name='BufferingTime' value='3'>
<param name='ShowControls' value='1'>
<param name='ShowPositionControls' value='1'>
<param name='ShowTracker' value='1'>
<param name='ShowDisplay' value='0'>
<param name='ShowTracker' value='0'>
<param name='ShowGotoBar' value='0'>
<param name='EnableTracker' value='0'>
<param name='width' value='<?php echo $videoWidth;?>'>
<param name='height' value='<?php echo $videoHeight+69;?>'>
<?php // Opera & Gecko parameters ?>
<embed name="playerVideo" id="playerVideoEmbed" type="application/x-mplayer2" src="<?php echo $sourceLink;?>" width="<?php echo $videoWidth;?>" height="<?php echo $videoHeight+69;?>" autostart="<?php echo $autoplay;?>" uiMode="none" ShowAudioControls="1" ShowPositionControls="1" ShowTracker="1" EnableContextMenu="1" ShowStatusBar="1"></embed>
</object>
<?php
}

/**
 * @desc insert DivX web Player, and start playing (if autostart is set to true)
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : source video width
 * @param integer $height : source video height
 * @param string $autoplay : true : start playing when loaded, false don't
 */
function insertVideoPlayerDivX($completeFilename, $videoWidth, $videoHeight, $autoplay=1){
	$sourceLink=cfExtVideo($completeFilename,'m3u8sa','absolute');
?>
<object classid="clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616" id="playerVideo" width="<?php echo $videoWidth;?>" height="<?php echo $videoHeight;?>" codebase="http<?php echo ((isset($_SERVER['HTTPS']))?'s':'');?>://go.divx.com/plugin/DivXBrowserPlugin.cab">
<param name="custommode" value="Stage6" />
<param name="mode" value="zero" />
<param name="src" value="<?php echo $sourceLink;?>" />
<embed id="playerVideoEmbed" type="video/divx" src="<?php echo $sourceLink;?>" custommode="Stage6" width="<?php echo $videoWidth;?>" height="<?php echo $videoHeight;?>" mode="zero"  pluginspage="http<?php echo ((isset($_SERVER['HTTPS']))?'s':'');?>://go.divx.com/plugin/download/">
</embed>
<?php
}

/**
 * @desc insert VLC Plugin Player, and start playing (if autostart is set to true)
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : source video width
 * @param integer $height : source video height
 * @param string $autoplay : true : start playing when loaded, false don't
 */
function insertVideoPlayerVLC($completeFilename, $videoWidth, $videoHeight, $autoplay=1){
	$sourceLink=cfExtVideo($completeFilename,'','absolute');
?>
<object id="playerVideo" classid="clsid:9BE31822-FDAD-461B-AD51-BE1D1C159921" codebase="http://downloads.videolan.org/pub/videolan/vlc/latest/win32/axvlc.cab" id="vlc" events="True" width="<?php echo $videoWidth;?>" height="<?php echo $videoHeight;?>">
	<param name="ShowDisplay" value="True" />
	<param name="AutoLoop" value="False" />
	<param name="AutoPlay" value="<?php echo $autoplay;?>" />
	<param name="src" value="<?php echo $sourceLink;?>" />
	<embed name="playerVideo" id="playerVideoEmbed" type="application/x-vlc-plugin" progid="VideoLAN.VLCPlugin.2" pluginspage="http://www.videolan.org" src="<?php echo $sourceLink;?>" width="<?php echo $videoWidth;?>" height="<?php echo $videoHeight;?>" autoplay="<?php echo $autoplay;?>" loop="no" hidden="no" target="" />
</object>
<?php
}

/**
 * @desc insert Fixed image and controls for reencoding / viewing iPhone formated mp4 video
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : source video width
 * @param integer $height : source video height
 * @param string $autoplay : true : start playing when loaded, false don't
 */
function insertVideoPlayerQTReenc($completeFilename, $videoWidth, $videoHeight, $autoplay=1){
	insertVideoPlayerQT($completeFilename, $videoWidth, $videoHeight, $autoplay);

	// Real-time streaming
	/*
	require_once(INCLUDE_DIR.'mobileViewFunctions.php');
	return mvfInsertVideoPlayerIPhone($completeFilename, $videoWidth, $videoHeight, $autoplay);
	*/

	/*
	// Video is reencoded
	if(isEncoded($completeFilename,'mp4')){
		return insertVideoPlayerQT($completeFilename, $videoWidth, $videoHeight, $autoplay);
	}
	// Not yet (fully) reencoded video
	else{
		// Generate still image
		if(file_exists(cfAppResourceDir().'/_temp_preview1.jpg')) @unlink(cfAppResourceDir().'/_temp_preview1.jpg');
		if(cfBGetVar('name')=='iPhone') list($pw,$ph)=efGetOutputImageDimensions($videoWidth,$videoHeight,480*0.95,228*0.95);
		else {$pw=$videoWidth;$ph=$videoHeight;}

		$cl='-i "'.$completeFilename.'" -vframes 1 -ss 3 -an -s '.($pw+$pw%2).'x'.($ph+$ph%2).' "'.cfAppResourceDir().'/_temp_preview%0d.jpg"';
		cfFfmpegExec($cl);

		// Display image with link
		echo '<div style="border:0px solid red;width:'.$pw.'px;height:'.$ph.'px;position:relative" id="playerVideo">';
		echo outImage(cfExtImage(cfAppResourceDir().'/_temp_preview1.jpg',$pw,$ph,false),false,false,'width:100%;height:100%;position:absolute;left:0');
		//echo outImage(cfExtImage(cfAppResourceDir().'/_temp_preview1.jpg',$pw,$ph,false),false,false,'width:100%;height:100%;position:absolute;left:0');
		echo '<center><div id="preencodeStuff" style="position:relative;top:'.($ph/2-40).'px">';
		$isEncoding=isEncoding($completeFilename,'mp4');

		// Encode video button
		echo '<a class="buttonBig" id="encodeVideoBt" href="javascript:encodeVideo(1)"'.(($isEncoding)?' style="display:none"':'').'>'.outImage(outIcon('gearsB')).'<br>'.cfCaption('genEncode').'</a>';

		// Stop encoding button
		echo '<a class="buttonBig" id="encodeVideoCancelBt" href="javascript:encodeVideo(0)"'.(($isEncoding)?'':' style="display:none"').'>'.outImage(outIcon('cancelBig')).'<br>'.cfCaption('genCancel').'</a>';

		// Encoding progressbar
		$progress=(($isEncoding)?encodingProgress($completeFilename,'mp4'):0);
		echo outProgressBar(($progress/100),'90%',floor($progress).' %','progressBar',false,false,'margin-top:1em'.(($isEncoding)?'':';visibility:hidden'));
		echo '</div></center></div>';
		echo '<div id="loadingIconDiv" style="visibility:hidden">'.outThemeImage(outIcon('loading'),false,'id="loadingIcon"',(($isEncoding)?'':'visibility:hidden')).'</div>';
	}
?>
<script type="text/javascript">
D.getElementsByTagName('td')[1].lastChild.innerHTML+=dgi('loadingIconDiv').innerHTML;dgi('loadingIconDiv').innerHTML='';
function encodeVideo(act){
	if(act){
		dgi('loadingIcon').style.visibility='';
		dgi('encodeVideoBt').style.display='none';
		dgi('progressBar').style.visibility='';
		dgi('encodeVideoCancelBt').style.display='';
		submitFrm('encodeVideo','qt');
	}
	else{
		dgi('encodeVideoCancelBt').style.visibility='hidden';
		dgi('progressBar').style.visibility='hidden';
		submitFrm('encodeVideoCancel','qt');
	}
}
function doneCancel(){
	dgi('loadingIcon').style.visibility='hidden';
	dgi('encodeVideoBt').style.display='';
	dgi('encodeVideoCancelBt').style.display='none';
	dgi('encodeVideoCancelBt').style.visibility='';
	monitorEncoding(-1);
}

function monitorEncoding(perc){
	window.setTimeout('submitFrm("monitorEncoding","qt")',2000);
	if(perc!==undefined) {
		if(perc<0) perc=0;
		progressSetPerc('progressBar',perc);
		progressSetText('progressBar',Math.floor(perc)+'%');
	}
}
stretchablePlayer=1;
<?php
if($isEncoding) echo 'monitorEncoding()';
?>
</script>
<?php
*/
}

/**
 * @desc insert Fixed image and send an m3u file
 *
 * @param string $completeFilename : complete file name of file to play
 * @param integer $width : source video width
 * @param integer $height : source video height
 * @param string $autoplay : true : start playing when loaded, false don't
 */
function insertVideoPlayerM3U($completeFilename, $videoWidth, $videoHeight, $autoplay=1){
	// Generate still image
	if(file_exists(cfAppResourceDir().'/_temp_preview1.jpg')) @unlink(cfAppResourceDir().'/_temp_preview1.jpg');
	if(cfBGetVar('name')=='iPhone') list($pw,$ph)=efGetOutputImageDimensions($videoWidth,$videoHeight,480*0.95,228*0.95);
	else {$pw=$videoWidth;$ph=$videoHeight;}

	$cl='-ss 3 -i "'.$completeFilename.'" -vframes 1 -an -s '.($pw+$pw%2).'x'.($ph+$ph%2).' "'.cfAppResourceDir().'/_temp_preview%0d.jpg"';
	cfFfmpegLog($cl,'Extract single video frame');
	cfFfmpegExec($cl);

	// Display image with link
	echo '<div style="border:0px solid red;width:'.$pw.'px;height:'.$ph.'px;position:relative" id="playerVideo">';
	echo outImage(cfExtImage(cfAppResourceDir().'/_temp_preview1.jpg',$pw,$ph,false),false,false,'width:100%;height:100%;position:absolute;left:0');
	echo '</div>';
	?>
<script type="text/javascript">window.setTimeout("submitFrm('sendM3U','','sync')",100)</script>
<?php
}

/**
 * @desc Insert HTML code for videoplayer
 *
 * @param string $videoPlayer: player to use : flash, flashOrVideo, flashWii, divx, wmp, vlc, qt, qtReenc
 * @param string $completeFilename: file & path of source video
 * @param string $videoWidth: source video width
 * @param string $videoHeight: source video height
 */
function insertVideoPlayer($videoPlayer,$completeFilename, $videoWidth, $videoHeight, $options=array()){
	$options['videoPlayer']=$videoPlayer;

	switch ($videoPlayer){
		case 'flashWii':
		case 'flashFlv':
		case 'flashF4v':
			insertVideoPlayerFlashReenc($completeFilename, $videoWidth, $videoHeight,$options);break;
		case 'html5Auto':
		case 'html5Webm':
		case 'html5Ogv':
			insertVideoPlayerHtml5Reenc($completeFilename, $videoWidth, $videoHeight,$options);break;
		case 'flash': insertVideoPlayerFlash($completeFilename, $videoWidth, $videoHeight);break;
		case 'divx': insertVideoPlayerDivX($completeFilename, $videoWidth, $videoHeight);break;
		case 'wmp': insertVideoPlayerWMP($completeFilename, $videoWidth, $videoHeight);break;
		case 'vlc': insertVideoPlayerVLC($completeFilename, $videoWidth, $videoHeight);break;
		case 'qtReenc': insertVideoPlayerQTReenc($completeFilename, $videoWidth, $videoHeight);break;
		case 'qt': insertVideoPlayerQT($completeFilename, $videoWidth, $videoHeight);break;
		case 'm3u': insertVideoPlayerM3U($completeFilename, $videoWidth, $videoHeight);break;
	}
}

/**
 * Insert Javascript used by all players
 *
 */
function insertCommonVideoJS(){
	echo cfScriptLink('wz_dragDrop.js');
?>
<style type="text/css">
.buttonBig{border:1px solid #DDD;background:rgba(255,255,255,0.3);width:64px;height:64px;text-align:center;margin-top:8px}
.buttonBig img{width:48px;height:48px}
</style>
<script type="text/javascript">
var player,windowReloading,html5Fullscreen=0;

function init(){
	var e,defaultPlayer;

	if(basename.substr(0,7)!='webcam:') winMe.setCaption(basename);

	// Ensure selected player is supported, and hide unsupported players
	for(e in vvfAllowedPlayers){
		if((e=='flashFlv' && !wl.flashVersion()) ||
			(e=='flashF4v' && !(wl.flashVersion()>9)) ||
			(e=='html5Webm' && !wl.html5Video('webm')) ||
			(e=='html5Ogv' && !wl.html5Video('ogv'))){
				try{$('bt_'+e).style.display='none'} catch(e){}
				vvfAllowedPlayers[e]=0
		}
		else if(!defaultPlayer) defaultPlayer=e;
	}

	try{btActivate('bt_'+videoPlayer)} catch(e){}
	if(W.isMobile) setTimeout(winResized,300); // Android resize issue WA

	// If selected player is not supported by browser, switch to 1st supported one
	if(!vvfAllowedPlayers[videoPlayer] && defaultPlayer && dgi('bt_'+defaultPlayer)) {
		// Select default player (if possible)
		try{return $('bt_'+defaultPlayer).click()} catch(err){}
	}
	player=(dgi('playerVideoEmbed')?dgi('playerVideoEmbed'):dgi('playerVideo'));
	if(W.orientation!=undefined) updateOrientation();
	else{
		if(stretchPlayer && stretchablePlayer) winResized(winMe.getInnerWidth(),winMe.getInnerHeight());
		else {
			resizeWin(outputWidth,outputHeight+playerControlsHeight);
			winResized();
		}
	}
	if(W.winMe) winMe.setCloseFunction("winOnClose");
}
function winOnClose(){
	// Remove player
	dgi("playerVideoDiv").innerHTML="";
}

// Window resized
function winResized(ww,wh){
	<?php if(cfBGetVar('OS')=='iOS') echo "return;";?>
	if(W.fullscreen || !player) return;

	if(ww==undefined) {ww=winMe.getInnerWidth();wh=winMe.getInnerHeight()}

	var mw=(ww*0.95), mh=wh, mT;
	if(dgi('extraControls')) mh=dgi('extraControls').offsetTop;
	mh-=dgi('headerTable').offsetHeight+actualOffsetTop(dgi('headerTable'));
	var maxH=mh;
	mw=Math.max(50,Math.floor(mw));
	mh=Math.max(40,Math.floor(mh*0.95));
	if(mw/mh>ratio) mw=Math.floor(mh*ratio); else mh=Math.floor(mw/ratio);

	if(!stretchablePlayer || (!stretchPlayer && mw>outputWidth)) {mw=outputWidth;mh=outputHeight}

	var i,pls=[dgi('playerVideo'),dgi('playerVideoEmbed')], pVI=dgi('playerVideoImage'),s;
	for(i=0;i<2;i++) if(pls[i]){
		s=pls[i].style;
		s.width=(mw)+'px';
		s.height=(mh+playerControlsHeight)+'px';
		mT=((maxH-mh-playerControlsHeight)/2-getElementStyle(dgi('headerTable'),'margin-bottom'));
		if(W.html5Fullscreen) mT=(wh-mh-playerControlsHeight)/2;
		s.marginTop=mT+'px';
	}
	if(pVI) {pVI.style.width=(mw)+'px';pVI.style.height=(mh)+'px'}
	// HTML5 stuff positionning
	try{
		if(!isMobile){
			$('html5PlayBt').style.left=(10+(ww-mw)/2)+'px';
			$('html5PlayBt').style.top=($('playerVideo').offsetHeight+mT-10-32)
			$('html5PauseBt').style.left=(10+(ww-mw)/2)+'px'
			$('html5PauseBt').style.top=($('playerVideo').offsetHeight+mT-10-32)
			$('html5Error').style.marginTop=mT/2;
			$('html5Loading').style.marginTop=mT/2;
		}
	} catch(e){}

	if(dgi('preencodeStuff')) dgi('preencodeStuff').style.top=(mh/2-dgi('preencodeStuff').offsetHeight/2)+'px';
}

function resizeWin(pw,ph){
	<?php if(cfBGetVar('name')=='iPhone') echo "return;";?>
	if(pw==undefined) pw=(player.offsetWidth)?player.offsetWidth:outputWidth;
	ph=Math.floor(pw/ratio+playerControlsHeight);
 	var ww=Math.floor(pw/0.95);
	var wh=Math.floor(ph/0.95+dgi('headerTable').offsetHeight+actualOffsetTop(dgi('headerTable')));
if(phpBrowser=='ie') wh+=30;
	if(dgi('extraControls')) wh+=dgi('extraControls').offsetHeight;
	winMe.resizeInnerTo(Math.max(ww,420),Math.max(wh,260),1);
}
function updateOrientation(){
	if(W.orientation==undefined) return;
	if(Math.abs(orientation)==90) {
		dgn('viewport').setAttribute("content", "width=480, height=268, user-scalable=yes,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0");
		D.body.style.height='268px';
	}
	else {
		dgn('viewport').setAttribute("content", "width=320, height=416, user-scalable=yes,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0");
		D.body.style.height='416px';
	}

	var maxH=D.body.offsetHeight-actualOffsetBottom(dgi('headerTable'));
	player.style.marginTop=((maxH-player.offsetHeight)/2-getElementStyle(dgi('headerTable'),'margin-bottom'))+'px';
	W.scrollTo(0,1);
	if(W.videoPlayerIPhoneOrientationChange) videoPlayerIPhoneOrientationChange();
}
/**
 * Set fullscreen video
 */
function setFullscreen(goFs){
	if(goFs===undefined) goFs=1-html5Fullscreen;

	// Try "hardware" fullscreen
	if($('playerVideo').e.toggleFullscreen(1)) {
		return;
	}

	var s=$('playerVideoDiv').style;
	dgi('btStretchPlayerF').style.display=(goFs)?'none':'';
	dgi('btStretchPlayerN').style.display=(goFs)?'':'none';
	html5Fullscreen=goFs;
	if(goFs) winMe.setFullscreen();
	else winMe.resetBorders();

	s.position=(goFs)?'fixed':'relative';
	s.width=(goFs)?'100%':'auto';
	s.height=(goFs)?'100%':'auto';
	s.zIndex=99;
	s.background=(goFs)?'black':'';

	winResized();
}
function setVideoPlayer(vp){submitFrm('setVideoPlayer',vp,true)}
function submitFrm(action,args,sync){windowReloading=1;fAS(undefined,'view',action,args,(sync)?"sync":true)}
function reloadPage(){submitFrm(false,false,1)}
function setTitle(title){dgi('viewVideoTitle').innerHTML=title}
</script>
<?php
}

/**
 * @display a clickable progress bar for use with flash / HTML5 streaming
 *
 * @param integer $completeFilename : video file name
 * @param integer $videoDuration : video file duration, in seconds
 * @param integer $videoPosition : offset position, in seconds
 * @param $videoWidth: video width (used for ratio computation)
 * @param $videoHeight: video height (used for ratio computation)
 */
function displayProgressBar($completeFilename,$videoDuration, $videoPosition=0,$width='90%',$videoWidth=0,$videoHeight=0){
	// Video size
	if($videoWidth) $r=$videoHeight/$videoWidth; else $r=3/4;


	// Progress bar (only if known video duration)
	if($videoDuration>0){
		$hours=floor($videoDuration/3600);
		$mins=floor(($videoDuration-3600*$hours)/60);
		$secs=$videoDuration-3600*$hours-60*$mins;

		$totalDurationCaption=cfCaption('javascriptHourMinSecFormat',str_pad($hours,2,'0',STR_PAD_LEFT),str_pad($mins,2,'0',STR_PAD_LEFT),str_pad($secs,2,'0',STR_PAD_LEFT));
		echo '<div class="eTC" style="padding:0;border:0;margin:0;background:none;position:relative">';
		if(cfGetBrowser()=='ie'){
			echo outProgressBar($videoPosition/$videoDuration,$width,$totalDurationCaption,'pb',false,false,'cursor:pointer');
			echo '<div id="pbMask" style="position:relative;top:-14px;left:0;height:100%;width:100%;background:url(\'/gfx/v.gif\')" onmouseout="pbMouseOut()" onmousemove="pbMouseMove()" onclick="pbClick()"></div>';
		}
		else {
			echo outProgressBar($videoPosition/$videoDuration,$width,$totalDurationCaption,'pb',false,false,'cursor:pointer');
			echo '<div style="position:absolute;top:0px;width:100%;height:130%; background:red;opacity:0.01;-moz-opacity:0.01;-khtml-opacity:0.01;" onmouseout="pbMouseOut()" onmousemove="pbMouseMove()" onclick="pbClick()">&nbsp;</div>';
		}
		echo "</div>\n";
		echo '<div id="mo" style="display:none"></div>';

		if(cfIsWII()){
?>
<style type="text/css">
#dd.elements.tooltip{width:200px;height:160px;display:block}
.progressOuter{background:#FFF; border:1px solid #AAA;height:22px !important; height:44px;overflow:hidden}
span.progressOuter{}
.progressInner{background:#06CEEE;height:22px;}
.progressText{font-size:16px;font-weight:bold;top:0px;color:#222}
</style>
<?php
		}
?>
<script type="text/javascript">
D.onmousemove = setState;
var XX,YY,mo=false;
var durationCaption="<?php echo cfCaption('javascriptHourMinSecFormat');?>";
var requestedPosition=<?php echo (int)$videoPosition;?>;

function pbMouseOut(){
	dgi("mo").innerHTML="";
	if(dgi('tooltip')) dgi('tooltip').style.display='none';
}
function setState(e){
	if (e && e.pageX) {
		XX=e.pageX;
		YY=e.pageY;
	}
	else {
		XX=event.x+D.body.scrollLeft;
		YY=event.y+D.body.scrollTop;
	}
}

function pbMouseMove(){
	var PHPBrowser="<?php echo cfGetBrowser();?>";
	dgi("mo").innerHTML="1";
	var pos=Math.min(1,Math.max(0,(XX-actualOffsetLeft(dgi('pb')))/(dgi("pb").offsetWidth)));
	progressSetPerc('pb',100*pos);
	requestedPosition=(pos*videoDuration);
	pos=Math.floor(pos*videoDuration);
	var hours=Math.floor(pos/3600); pos-=hours*3600; if(hours<10) hours='0'+hours;
	var mins=Math.floor(pos/60); pos-=mins*60; if(mins<10) mins='0'+mins;
	pos=Math.floor(pos);
	if(pos<10) pos='0'+pos;
	dgi('pb_text').innerHTML=durationCaption.replace(/%1/gi,hours).replace(/%2/gi,mins).replace(/%3/gi,pos) + " / <?php echo $totalDurationCaption; ?>";

	var w=<?php echo cfConstant('DYNAMIC_VIDEO_THUMBNAIL_WIDTH');?>,h=<?php echo floor(cfConstant('DYNAMIC_VIDEO_THUMBNAIL_WIDTH')*$r)?>;

	// Create tooltip DOM node
	if(!dgi('tooltip')){
		with(tooltipDiv=D.createElement("div")){
			id='tooltip';
			className='tooltip';
			style.visibility='hidden';
			innerHTML='<img id="dynPreviewImg" src="" style="display:block;width:'+w+'px;height:'+h+'px" onload="pbDynPreviewLoaded()" alt="">';
		}
		D.body.appendChild(tooltipDiv)
		ADD_DHTML('tooltip',1);
		dd.elements.tooltip.maximizeZ();
	}
	var td=$('tooltip'),ttw=td.offsetWidth;
	td.style.display='';

	// Move tooltip
	var tt=dd.elements.tooltip,x=mouseX-ttw/2;
	x=Math.max(0,Math.min(x,winMe.getInnerWidth()-ttw))
	tt.moveTo(x,mouseY-td.offsetHeight-10);

	// Set URL of image and load
	pbDynPreviewLoad(w,h,requestedPosition);
}
function pbClick(){
	videoPosition=requestedPosition;
	changeQualitySettings();
}
function pbDynPreviewLoad(w,h,pos){
	if(!pos) return;
	// Set tooltip image src
	var i,src="<?php echo $_SERVER['REQUEST_URI'];?>";
	D.comForm.data4.value='dynPreview';
	D.comForm.data5.value=w+"/"+h+"/"+pos;
	for(i=1;i<=5;i++) src+=((i>1)?'&':'?')+'data'+i+'='+escape(D.comForm['data'+i].value.replace(/%/g, '%25').replace(/\+/g, '%2B').replace(/%25/g, '%'));
	if(src===W.pbDynNextURL) return;

	// New URL
	W.pbDynNextURL=src;
	// If previous image is still loading, wait
	if(W.pbDynPreviewLoading) {
		W.pbDynPreviewNewURL=1;
		return;
	}
	// Else set image source
	$('dynPreviewImg').src=src;
	W.pbDynPreviewLoading=true;
}
function pbDynPreviewLoaded(){
	dgi('tooltip').style.visibility='';
	if(!W.pbDynPreviewNewURL) return W.pbDynPreviewLoading=0;
	W.pbDynPreviewNewURL=0;
	$('dynPreviewImg').src=W.pbDynNextURL;
}
function processFlashCommand(commande,args) {
	//if(commande=='debug') {dgi("debug").innerHTML+=args;return;}
	if(dgi("mo").innerHTML=="1" || isNaN(args) || commande!='setTime') return;
	if(commande=='setTime' && dgi("mo").innerHTML!="1" && !isNaN(args)) setTime(args);

}
function setTime(args){
	videoPosition=args*1;
	progressSetPerc('pb',100*videoPosition/videoDuration);
	var pos=Math.floor(videoPosition);
	var hours=Math.floor(pos/3600); pos-=hours*3600; if(hours<10) hours='0'+hours;
	var mins=Math.floor(pos/60); pos-=mins*60; if(mins<10) mins='0'+mins;
	if(pos<10) pos='0'+pos;
	progressSetText('pb',durationCaption.replace(/%1/gi,hours).replace(/%2/gi,mins).replace(/%3/gi,pos)+" / <?php echo $totalDurationCaption; ?>");
}
</script>
<?php
	}
	/**
	 * Encoding & player options
	 */

?>
<fieldset><span style="margin-right:0.5em;font-weight:bold"><?php echo cfCaption('videoCfgFlashSize'); ?></span>
<select id="flashOutputSize" size="1" class="textInput" onchange="changeQualitySettings()">
<?php

	if(@is_file($completeFilename) || cfCmpLeft($completeFilename,'webcam:')) list($sourceWidth, $sourceHeight)=vvfVideoDimensions($completeFilename,true); else $sourceWidth=0;

	/**
	 * Video size selection control
	 */
	if(@$sourceWidth) {
		// Display source
		if(cfRGetVar('flashOutputSizeMax')=='original' || cfRGetVar('flashOutputSizeMax')>=$sourceWidth)
			echo '<option value="original" selected>'.cfCaption('videoCfgSource').' ('.$sourceWidth.'x'.$sourceHeight.')</option>';
	}

	switch (cfRGetVar('flashOutputSizeMax')){
		case 'original':
			//echo '<option value="original" selected>'.cfCaption('videoCfgSource').(($sourceWidth && $videoHeight)?' ('.$sourceWidth.'x'.$videoHeight.')':'').'</option>';
		case 1280:
			if(!$sourceWidth || $sourceWidth>=1280) echo '<option value="1280" '.((cfRGetVar('flashOutputSize')=='1280')?'selected':'').'>1280x'.floor(1280*$r).'</option>';
		case 900:
			if(!$sourceWidth || $sourceWidth>=900) echo '<option value="900" '.((cfRGetVar('flashOutputSize')=='1280')?'selected':'').'>900x'.floor(900*$r).'</option>';
		case 640:
			if(!$sourceWidth || $sourceWidth>=640) echo '<option value="640" '.((cfRGetVar('flashOutputSize')=='640')?'selected':'').'>640x'.floor(640*$r).'</option>';
		case 400:
			if(!$sourceWidth || $sourceWidth>=400) echo '<option value="400" '.((cfRGetVar('flashOutputSize')=='400')?'selected':'').'>400x'.floor(400*$r).'</option>';
		case 320:
			if(!$sourceWidth || $sourceWidth>=320) echo '<option value="320" '.((cfRGetVar('flashOutputSize')=='320')?'selected':'').'>320x'.floor(320*$r).'</option>';
		default:
			if(!$sourceWidth || $sourceWidth>=160) echo '<option value="160" '.((cfRGetVar('flashOutputSize')=='160')?'selected':'').'>160x'.floor(160*$r).'</option>';
	}
?>
</select>
</fieldset>

<fieldset><span style="margin-right:0.5em;margin-left:1em;font-weight:bold"><?php echo cfCaption('genQuality');?></span>
<select id="flashOutputQuality" size="1" class="textInput" style="margin-right:1em" onchange="changeQualitySettings()">
<?php
switch (cfRGetVar('flashOutputQualityMax')){
	case 'original':
		echo '<option value="original" selected>'.cfCaption('videoCfgSource').'</option>';
	case 'high':
		echo '<option value="high" '.((cfRGetVar('flashOutputQuality')=='high')?'selected':'').'>'.cfCaption('genHigh').'</option>';
	case 'medium':
		echo '<option value="medium" '.((cfRGetVar('flashOutputQuality')=='medium')?'selected':'').'>'.cfCaption('genMedium').'</option>';
	default:
		echo '<option value="low" '.((cfRGetVar('flashOutputQuality')=='low')?'selected':'').'>'.cfCaption('genLow').'</option>';
}
?>
</select>
</fieldset>

<fieldset>
<?php
// Image ratio
echo '<span style="'.((VVOptions::noRatio())?'display:none':'').'">';
echo '<span style="margin-right:0.5em;margin-left:1em;font-weight:bold">'.cfCaption('genFormat').'</span>';
echo '<select id="abmRatio" size="1" class="textInput" style="margin-right:1em" onchange="changeQualitySettings()">';
echo '<option value="" '.((cfRGetVar('abmRatio')=='')?'selected':'').'>'.cfCaption('videoCfgSource').'</option>';
echo '<option value="*4*3*" '.((cfRGetVar('abmRatio')=='*4*3*')?'selected':'').'>4/3</option>';
echo '<option value="*16*9*" '.((cfRGetVar('abmRatio')=='*16*9*')?'selected':'').'>16/9</option>';
echo '<option value="*20*9*" '.((cfRGetVar('abmRatio')=='*20*9*')?'selected':'').'>20/9</option>';
echo '<option value="*24*9*" '.((cfRGetVar('abmRatio')=='*24*9*')?'selected':'').'>24/9</option>';
echo '</select></span></fieldset>';

// H264 encoding
//if(cfFfmpegVersion()=='GPL') echo '&nbsp;H264&nbsp;<input type="checkbox" id="h264Encoding" '.((cfRGetVar('disableH264FlvEncoding'))?'':'checked="checked"').' onchange="changeQualitySettings()">';

?>
</span>
<script type="text/javascript">
function changeQualitySettings(){
	submitFrm('changeFlashQuality',dgi('flashOutputSize').value+dgi('abmRatio').value+'/'+dgi('flashOutputQuality').value+'/'+(0+videoPosition)+'/'+videoDuration,1);
}
var videoStartPosition=<?php echo (int)$videoPosition;?>, videoPosition=<?php echo (int)$videoPosition;?>, videoDuration=<?php echo $videoDuration;?>;
</script>
<?php
}

/**
 * @desc Main function called by viewFunctions.php
 *
 * @param string $completeFilename: video file to view
 * @param string $command
 * @param string $extraData
 * @param array $options: 'externalFilename'=>filename to use for external ref
 * 						  'streaming'=>true to indicate live streaming (no duration)
 * 						  'videoPlayer'=>set a video player name to force (instead of using cfRGetVar) - so far only used by mobileViewFunctions for HTML5
 */
function vfvViewVideo($completeFilename,$command,$extraData,$options=array()){

	// Set file name that will be used to build video url with cfExtVideo function
	$externalFilename=(isset($options['externalFilename']))?$options['externalFilename']:$completeFilename;

	cfDebugTag('videoDebugTagTop');

	// Video ratio management
	cfRSetVar('abmRatio','');
	if (strpos($extraData,'*')!=false){
		list($s1,$s2,$s3)=explode('*',$extraData);
		$s='*'.$s2.'*'.$s3.'*';
		cfRSetVar('abmRatio',$s);
		$extraData=str_replace($s,'',$extraData);
	}

	/**
	 * Init data - V1.0.9- compatibility
	 */
	if(!cfRGetVar('reencodedVideo') && !cfRGetVar('allowOriginalVideo')) cfRSetVar('allowOriginalVideo',1);
	if(!cfRGetVar('qtOutputSize') || !is_numeric(cfRGetVar('qtOutputSize'))) cfRSetVar('qtOutputQuality',480);
	if(cfRGetVar('qtOutputQuality')!='high' && cfRGetVar('qtOutputQuality')!='medium' && cfRGetVar('qtOutputQuality')!='low') cfRSetVar('qtOutputQuality','medium');

	if(cfBGetVar('name')=='iPhone') $_ENV['meta']='<meta name="viewport" content="width=480, height=268, initial-scale=1.0,minimum-scale=1.0, maximum-scale=1.0, user-scalable=yes">';

	// Set video player stretched to fit window by default
	cfRInitVar('stretchPlayer',1);

	// Get video player
	if(isset($options['videoPlayer'])) $videoPlayer=$options['videoPlayer'];
	else $videoPlayer=cfRGetVar('videoPlayer');

	$allowedPlayers=vvfAllowedPlayers(cfFileExtension($completeFilename));

	// If browser not allowed for this extension, check if user has already selected a video player for this ext
	if(!isset($allowedPlayers[$videoPlayer])){
		if(cfRGetVar('selectedPlayer',cfFileExtension($completeFilename))) $videoPlayer=cfRGetVar('selectedPlayer',cfFileExtension($completeFilename));
		// No player already selected by user
		else {
			$videoPlayer=false;
			if($allowedPlayers && count($allowedPlayers)) {
				// Find "best" player / combo
				foreach ($_ENV[((cfIsMobile())?'preferedMobilePlayers':'preferedPlayers')] as $k=>$v) {
					if(isset($allowedPlayers[$k]) && ($v!=='GPL' || cfFfmpegVersion()=='GPL')){
						$videoPlayer=$k;
						break;
					}
				}

				// If none matches (should not happend), use 1st one
				if(!$videoPlayer) {reset($allowedPlayers);$videoPlayer=key($allowedPlayers);}
			}
		}
	}
	cfRSetVar('videoPlayer',$videoPlayer);

	// No possible video player : display error page
	if(!$videoPlayer) {
		// iPhone, GPL ffmpeg not installed
		if(cfBGetVar('name')=='iPhone' && cfFfmpegVersion()!='GPL') outDisplayErrorPage(outImage(outIcon('noPlayer')).'<br>'.cfCaption('ffmpegGPLIPhoneWarning'));

		// others
		else outDisplayErrorPage(outImage(outIcon('noPlayer')));
	}

	// Set movie dimensions
	list($width, $height)=vvfVideoDimensions($completeFilename,true);
	$videoDuration=0; $videoPosition=0; cfRSetVar('flashOutputOffset',0);

	/*
	 ***************************************************************************************************************************
	 * Process POST commands
	 ***************************************************************************************************************************
	 */
	if(cfIsAsync()){
		//cfDbg($command,1);
		cfAsyncHeader();
		switch ($command){
			// Preencode video
			case 'encodeVideo':
				viewVideoPreencodeVideo($completeFilename,$extraData,$width,$height);
			break;
			// Cancel encoding
			case 'encodeVideoCancel':
				if($extraData=='qt' && ($pid=isEncoding($completeFilename,'mp4')) && !isEncoded($completeFilename,'mp4')){
					cfServerSendCommand('killFfmpeg pid="'.$pid.'"');
					setEncoding($completeFilename,'mp4',0);
					$rfn=reencodedCompleteFilename($completeFilename,'mp4');
					$i=0; while(file_exists($rfn) && $i++<7) {@unlink($rfn); sleep(1);}
					echo cfAsyncXMLJSaction('doneCancel()');
				}
				if($extraData=='flv' && ($pid=isEncoding($completeFilename,'flv')) && !isEncoded($completeFilename,'flv')){
					cfServerSendCommand('killFfmpeg pid="'.$pid.'"');
					setEncoding($completeFilename,'flv',0);
					$rfn=reencodedCompleteFilename($completeFilename,'flv');
					$i=0; while(file_exists($rfn) && $i++<7) {@unlink($rfn); sleep(1);}
					echo cfAsyncXMLJSaction('doneCancel()');
				}
			break;
			// Monitor encoding
			case 'monitorEncoding':
				// Quicktime / mp4
				if($extraData=='qt'){
					// Done encoding: redirect browser to encoded file (iPhone)
					if(isEncoded($completeFilename,'mp4')) {
						// Redirect browser to encoded file (standaloneMediaPlayer)
						if(cfBGetVar('standaloneMediaPlayer')){
							cfPingUpdate('force',1.5*vvfVideoDuration($completeFilename));
							echo cfAsyncXMLJSaction('location.replace("'.cfHostName().cfExtVideo(reencodedCompleteFilename($completeFilename,'mp4')).'")');
						}
						// Reload page to display inline player
						else echo cfAsyncXMLJSaction('reloadPage()');
					}
					// Not yet finished encoding, update progressBar
					else echo cfAsyncXMLJSaction('monitorEncoding('.encodingProgress($completeFilename,'mp4').')');
				}
				// Flash / flv
				if($extraData=='flv'){
					// Done encoding
					if(isEncoded($completeFilename,'flv')) {
						// Redirect browser to encoded file (standaloneMediaPlayer)
						if(cfBGetVar('standaloneMediaPlayer')){
							$playerAdress=cfHostName().'/js/vplayer.swf?file='.cfExtVideo(reencodedCompleteFilename($completeFilename,'flv')).'&w='.$width.'&h='.$height.'&offset='.cfRGetVar('flashOutputOffset').'&autoplay=1&noCallback=1&buttonSize=50&buffer='.((cfRGetVar('bufferTime'))?cfRGetVar('bufferTime'):2).'&ping='.max(cfGGetVar('clientPingInterval'),1).'&browser='.cfGetBrowser().'&ffmpegVersion='.cfFfmpegVersion();
							echo cfAsyncXMLJSaction('location.replace("'.cfHostName().$playerAdress.'")');
						}
						// Reload page to display inline player
						else echo cfAsyncXMLJSaction('reloadPage()');
					}
					// Not yet finished encoding, update progressBar
					else echo cfAsyncXMLJSaction('monitorEncoding('.encodingProgress($completeFilename,'flv').')');
				}
			break;
		}

		cfAsyncFooter();
	}

	// Change video player
	if($command=='dynPreview'){
		WHeaders::contentType('jpg');
		WHeaders::noCache();
		list($w,$h,$pos)=explode('/',$extraData);

		echo efMakeVideoThumbnail($completeFilename,false,$w,$h,65,floor($pos*10)/10);
		exit;
	}
	if($command=='setVideoPlayer' && $extraData && !cfRGetVar('videoPlayerLocked')){
		if(isset($allowedPlayers[$extraData])) {
			cfRSetVar('videoPlayer',$videoPlayer=$extraData);
			// Mem selected player for this extension
			cfRSetVar('selectedPlayer',cfFileExtension($completeFilename),$extraData);
		}
	}

	// Process quality/output dimensions/position change requests (real-time flash encoding only)
	if($command=='changeFlashQuality' && $extraData && cfRGetVar('reencodedVideo')){

		list($ns, $nq, $videoPosition, $videoDuration)=explode('/',$extraData);
		if(!cfRGetVar('renderingLocked')){
			if($ns=='original' || $ns=='1280' || $ns=='900' || $ns=='640' || $ns=='400' || $ns=='320' || $ns=='160') cfRSetVar('flashOutputSize',$ns);
			if($nq=='original' || $nq=='high' || $nq=='medium' || $nq=='low') cfRSetVar('flashOutputQuality',$nq);
			if(is_numeric($videoPosition)) cfRSetVar('flashOutputOffset',$videoPosition);
		}
		//cfRSetVar('disableH264FlvEncoding',(($h264Encoding)?0:1));
		//echo cfAsyncXMLJSaction('setFlashPlayerSrc("'.cfExtVideo('*singleVideo*.xml').'")');
	}

	// Send m3u file
	if($command=='sendM3U'){
		header('Content-Type: audio/x-mpegurl');
		header("Pragma:"); // Do not remove, required by IE in SSL mode
		header('Content-Disposition: attachment; filename='.cfFileWithoutExtension(basename($completeFilename)).'.m3u');
		$output="#EXTM3U\n#EXTINF:,".basename($completeFilename)."\n";
		$output=cfExtVideo($externalFilename,'','absolute');
		die($output);
	}



	/**
	 * Misc
	 */

	// Set ratio
	if (cfRGetVar('abmRatio')!=''){
		list($s1,$s2)=explode('*',substr(cfRGetVar('abmRatio'),1,-1));
		$height=$width*$s2/$s1;
	}
	if($height) $ratio=$width/$height; else $ratio=4/3;

	// if reencoded video, use reencoding dimensions instead of original video dimensions
	if(($videoPlayer=='flashFlv' || $videoPlayer=='flashF4v' || $videoPlayer=='html5Webm' || $videoPlayer=='html5Ogv' || $videoPlayer=='html5Auto' || $videoPlayer=='flashWii') && cfRGetVar('reencodedVideo') && cfRGetVar('flashOutputSize')!='original'){
		list($width,$height)=efGetOutputImageDimensions($width,$height,cfRGetVar('flashOutputSize'));
	}

	/**
	 * Standalone media player : directly open file in browser
	 */
	if(cfBGetVar('standaloneMediaPlayer')){
		// Quicktime reencoding standalone player
		if($videoPlayer=='qtReenc' && isEncoded($completeFilename,'mp4')){
			cfPingUpdate('force',1.5*vvfVideoDuration($completeFilename));
			header('Location: '.cfExtVideo(reencodedCompleteFilename($completeFilename,'mp4')));
			exit();
		}
	}

	/*
	 ***************************************************************************************************************************
	 * Display page
	 ***************************************************************************************************************************
	 */

	cfInsertHEAD(false,(cfIsMobile())?array('noMeta'):array()); // For mobile, don't use common viewport meta

	if(cfIsMobile() && cfBGetVar('OS')!=='iOS') echo '<meta name="viewport" id="viewport" content="width=device-width, height=500, initial-scale=1.0, user-scalable=no, target-densitydpi=medium-dpi">';

	insertCommonVideoJS();
	?>
<script type="text/javascript">
var ratio=<?php echo number_format($ratio,3,'.','');?>,
videoPlayer="<?php echo $videoPlayer;?>",
vvfAllowedPlayers=<?php echo json_encode($allowedPlayers);?>,
stretchablePlayer=<?php echo $_ENV['stretchablePlayer'][$videoPlayer];?>,
playerControlsHeight=<?php echo $_ENV['playerControlsHeight'][$videoPlayer];?>,
stretchPlayer=<?php echo (int)(cfRGetVar('stretchPlayer'));?>,
outputWidth=<?php echo $width;?>, outputHeight=<?php echo $height;?>,
basename="<?php echo cfUTF8Encode(basename(cfFileWithoutExtension($completeFilename)));?>";
<?php echo VVOptions::windowSizeAutoAdaptScript($width,$height);  ?>
</script>
</head>
<body class="iframeBody" id="videoFrameBody" style="overflow:hidden;padding:0;margin:0" onorientationchange="updateOrientation()">
	<?php
	// Force an external file identifier not based on filename (webTV: use channel id instead of url)
	if(is_numeric($externalFilename)) $fileDesc=$externalFilename;
	else if(cfRGetVar('vvfForcedVideoExternalId')) $fileDesc=cfRGetVar('vvfForcedVideoExternalId');
	else $fileDesc=cfUTF8Encode(basename($completeFilename));

	outInsertStandardComForm($_SERVER['PHP_SELF'],((@is_file($externalFilename))?cfUTF8Encode(dirname(cfResourceRelativePath($externalFilename))):$externalFilename),$fileDesc,'view');

	// Set video name from forced value first, or from filename by default
	if(!($videoName=cfRGetVar('vvfForcedVideoName'))) $videoName=cfUTF8Encode(basename(cfFileWithoutExtension($completeFilename)));

	/*****************************************************************
	 * Mobile Page header
	 *****************************************************************/
	if(cfIsMobile()) {
		mfInsertTopControls('view',$videoName,VVOptions::getMobileTopControlsExtra());
		echo '<div id="headerTable"></div>'; // Required for resize scripts
	}
	/*****************************************************************
	 * Browser Page header
	 *****************************************************************/
	else{
		?><div class="frame1" style="height:100%;margin:0;overflow:hidden"><?php


		// Player icon & title
		if(VVOptions::useDropDownMenuForPlayerSelection())
			$l=outThemeImage(outIcon(playerIcon($videoPlayer)),false,false,'margin-right:1em;vertical-align:middle');
		else
			$l='';
		$l.='<span id="viewVideoTitle" style="width:100%;white-space:nowrap;text-overflow:ellipsis">'.$videoName.'</span>';

		// Other players
		$r='';
		$r.=VVOptions::getDesktopTopControlsExtra();

		if(VVOptions::useDropDownMenuForPlayerSelection()){
			if((count($allowedPlayers>1) || VVOptions::getExtraPlayers()) && !cfRGetVar('videoPlayerLocked')){
				$r.='<span class="link" id="videoPlayerBt">'.cfCaption('explorerAudioPlayer').'...'.outImage(outIcon('down'),false,false,'vertical-align:middle"').'</span>';

				$items='[';

				foreach ($allowedPlayers as $key=>$value) {
					if($key!='html5Auto'){
						$items.=(($items==='[')?'':',').'ContextMenuItem("'.outIcon($key).'","'.$value.'","setVideoPlayer(\''.$key.'\')")';
					}
				}
				if(VVOptions::getExtraPlayers()){
					$items.=',ContextMenuItem("sep")';
					foreach (VVOptions::getExtraPlayers() as $value) {
						list($id,$icon,$caption)=$value;
						$items.=',ContextMenuItem("'.outIcon($icon).'","'.$caption.'","setVideoPlayer(\''.$id.'\')")';
					}
				}
				$items.=']';

				$r.='<script type="text/javascript">';
				$r.="ContextMenuAttach(dgi('videoPlayerBt'),".$items.",{'align':'right','leftClick':1,'button':1,'style':{'width':130}})";
				$r.='</script>';

			}
		}
		elseif(count($allowedPlayers>1) && !cfRGetVar('videoPlayerLocked')){
			$r='<span style="white-space:nowrap">';
			foreach ($allowedPlayers as $key=>$value) {
				if($key!='html5Auto'){
					$r.=outBt(false,outIcon($key),'javascript:setVideoPlayer(\''.$key.'\')','bt_'.$key,false,outToolTip($value,false,false,true));
				}
			}
			$r.='</span>';
		}
		// Resize button(s)
		if($_ENV['stretchablePlayer'][$videoPlayer] && $videoPlayer!='flashFlv' && $videoPlayer!='flashF4v'){
			$r.='&nbsp;'.outButton(false,'javascript:setFullscreen(1)',outIcon('fullSize'),cfCaption('fullscreen'),'btStretchPlayerF');
			$r.=outButton(false,'javascript:setFullscreen(0)',outIcon('normalSize'),false,'btStretchPlayerN','style="display:none"');
		}
		echo outFrameHeaderTable('frame1Header',$l,$r,'id="headerTable"',(cfTGetVar('avoidTitleBar'))?'display:none':'');
	}

	/**
	 * Video Player
	 */
	echo '<center>';

	insertVideoPlayer($videoPlayer,$externalFilename,$width,$height,$options);
	echo '</center>';
	echo cfDragRegisterItems();
?>
	<script type="text/javascript">init()</script>
	</div></body></html>
<?php
}