<?php
// Class definition found at http://www.zend.com/zend/spotlight/creating-zip-files3.php
// Some alterations to the original posted code were made in order to get everything working properly

// Other modification have been made to include Weezo transfer speed limitation and already-compressed-files optimization

// http://www.pkware.com/documents/casestudies/APPNOTE.TXT

define('OUTPUT_BUFFER_SPEED',2048);

// Required for mime types
include_once(INCLUDE_DIR.'/explorerFunctions.php');

function out($txt){
	return;
	//cfDbg($txt,cfAppDocRoot().'/zip.log',true);
}

/**
 * @desc zip compression class
 * 		difference with zipfile class : real-time zip generation, no memory buffering, direct send to output
 *
 */
class zipfile2
{
	var $datasec = array();
	var $ctrl_dir = array();
	var $old_offset = 0;

	var $nbDatasec=0;
	var $nbCtrl_dir=0;
	var $lenData=0;
	var $lenCtrl_dir=0;

	var $downloadSpeedLimit=0;	// Speed limit, in KBytes per seconds
	var $speedLimit;			// Transfert limitation object

	var $uncompressedSize=0;
	var $readSize=0;
	var $t=null; // Transfer

	var $compressionLevel=9; // ZLib Compression (0 uncompressed - 9 max compression). -1 to stream uncompressed file (doesn't work so far)

	/**
	 * @desc Set a speed limit for download
	 *
	 * @param int $sl: speed limitation, in KBytes per seconds
	 * @return unknown
	 */
	function setSpeedLimit($sl){
		if(!is_numeric($sl)) return false;
		if($sl) $this->speedLimit=new transferLimitation($sl*1024);
		$this->downloadSpeedLimit=$sl*1024;
	}

	/**
	 * Set zip compression level (0-9)
	 *
	 * @param int $compressionLevel
	 */
	function setCompressionLevel($compressionLevel){
		if(is_integer($compressionLevel) && $compressionLevel>=0 && $compressionLevel<=9) $this->compressionLevel=floor($compressionLevel);
	}

	/**
	 * @desc Encode filename in CP850
	 *
	 * @param string $name: basename encoded in system codepage
	 * @return string
	 */
	function encode_filename($name){
		return $name;
		$l=strlen($name);
		for ($i=0;$i<$l;$i++){
			$c=@iconv(cfMGetVar('systemCodepage'),'850',substr($name,$i,1));
			if(strlen($c)) $name[$i]=$c; //else $name[$i]='-';
		}
		return $name;
	}

	/**
	 * @desc Add an empty directory to archive
	 *
	 * @param string $completeFilename: complete path
	 * @param string $name: path name beetween zip root and current dir (encoded in system codepage)
	 */
	function add_dir($completeFilename, $name) {
		$name = str_replace("\\", "/", $name);
		$encodedName=$this->encode_filename($name);
out('------------ Add directory: '.$completeFilename.' --------------');
		// Local file header
		$fr = "\x50\x4b\x03\x04";// local file header signature
		$fr .= "\x14\x00";// version needed to extract
		$fr .= "\x00\x00";// general purpose bit flag
		$fr .= "\x08\x00";// compression method
		$time = getdate(filemtime($completeFilename));
		$fr .= pack('vv',2048*$time['hours']+32*$time['minutes']+floor($time['seconds']/2),512*($time['year']-1980)+32*$time['mon']+$time['mday']); // MS-DOS formated modification time & date
		$fr .= pack("V",0);// crc-32
		$fr .= pack("V",0);// compressed size
		$fr .= pack("V",0);// uncompressed size
		$fr .= pack("v", strlen($encodedName) );// file name length
		$fr .= pack("v", 0 );// extra field length
		$fr .= $encodedName;// file name

		$this->nbDatasec++;
		$this->lenData+=strlen($fr);
		echo $fr;

		$new_offset = $this->lenData;

		// Central directory header
		$cdrec = "\x50\x4b\x01\x02"; // local file header signature
		$cdrec .="\x14\x0b"; // version made by
		$cdrec .="\x14\x00"; // version needed to extract
		$cdrec .="\x00\x00"; // general purpose bit flag
		$cdrec .="\x08\x00"; // compression method
		$cdrec .= pack('vv',2048*$time['hours']+32*$time['minutes']+floor($time['seconds']/2),512*($time['year']-1980)+32*$time['mon']+$time['mday']); // MS-DOS formated modification time & date
		$cdrec .= pack("V",0); // crc-32
		$cdrec .= pack("V",0); // compressed size
		$cdrec .= pack("V",0); // uncompressed size
		$cdrec .= pack("v", strlen($encodedName) ); // file name length
		$cdrec .= pack("v", 0 ); // extra field length
		$cdrec .= pack("v", 0 ); // file comment length
		$cdrec .= pack("v", 0 ); // disk number start
		$cdrec .= pack("v", 0 ); // internal file attributes
		$cdrec .= pack("V", 16 ); // external file attributes
		$cdrec .= pack("V", $this -> old_offset ); // relative offset of local header
		$cdrec .= $encodedName; // file name

		$this -> ctrl_dir[] = $cdrec;
		$this->nbCtrl_dir++;
		$this->nbCtrl_dir+=strlen($cdrec);
		$this -> old_offset = $new_offset;
out('---------- Done adding directory --------------');
		return;
	}

	/**
	 * @desc Add a file to zip archive
	 *
	 * @param string $completeFilename: file path & name
	 * @param string $name
	 * @return boolean true if OK, false if file couldn't be opened
	 */
	function add_file($completeFilename, $name) {
		static $deflatableExt=array('bmp','wav');
		static $memoryLimit=0;

		// Initiate memory limit
		if(!$memoryLimit) $memoryLimit=1024*1024*str_replace('M','',ini_get('memory_limit'));
out('------------------------  add file: '.$completeFilename.' (memory limit:'.$memoryLimit);
		// Remove Thumbs.db hidden files from archive
		if(basename($name)=='Thumbs.db' && substr(cfFileAttributes($completeFilename),0,1)=='H') return;

		$filesize=cfFileSize($completeFilename);

		$compressionLevel=$this->compressionLevel;

		// Set compression level to 0 for audio, images and videos (except uncompressed ones)
		if(in_array(efFileType($completeFilename),array('audio','video','image')) && !in_array(cfFileExtension($completeFilename),$deflatableExt)){
			$compressionLevel=0;
		}
		else{
			// Memory limit
			if($filesize*2.5>$memoryLimit) $compressionLevel=0;
		}


		$name = str_replace("\\", "/", $name);
		$time = getdate(filemtime($completeFilename));
		$unc_len = $filesize;

		// No compression: directly process file
		if($compressionLevel==0){
			$crc=unpack('N',pack('H*',hash_file('crc32b',$completeFilename))); $crc=$crc[1];
			$c_len=$filesize;
		}

		// Normal compression: read data in memory and process
		else{
			$data=file_get_contents($completeFilename);
			$crc=crc32($data);
			// GZip data
			$zdata = substr(gzcompress($data,$compressionLevel), 2, -4);
			unset($data);
			$c_len = strlen($zdata);
		}

		$encodedName=$this->encode_filename($name);

		// Local file header
		$fr = "\x50\x4b\x03\x04";// local file header signature
		$fr .= "\x00\x00";// version needed to extract
		$fr .= "\x00\x00";// general purpose bit flag
		if($compressionLevel==0) $fr .= "\x00\x00"; else $fr .= "\x08\x00"; // compression method

		// MS-DOS formated modification time & date
		$fr .= pack('vv',2048*$time['hours']+32*$time['minutes']+floor($time['seconds']/2),512*($time['year']-1980)+32*$time['mon']+$time['mday']);

		$fr .= pack("V",$crc); // crc-32
		$fr .= pack("V",$c_len); // compressed size
		$fr .= pack("V",$unc_len); // uncompressed size
		$fr .= pack("v", strlen($encodedName)); // file name length
		$fr .= pack("v", 0 ); // extra field length
		$fr .= $encodedName; // file name
		echo $fr;

		$this->nbDatasec++;
		$dataLen=$c_len;
		$this->lenData+=strlen($fr)+$dataLen;

		// No compression: use standard file streaming proc
		if($compressionLevel==0) cfStreamFile($completeFilename,$this->t,$this->speedLimit,false,array('noHeaders'=>1,'noSessionClose'=>1));


		// Compression: if no speed limit, directly output created zip file
		else if(!$this->downloadSpeedLimit){
			$pos=0; $bufferSize=OUTPUT_BUFFER_SPEED;

			// Stream data
			while ($pos<$dataLen) {
				echo substr($zdata,$pos,$bufferSize);
				$pos+=$bufferSize;
				usleep(1000);
				cfPingUpdate(1000);
			}
			cfPingUpdate(1);

			$this->readSize+=$unc_len;
			// Transfert monitoring
			if($this->t && (!$this->t->exists() || $this->t->status==9)) exit; // Transfert stopped : quit
	 		if($this->t && $this->uncompressedSize) {
				$this->t->updateProgress($this->readSize/$this->uncompressedSize,true);
				$this->t->commitToFile(false,false);
			}
		}
		// Compression Speed limit
		else{
			$p=0;
			$bufferSize=min(OUTPUT_BUFFER_SPEED,$this->downloadSpeedLimit);
			$zipRatio=$unc_len/$this->lenData;

			while ($p<$dataLen) {
				echo substr($zdata,$p,$bufferSize);
				$this->speedLimit->addData($bufferSize);
				cfPingUpdate(100);
				$p+=$bufferSize;
				// Transfert monitoring
				if($this->t && (!$this->t->exists() || $this->t->status==9)) exit; // Transfert stopped : quit
				if($this->t && $this->uncompressedSize){
					$this->t->updateProgress(($this->readSize+$p*$zipRatio)/$this->uncompressedSize,true);
					$this->t->commitToFile(false,false);
				}
			}
			$this->readSize+=$unc_len;
			// Transfert monitoring
			if($this->t && (!$this->t->exists() || $this->t->status==9)) exit; // Transfert stopped : quit
			if($this->t && $this->uncompressedSize) {
				$this->t->updateProgress($this->readSize/$this->uncompressedSize,true);
				$this->t->commitToFile(false,false);
			}
		}

		$new_offset = $this->lenData;

		// Central directory header
		$cdrec = "\x50\x4b\x01\x02";// local file header signature
		$cdrec .="\x14\x0b";// version made by
		$cdrec .="\x00\x00";// version needed to extract
		$cdrec .="\x00\x00";// general purpose bit flag
		if($compressionLevel==0)  $cdrec .="\x00\x00"; else $cdrec .="\x08\x00";// compression method
		$cdrec .= pack('vv',2048*$time['hours']+32*$time['minutes']+floor($time['seconds']/2),512*($time['year']-1980)+32*$time['mon']+$time['mday']); // MS-DOS formated modification time & date
		$cdrec .= pack("V",$crc); // crc-32
		$cdrec .= pack("V",$c_len); // compressed size
		$cdrec .= pack("V",$unc_len); // uncompressed size
		$cdrec .= pack("v", strlen($encodedName)); // file name length
		$cdrec .= pack("v", 0 ); // extra field length
		$cdrec .= pack("v", 0 ); // file comment length
		$cdrec .= pack("v", 0 ); // disk number start
		$cdrec .= pack("v", 0 ); // internal file attributes
		$cdrec .= pack("V", 32 ); // external file attributes
		$cdrec .= pack("V", $this -> old_offset ); // relative offset of local header

		$this -> old_offset = $new_offset;

		$cdrec .= $encodedName;
		$this->nbCtrl_dir++;
		$this->nbCtrl_dir+=strlen($cdrec);

		$this -> ctrl_dir[] = $cdrec;

		return true;
	}

	// End of central directory record
	function file() {
out('------------- file() ?????? --------------');
		$ctrldir = implode("", $this -> ctrl_dir);
		echo
			$ctrldir .
			"\x50\x4b\x05\x06" .//  end of central dir signature
			"\x00\x00\x00\x00" . // number of this disk + number of this disk with the start of the central directory
			pack("v", sizeof($this -> ctrl_dir)) .// total number of entries in the central directory
			pack("v", sizeof($this -> ctrl_dir)) .// total number of entries in the central directory on this disk
			pack("V", strlen($ctrldir)) . // size of the central directory
			pack("V", $this->lenData) . // offset of start of central directory
			"\x00\x00";
	}
}
?>
