<?php
/**
 * Check timeout of logged users and check session files are all used (if cleanSessionFiles GET is set)
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2008 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

require(INCLUDE_DIR.'daemonFunctions.php');


/**
 * @desc Remove unused session files
 */
function cleanSessionFiles(){
	$ull=new WUsersLoggedList();
	$users=$ull->getUsers();

	// Browse session files to detect unused ones
	foreach (cfGlob(cfAppDataDir().'/sessiondata/sess_*') as $cfn){
		$sessId=substr(basename($cfn),5);
		if(!cfCmpLeft($sessId,'UI_') && !isset($users[$sessId])){
			@unlink($cfn);
		}
	}

	// Clean stuff
	unset($users);
	unset($ull);
}

/**
 * @desc Check for disconnected users (not pinging for more than allowed time)
 */
function disconnectTimedOutUsers(){
	$ull=new WUsersLoggedList();
	$users=$ull->getUsers();

	foreach ($users as $sessId=>$loggedUser) {
		if($loggedUser->isTimedOut()) $ull->disconnectUser($sessId);
	}

	// Clean stuff
	unset($users);
	unset($ull);
	unset($loggedUser);
}

/**
 * Ping BitTorrent seeder
 */
function checkBitTorrentSeeder(){
	if(WEnv::config()->getVar('bittorrentEnabled')){
		// Set info that seeder is being checked so debug info is not displayed by seeder
		cfMSetVar('checkingSeeder',time());

		// Open a connection on seeder to see if it's responding
		if($sock=@fsockopen('127.0.0.1',WEnv::config()->getVar('bittorrentPort'),$errno,$errstr,3)) fclose($sock);
		else{
			cfLog('BitTorrent seeder not responding, restart...',LOG_ERROR);
			cfPHPAsyncScript('/bittorrentSeeder.php?action=start',1000);
		}
	}
}


// Initialize daemon
WDaemonInstance::init();

WDaemonInstance::addCron('cleanSessionFiles','cleanSessionFiles',60); 				// Remove unused session files job
WDaemonInstance::addCron('disconnectTimedOutUsers','disconnectTimedOutUsers',5);	// Check for disconnected users (not pinging for more than allowed time)
WDaemonInstance::addCron('checkBitTorrentSeeder','checkBitTorrentSeeder',30,6);		// Check bittorrent seeder is still alive

/**
 ***************************************************************************************************************************
 * Main loop
 ***************************************************************************************************************************
 */
WDaemonInstance::loop();