<?php
/**
 * Process commands sent by weezo application
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.5
 */

ignore_user_abort(true);

// Initialize script data
require_once('localFunctions.php');
if(!function_exists('ifLoadRegInfo')) require(INCLUDE_DIR.'initFunctions.php'); // Workaround as require_once doesn't work !?!?!?

// Set key used for UI scripts access
if(isset($_GET['setWeezoKey']) && $_SERVER['REMOTE_ADDR']=='127.0.0.1'){
	if(!cfServerKeyIsset()) {
		// Set key
		cfServerKeySet($_GET['setWeezoKey']);

		// Force $_GET['setWeezoKey'] to pass cfIsInApp test below
		$_GET['weezoKey']=$_GET['setWeezoKey'];

		// Response to UI
		echo 'key set';
	}

	// Respond OK in case of double initialization
	elseif (cfServerKeyCheck($_GET['setWeezoKey'])) echo 'key set';
}



if(!cfIsInApp(true)) die('Unauthorized access to server management');

/**
 * Clear general.ini and languages data into server memory
 */
if(isset($_GET['commandResponse']) && isset($_GET['responseId'])){
	cfMSetVar('UICommandResponse'.$_GET['responseId'],$_GET['commandResponse']);
	exit;
}

/**
 * Clear general.ini and languages data into server memory (called once by 1st launch assistant)
 */
if(isset($_GET['resetConfig'])){
	@cfMUnsetVar('weezoGeneral');
	ifInitializeServer();
}

/**
 * Clear general.ini and languages data into server memory
 */
if(isset($_GET['resetGeneral'])){
	ifLoadGeneral();
}

/**
 * Update a single general.ini parameter
 */
if(isset($_GET['changeParameter'])) cfGUpdateVar($_GET['changeParameter'],$_GET['value'],'serverOnly');

/**
 * Set mailslot index, used for mailslot communication with application
 */
if(isset($_GET['setMailslotIndex']) && is_numeric($_GET['setMailslotIndex'])) cfMSetVar('mailslotIndex',$_GET['setMailslotIndex']);

/**
 * Set external IP address
 */
if(isset($_GET['setExternalIP']) && $_GET['setExternalIP']) cfGUpdateVar('externalIP',$_GET['setExternalIP'],'serverOnly');

/**
 * Ban IP address
 */
if(isset($_GET['banIP'])) cfIPBan($_GET['banIP']);

/**
 * Reset (only) user data into server memory
 */
if(isset($_GET['resetUsers'])){ifLoadUsers();}

/**
 * Reset (only) resources data into server memory
 */
if(isset($_GET['resetResources'])){
	if(isset($_GET['resourceFile'])) {
		ifLoadResource($_GET['resourceFile']);
		// Indicate UI resource tab web interface to focus on this resource at next reload
		cfGUpdateVar("uiResourceSelected",$_GET['resourceFile']);
	}
	else ifLoadResources();
}

/**
 * Reset resources definition
 */
if(isset($_GET['resetResourcesDefinition'])){ifResourcesDefinitions();}


/**
 * Reset all logged users from actually existing / associated resources
 */
if(isset($_GET['resetLoggedUsersResources'])){
	ifResetResourcesUsers(false && isset($_GET['resetUsers']),false && isset($_GET['resetResources']));
}

/**
 * Clear in-memory and files sessions
 */
if(isset($_GET['destroySession']) && substr($_GET['destroySession'],0,5)=='sess_'){
	@cfMUnsetVar($_GET['destroySession']);
	@unlink(cfAppDataDir().'/sessiondata/'.$_GET['destroySession']);
}

/**
 * Update weezo.net account registration information
 */
if(isset($_GET['updateRegInfo'])) ifLoadRegInfo();


if(isset($_GET['checkModules'])) ifCheckModules();


// Start bittorrent seeder
if(isset($_GET['bittorrentStart'])) cfPHPAsyncScript('/bittorrentSeeder.php?action=start',1000);

// Set password key
if(isset($_GET['setPwdKey'])) cfMSetVar('weezoPwdKey',base64_decode($_GET['setPwdKey']));


/**
 * Set server state ('web' or 'lan')
 */
if(isset($_GET['serverState']) && ($_GET['serverState']=='web' || $_GET['serverState']=='lan')) cfMSetVar('weezoServerState',$_GET['serverState']);


/**
 * Server shuting down: log event
 */
if(isset($_GET['serverShutdown'])){
	cfLogEvent(cfCaption('activityWeezoStop',false,false,false,true),EVENT_SERVERSTATE,S_EVENT_OFF);
}

/**
 * Application exited, but server should remain on: server get informed so application is not restarted if weezoCrashMonitor is set to true
 */
if(isset($_GET['appExited'])){
	cfMSetVar('appExited',1);
}

/**
 * Server starting (on web) for the 1st time since app launched: log
 */
if(isset($_GET['logStartEvent'])){
	// Monitor event
	if(cfRegistrationData()) {
		$regInfo=cfMGetVar('weezoRegInfo');
		$logCaption=cfCaption('apacheStartedWeb',false,false,false,true).cfCaption('genSeparator').cfSiteLink($regInfo['regName']);
	}
	else $logCaption=cfCaption('apacheStartedWeb',false,false,false,true).cfCaption('genSeparator');
	cfLogEvent($logCaption,EVENT_SERVERSTATE,((cfMGetVar('weezoServerState')=='web' || (@$_GET['serverState']=='web'))?S_EVENT_WEB:S_EVENT_LAN));
}

/**
 * Unlock remote desktop access. Confirmation code includes session ID and a secret code
 */
if(isset($_GET['unlockRDeskAccess']) && isset($_GET['confirmationCode'])){
	@list($sessionId,$code)=explode('/',$_GET['confirmationCode']);

	// Restart session with passed id in order to confirm desktop access right
	wSession_write_close();
	$_SESSION=null;

	$_ENV['wSession']['id']=$sessionId;
	wSession_start();

	// Check code and activate
	if($code==='forbidden')	cfGSetVar('readOnlyActivated','forbidden',false,false);
	if(cfGGetVar('readOnlyActivationCode')===$code)	cfGSetVar('readOnlyActivated',true,false);
	wSession_write_close();
}

/**
 * Update user interface bottom-left status & status icon
 */
if(isset($_GET['setUIStatus'])){
	if(isset($_GET['text']))
		cfMSetVar('uiMainStatus',@$_GET['iconURI'].'|'.cfUTF8Encode(base64_decode($_GET['text'])).'|'.(int)(@$_GET['displayProfileFormLink']));
	elseif(isset($_GET['utf8text']))
		cfMSetVar('uiMainStatus',@$_GET['iconURI'].'|'.$_GET['utf8text'].'|'.(int)(@$_GET['displayProfileFormLink']));
}



/**
 * One way communication to UI V2: loop and send content set into UICommand memory var
 */
if(isset($_GET['receiveCommands'])){
	if(!cfIsInApp()) die();

	ignore_user_abort(false);
	wSession_write_close();

	$res=@include(INCLUDE_DIR.'daemonFunctions.php');

	$loop=0;
	set_time_limit(0);

	echo '<UICommand>startReceivingCommands</UICommand>';

	while (1) {
		if(cfMGetVar('UICommand') && !cfMIssetVar('UICommandWriting')){
			cfMSetVar('UICommandReading',1);
			$buffer=cfMGetVar('UICommand');
			cfMSetVar('UICommand',"");
			cfMUnsetVar('UICommandReading');
			echo $buffer;
			flush();
		}

		// Check main daemon is running and if not, reboot (this might be usefull as UI V2 doesn't ping.php server anymore
		if(!($loop%200)){
			if(!WDaemons::daemon('main')->isAlive()) WDaemons::daemon('main')->restart();
		}

		usleep(10000);
		$loop++;
	}
}
?>