<?php
/**
 * Send error report
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.8
 */

//define('W_ERROR_REPORT_URL','http://localhost/test.php');
define('W_ERROR_REPORT_URL','http://weezo.net/dev/bugReport.php');
define('W_ERROR_REPORT_PROCESSES',false); // Set to true to display a list of running processes
define('W_ERROR_REPORT_CACHE_FILE',cfAppTempDir().'/report.txt');
define('W_ERROR_REPORT_RETURN_URL',cfLocalhostName().$_SERVER['PHP_SELF'].'?submited');

/**
 * @desc Generate report
 *
 */
function sERReport(){
	ob_start();
	echo APPLICATION_NAME." error report\n\n";

	// Registered name
	if($reg=cfRegistrationData()) echo 'weezo.net account: '.$reg['regName']."\n\n";

	// OS
	echo 'Weezo version: '.cfGGetVar('appVersion')."\n";
	echo "OS:".cfGetHostOSVersion()."\n";
	echo sERSep();

	// Directories
	echo "\nDirectories\n";
	echo sERSep();
	echo "Root Dir: ".cfAppRootDir()."\n";
	echo "Data Root Dir: ".cfAppDataRootDir()."\n";
	echo "Doc root: ".cfAppDocRoot()."\n"	;
	echo "bin: ".cfAppBinDir()."\n"	;

	// Processes
	echo "\nRunning Processes\n";
	echo sERSep();
	if(W_ERROR_REPORT_PROCESSES) foreach (cfProcList() as $key=>$value) echo $key."=".$value['exe']."\n";
	else echo "Skipped\n";

	// General.ini
	echo "\nMain configuration file\n";
	echo sERSep();
	echo file_get_contents(cfAppDataDir().'/general.ini');
	echo "\n\n";

	// php.ini
	echo "\nPHP.INI\n";
	echo sERSep();
	echo file_get_contents(cfAppDataDir().'/php.ini');
	echo "\n\n";

	// httpd.conf
	echo "\nhttpd.conf\n";
	echo sERSep();
	echo file_get_contents(cfAppDataDir().'/httpd.conf');
	echo "\n\n";

	// Configuration files

	// LOGS
	$out.="\n\nLogs\n";
	echo sERSep();
	sERRDisplayFile('httpdStartupError.txt');
	sERRDisplayFile('httpdErrors.log');
	sERRDisplayFile('phpErrors.txt');
	sERRDisplayFile('appLog.txt');

	// Files map
	$out.="\n\nFiles\n";
	echo sERSep();
	cfRecurseBrowse(cfAppDocRoot(),'sERFile');
	cfRecurseBrowse(cfAppBinDir(),'sERFile');
	cfRecurseBrowse(cfAppRootDir().'/Apache','sERFile');
	cfRecurseBrowse(cfAppRootDir().'/ApacheVC9','sERFile');
	cfRecurseBrowse(cfAppRootDir().'/PHP','sERFile');
	cfRecurseBrowse(cfAppRootDir().'/PHPVC9','sERFile');
	foreach (cfGlob(cfAppDataDir().'/*') as $cfn) sERFile($cfn);

	return ob_get_clean();
}

/**
 * @desc Display log file content
 *
 * @param string $filename: name of file located into log directory
 */
function sERRDisplayFile($filename){
	$filename=cfAppLogDir().'/'.$filename;
	echo basename($filename).' size: '.@filesize($filename);
	if(!file_exists($filename)) return;

	$maxSize=5000;
	if(filesize($filename)<$maxSize) echo file_get_contents($filename);
	elseif($fp=fopen($filename,'r')){
		fseek($fp,filesize($filename)-$maxSize);
		while(!feof($fp)) echo fread($fp,4096);
		fclose($fp);
	}
}

/**
 * @desc Display a file name & path, and it's size
 *
 * @param string $cfn: file
 */
function sERFile($cfn){
	echo $cfn.' '.filesize($cfn)."\n";
}

function sERSep(){
	flush();
	return ">***********************************************************************************************\n";
}

/**
 * @desc Upload data using POST/Curl
 *
 * @param array $postArray: dataName=>dataValue
 * @param string $url
 * @return boolean result
 */
function sERUpload($postArray, $url){
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($curl, CURLOPT_HTTPHEADER , array('Content-Type: multipart/form-data', 'Content-Transfer-Encoding: binary'));
    curl_setopt($curl, CURLOPT_POSTFIELDS, $postArray);
    return curl_exec($curl);
}

$_ENV['configurationEnvironment']='application';
require_once(INCLUDE_DIR.'outputFunctions.php');

// Ensure user has a registered account
if(!($reg=cfRegistrationData())){
	cfInsertHEAD(false,array('size'=>'800x600','icon'=>'bug','title'=>'Error reporting'));
	outDisplayErrorPage(cfCaption('registrationNeeded').'<br><br>'.outBt(cfCaption('genClose'),outIcon('close'),"javascript:void(wl.UICommand('close'))"));
	exit;
}


// Command line call (server not started): generate bug report and upload using CURL
if(cfIsCLI()){
	$res=sERUpload(array('userName'=>$reg['regName'],'report'=>gzencode('account:'.$reg['regName']."\n\nDescription".'SERVER NOT STARTED'."\n\n".sERReport(),9)),W_ERROR_REPORT_URL);
	exit;
}

// Proceed submited form
if(isset($_POST['description']) && is_file(W_ERROR_REPORT_CACHE_FILE)){

	$res=sERUpload(array(
			'userName'=>$reg['regName'],
			'report'=>base64_encode(gzencode('account:'.$reg['regName']."\n\nDescription".cfUTF8Decode($_POST['description'])."\n\n".file_get_contents(W_ERROR_REPORT_CACHE_FILE),9)),
		),
		W_ERROR_REPORT_URL);

	// All OK
	if(strpos($res,'OK')!==false){
		cfInsertHEAD(false,array('size'=>'800x600','icon'=>'warning','title'=>'Error reporting'));
		?><body>
		<script type="text/javascript">
		wl.UICommand('close');
		</script>
		</body> <?php
		exit;

	}

	// Error
	else{
		outDisplayErrorPage(cfCaption('genError').'<br><br>'.outBt(cfCaption('genClose'),outIcon('close'),"javascript:void(wl.UICommand('close'))"));
	}
	@unlink(W_ERROR_REPORT_CACHE_FILE);
	exit;
}

// Process weezo.net response to report submition
if(isset($_GET['submited'])){
	cfInsertHEAD(false,array('size'=>'800x600','icon'=>'warning','title'=>'Error reporting'));
?>
<body>
<script type="text/javascript">
wl.UICommand('close');
</script>
</body>
<?php
	exit;
}


// If "preload" is passed as a get parameter, just display empty page and reload. Used as User Interface wait for page to be loaded before displaying form
if(isset($_GET['preload'])){
	cfInsertHEAD(false,array('size'=>'800x600','icon'=>'warning','title'=>'Error reporting'));
?>
<body>
<script type="text/javascript">
setTimeout('D.location="<?php echo $_SERVER['PHP_SELF'];?>"',1);
</script>
</body>
<?php
	exit;
}



/**
 * @desc Submit PHP error report
 *
 */
cfInsertHEAD(false,array('size'=>'800x600','icon'=>'warning','title'=>'Error reporting'));
//echo cfScriptLink('winClient.js');
?>
</head>
<body>
<?php
echo outImage(outIcon('loadingBig'),false,'id="loadingIcon"','position:absolute;top:50%;left:50%;margin-left:-16px');
echo '<div style="position:absolute;bottom:7px;right:5px">';
echo outBt(cfCaption('genSend'),outIcon('go'),"javascript:doSubmit()",false,false,'style="display:none" id="submitBt"');
echo '&nbsp;&nbsp;&nbsp;&nbsp;';
echo outBt(cfCaption('genCancel'),outIcon('cancel'),"javascript:void(wl.UICommand('close'))",'loadingBt',false,'');
echo '</div>';
?>

<form name="bugReport" action="<?php echo $_SERVER['PHP_SELF']; ?>" method="POST">
<input name="userName" style="display:none" value="<?php echo $reg['regName'];?>">
<b><?php echo cfCaption('explorerFileSendMessage');?></b><br>
<textarea name="description" style="height:90px;width:100%;display:block;color:black" id="description"></textarea>
<script type="text/javascript">$('description').focus()</script>
<textarea name="report" style="height:440px;width:100%;display:block;color:black" readonly="readonly">
<?php
//echo $out;
$report=sERReport();
echo $report;
flush();
file_put_contents(W_ERROR_REPORT_CACHE_FILE,$report);
?>
</textarea>
<script type="text/javascript">
function doSubmit(){
	D.bugReport.submit();
	//wl.UICommand('close')
}
$('submitBt').style.display='';
$('loadingIcon').e.remove();
$('description').focus()
</script>
</form>