<?php
/**
 * Remote desktop control script
 *
 * Remote control of computer.
 * Features :	display desktop, divided in "squares", refresh only needed squares
 * 				capture keystrokes and mouseclicks
 *
 * this script is included by index.php and shouldn't be directly called
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

//cfMSetVar('debug',1);
define('RD_SQUARES_MAXWIDTH',256);
define('RD_DISABLE_LOCALHOST',true);

/**
 * INIT RESOURCE DATA
 */
function initResource(){
	if(cfRGetVar('resourceParametersInitialized')) return;
	// Disable inputs (read-only desktop, from /res/webcam/remoteDesktop)
	if(@$_ENV['readOnlyDesktop'] && !cfRGetVar('noHostConfirmation')) {
		$code=cfGenerateID(16);
		cfRSetVar('readOnly',true);
		cfGSetVar('readOnlyActivationCode',$code);
		cfGSetVar('readOnlyActivated',false); // Read-only desktop: this value is set to true or "forbidden" by UI->serverManagement.php?unlockRDeskAccess=x&confirmationCode=y

		// Show activation message box
		cfServerSendCmd('requestRDeskAuthorization',array('code'=>wSession_id().'/'.$code,'userLabel'=>WEnv::user()->label()));
	}

	//cfRInitVar('compression','auto'); // Default squares compression ("gif", "png", "jpg", "auto")
	//cfRInitVar('jpgQuality','80'); // Default jpg quality

	cfRInitVar('quality','1'); // Default quality from 0 (fast, ugly) to 2 (nice, slow)

	cfRInitVar('screenUpdateMethod','canvas'); // Method used for screen update : 'canvas' or 'squares'

	// Set default client window width & height
	cfRSetVar('clientWidth',(cfGGetVar('userScreenWidth'))?cfGGetVar('userScreenWidth'):1280);
	cfRSetVar('clientHeight',(cfGGetVar('userScreenHeight'))?cfGGetVar('userScreenHeight'):1024);

	// Mobile iOS webkit canvas speed issues: force squares
	if(cfBGetVar('OS')=='iOS' || cfBGetVar('OS')=='webOS' || cfBGetVar('OS')=='Android') {
		cfRSetVar('screenUpdateMethod','squares');
		if(cfBGetVar('OS')=='Android') {
			cfRSetVar('screenUpdateMethod','canvas');
			cfRsetVar('adaptZoom',true); // Fake adapt zoom to viewport size
		}
		cfRSetVar('screenFixedSquares',true);
		cfRsetVar('forcedCompression','png'); // Force png compression
	}

	// Android
	if(0&&cfBGetVar('name')=='Android') {
		cfRSetVar('screenUpdateMethod','squares');
		cfRSetVar('screenFixedSquares',false);
		cfRsetVar('forcedCompression','png'); // Force png compression
		cfRsetVar('adaptZoom',true); // Fake adapt zoom to viewport size
		cfRsetVar('fixedArea',1);
	}

	cfRInitVar('maskSquaresSide',32); // Width & height of squares for mask method

	cfRInitVar('dblClickTime',600);
	cfRInitVar('refreshAfterEvent',300);
	cfRInitVar('refreshAfterNoEvent',300);

	cfRInitVar('squaresMaxWidth',RD_SQUARES_MAXWIDTH);

	cfRInitVar('screenCurrent',1); // Current screen (screens numbering starts from 1, not zero)

	// Init sequential number used to avoid double processing of same async request
	cfRSetVar('seq',1);

	// No zoom
	cfRInitVar('adaptZoom',false);// cfRSetVar('zoom',1.0);

	// Mobile devices: use alt click mode
	if(cfIsMobile()) cfRSetVar('altClick',true);

	/**
	 * Init screens
	 */
	if($screensResolution=rDGetMonitors()){
		$screenNb=count($screensResolution);
		$screenCurrent=max(min(cfRGetVar('screenCurrent'),$screenNb),1);
		$screenWidth=$screensResolution[$screenCurrent-1]['width'];
		$screenHeight=$screensResolution[$screenCurrent-1]['height'];

		// Store data
		cfRSetVar('screenWidth',$screenWidth);
		cfRSetVar('screenHeight',$screenHeight);

		// Android: compute initial zoom level
		if(cfRGetVar('fixedArea')){
			$zoom=max(cfRGetVar('clientWidth'),cfRGetVar('clientHeight'))/$screenHeight;
			cfRsetVar('zoom',$zoom);
			cfRSetVar('fixedAreaWidth',floor($screenWidth*$zoom));
			cfRSetVar('fixedAreaHeight',floor($screenHeight*$zoom));
			//cfDbg('usw:'.cfGGetVar('userScreenWidth').'x'.cfGGetVar('userScreenHeight'));
			//cfDbg(cfRGetVar('zoom'));
		}
	}
	// Error
	else{
		die('Error: cannot get monitor(s) information');
	}

	// parameters loaded...
	cfRSetVar('resourceParametersInitialized',true);

	// If adapted zoom requested, just display a page setting window inner width & height
	if(cfRGetVar('adaptZoom') && !cfIsMobile()){
		cfInsertHEAD();
		require_once(INCLUDE_DIR.'outputFunctions.php');
?>
<body onload="init()">
<?php outInsertStandardComForm($_SERVER['PHP_SELF']);?>
<script type="text/javascript">function init(){fAS(false,'adapt',winMe.getInnerWidth(),winMe.getInnerHeight())}</script>
</body>
<?php
		exit;
	}
}

/**
 * @desc Update screen data and all squares
 *
 */
function rDSyncScreenUpdate(){
	global $screenWidth;
	global $screenHeight;
	global $screenNb;;
	global $screenCurrent;

	// Init screens
	if($screensResolution=rDGetMonitors()){
		$screenNb=count($screensResolution);
		$screenCurrent=max(min(cfRGetVar('screenCurrent'),$screenNb),1);
		$screenWidth=$screensResolution[$screenCurrent-1]['width'];
		$screenHeight=$screensResolution[$screenCurrent-1]['height'];
	}
	// Error
	else{$screenNb=1;$screenCurrent=1; $screenWidth=0; $screenHeight=0;}

	// Compute zoom from client window dimensions
	$zoom=cfRGetVar('zoom');
	cfRSetVar('clientWidth',cfGGetVar('userScreenWidth'));
	cfRSetVar('clientHeight',cfGGetVar('userScreenHeight'));

	// Auto compute number of x & y squares
	if(cfIsMobile()){
		cfRSetVar('remoteDesktopXSquares',max(1,round(($screenWidth)/cfRGetVar('squaresMaxWidth'))));
		cfRSetVar('remoteDesktopYSquares',max(1,round(($screenHeight)/cfRGetVar('squaresMaxWidth'))));
	}
	else{
		cfRSetVar('remoteDesktopXSquares',max(1,round(($zoom*$screenWidth)/cfRGetVar('squaresMaxWidth'))));
		cfRSetVar('remoteDesktopYSquares',max(1,round(($zoom*$screenHeight)/cfRGetVar('squaresMaxWidth'))));
	}

	// Store data
	cfRSetVar('screenCurrent',$screenCurrent);
	cfRSetVar('screenWidth',$screenWidth);
	cfRSetVar('screenHeight',$screenHeight);

	// mobile webkit specific
	if(cfBGetVar('OS')=='iOS'||cfBGetVar('name')=='Android') {
		//$minScale=@min(cfGGetVar('userScreenWidth')/$screenWidth,cfGGetVar('userScreenHeight')/$screenHeight);
		//cfRSetVar('meta','<meta name="viewport" content="width='.$screenWidth.', height='.$screenHeight.', maximum-scale=1.0, minimum-scale='.$minScale.', initial-scale='.$minScale.', user-scalable=yes">');
	}
	// Fix zoom factor (Android)
	if(cfRGetVar('fixedArea')) 	cfRSetVar('meta','<meta name="viewport" content="width='.cfRGetVar('fixedAreaWidth').', height='.cfRGetVar('fixedAreaHeight').', initial-scale=1, user-scalable=no">');

	// Required by IE9 for canvas
	if(cfGetBrowser()=='ie') cfRSetVar('meta','<meta http-equiv="X-UA-Compatible" content="IE=9" />');

	// Do update screen
	rDScreenUpdate(true);
}

/**
 * @desc Send a keyboard event on host
 *
 * @param int $keyCode: javascript/Windows key code
 * @param bool $altDown: true if alt is pressed (deprecated)
 * @param bool $ctrlDown: true if alt is pressed (deprecated)
 * @param bool $shiftDown: true if alt is pressed (deprecated)
 * @param bool $keyDown: true if key is pressed, false if released
 */
function rDSendKeyboardEvent($keyCode,$altDown,$ctrlDown,$shiftDown,$keyDown){
	global $refreshDesktop;

	if(function_exists('customSendKeyboardEvent')) return customSendKeyboardEvent($keyCode,$altDown,$ctrlDown,$shiftDown,$keyDown);
	$refreshDesktop=true;
	$_ENV['rDEventsQueueForUI'][]=array($keyCode,($keyDown)?"d":"u");
}

/**
 * @desc Type a string on host / Used for mobile access
 *
 * @param string $string: string to type, including {XXX} special chars or combos, and with backslash-escaped "\" and "{"
 */
function rDSendStringEvent($string){
	/**
	 * @desc Send a special key, sent using buttons or drop-down list
	 * @param string $key: special key name (DOWN, BACKSPACE..., see specialKeys())
	 */
	if(!function_exists('rDSendSpecialKey')){
		function rDSendSpecialKey($key){
			static $sk=false;
			if(!$sk) $sk=specialKeys();

			$ctrl=0;

	//cfDbg('in braces:'.$key);
			// Search for key in known keycodes
			foreach ($sk as $keyCode=>$v) if($v==='{'.$key.'}') {
				$found=1;
				break;
			}
			if(!@$found) return ;

			// Special: copy & paste: turn into ctrl-c or ctrl-v
			if($found && ($keyCode==998||$keyCode==999)){
				$ctrl=1;
				if($keyCode==998) $keyCode=99;// Copy (c)
				if($keyCode==999) $keyCode=118;// Paste (v)

				rDSendKeyboardEvent(17,false,false,false,true);
				customSendKeyboardString(rDAsciiFix($keyCode),-1);
				rDSendKeyboardEvent(17,false,false,false,true);
			}
			// Send key up + down
			else{
				//$keyCode=rDAsciiFix($keyCode);
				rDSendKeyboardEvent($keyCode,true,false,false,true); usleep(1000);
				rDSendKeyboardEvent($keyCode,true,false,false,false); usleep(1000);
			}
		}
	}

	$i=0;
	$c=false;
	$inBraces=false;

	while ($i<strlen($string)) {
		$c=substr($string,$i,1);
		$i++;
		$nextC=@substr($string,$i,1);

		// Unescape \ and {
		if($c==='\\' && ($nextC=='\\'||$nextC=='{')){
			customSendKeyboardString2($nextC,-1);
			$i++;
		}
		// Entering braces
		elseif ($c=='{') {
			$inBraces=true;
			$bracesKey='';
		}
		// Leaving braces
		elseif($c=='}' && $inBraces){
			$inBraces=false;
			rDSendSpecialKey($bracesKey);
		}
		// Adding to in-braces buffer
		elseif ($inBraces) $bracesKey.=$c;

		// Normal char: send
		else {
			cfDebugSingle('SEND:'.$c);
			customSendKeyboardString2($c,-1);
		}
	}
}

/**
 * @desc Send a mouse event
 *
 * @param string $action: mld|mlu|mmv|mlc|mrc|mrd|mru
 * @param int $x
 * @param int $y
 */
function rDSendMouseEvent($action,$x,$y){
	global $refreshDesktop;

	if(function_exists('customSendMouseEvent')) return customSendMouseEvent($action,$x,$y);
	$refreshDesktop=true;
	$_ENV['rDEventsQueueForUI'][]=array($x.'x'.$y,'m'.$action);
}

/**
 * @desc Request update of squares (rDesk.dll through php_custom.dll)
 *
 * @param bool $forceUpdate: set to true to request a full screen update, false to request a diffential update
 * @return string squares update string, or "refreshScreen" if a sync page reload is requested
 */
function rDScreenUpdate($forceUpdate=false){
	$screens=rDGetMonitors();
	// If current screen has been turned off, reset to 1st screen, and reset zoom
	if(!isset($screens[cfRGetVar('screenCurrent')-1])){
		cfRSetVar('screenCurrent',1);
		cfRSetVar('zoom',1);
		$reset=true;
	}

	if(!($screen=@$screens[cfRGetVar('screenCurrent')-1])) return;

	// If current screen resolution has changed
	if ($screen['width']!=cfRGetVar('remoteDesktopWidth') || $screen['height']!=cfRGetVar('remoteDesktopHeight')){
		cfRSetVar('zoom',1);
		cfRSetVar('remoteDesktopWidth',$screen['width']);
		cfRSetVar('remoteDesktopHeight',$screen['height']);
		$reset=true;
	}

	// If monitor resolution or activation has changed, and we're async refreshing, request a synchronous refresh
	if(isset($reset) && cfIsAsync()) return 'refreshScreen';

	$zoom=cfRGetVar('zoom');

	// Initial image: degrade quality
	if(!cfIsAsync() && !isset($_POST['noDownScale'])) $zoom/=4;

	switch (cfRGetVar('quality')){
		case 0:
			$compression='jpg';
			if($zoom<0.6) $jpgQuality=75; else $jpgQuality=67;
			break;
		case 1:
			$compression='jpg';
			if($zoom<0.6) $jpgQuality=85; else $jpgQuality=75;
			break;
		case 2:
		default:
			$compression='png';
			if($zoom<0.7) {
				$jpgQuality=100; // useless
				$compression='png';
			}
			else {
				$jpgQuality=92;
				$compression='pngjpg';
			}
	}
	if(cfRGetVar('forcedCompression')) $compression=cfRGetVar('forcedCompression'); // Apply quality and zoom independant compression (used for mobile access, forced to png)

	cfRSetVar('compression',$compression); // Save in order to select right image on browser's request

	//cfDebugSingle($zoom.' - '.cfRGetVar('screenCurrent').' - comp:'.$compression.' q='.$jpgQuality.' 99 '.cfRGetVar('maskSquaresSide'));

	// convert queued inputs to a string to be sent to UI (OSX/Linux)
	$inputs='';
	if(isset($_ENV['rDEventsQueueForUI'])){
		foreach ($_ENV['rDEventsQueueForUI'] as $v) {
			list($kc,$kd)=$v;
			$inputs.='/'.$kd.$kc;
		}
		$inputs=substr($inputs,1);
	}

	/**
	 * Update using mask method
	 */
	if(cfRGetVar('screenUpdateMethod')=='canvas'){
		if(function_exists('customRDeskUpdate2'))
			$ret=customRDeskUpdate2(
				$zoom,
				cfRGetVar('screenCurrent'),
				cfRGetVar('maskSquaresSide'),
				str_replace('/','\\',cfAppResourceDir()), // Path to store squares jpg's
				(int)$forceUpdate,
				$compression,
				$jpgQuality
			);
		else{
			$ret=cfServerSendRequest('rDeskUpdateCanvas',array(
				'zoom'=>$zoom,
				'screenNb'=>cfRGetVar('screenCurrent'),
				'maskSquaresSide'=>cfRGetVar('maskSquaresSide'),
				'path'=>cfAppResourceDir(),
				'forceUpdate'=>(int)$forceUpdate,
				'saveFormats'=>$compression,
				'jpgQuality'=>$jpgQuality,
				'inputs'=>$inputs
			));
		}
		// Send screenshot width & height
		$updateString=floor(cfRGetVar('remoteDesktopWidth')*cfRGetVar('zoom')).'x'.floor(cfRGetVar('remoteDesktopHeight')*cfRGetVar('zoom')).'/';
//cfDbg($updateString);

		// Send number of columns and lines
		$nbx=ceil(floor(cfRGetVar('remoteDesktopWidth')*cfRGetVar('zoom'))/cfRGetVar('maskSquaresSide'));
		$nby=ceil(floor(cfRGetVar('remoteDesktopHeight')*cfRGetVar('zoom'))/cfRGetVar('maskSquaresSide'));
		$updateString.=$nbx.'x'.$nby.'/';

		// Compact update string
		$s='0';$nb=0;$started=false;
		for($i=0;$i<strlen($ret);$i++){
			$c=substr($ret,$i,1);
			if($c===$s) $nb++;
			else {
				if($started) $updateString.=':';
				if($nb!==1) $updateString.=$nb;
				$started=true; $s=$c; $nb=1;
			}
		}
		$updateString.=(($started)?':':'').$nb;
	}

	/**
	 * Take screenshot and do the differential update
	 */
	else{
		// Using windows DLL
		if(function_exists('customRDeskUpdate')){
			//cfDbg($compression.' > '.$jpgQuality.' z='.$zoom.' x='.cfRGetVar('remoteDesktopXSquares').' y='.cfRGetVar('remoteDesktopYSquares'));

			$updateString=customRDeskUpdate(
				$zoom,
				cfRGetVar('screenCurrent'),
				cfRGetVar('remoteDesktopXSquares'),
				cfRGetVar('remoteDesktopYSquares'),
				str_replace('/','\\',cfAppResourceDir()), // Path to store squares jpg's
				(int)$forceUpdate,
				$compression,
				$jpgQuality
			);
		}
		// Through application
		else{
			$ret=cfServerSendRequest('rDeskUpdateSquares',array(
				'zoom'=>$zoom,
				'screenNb'=>cfRGetVar('screenCurrent'),
				'nbSquaresX'=>cfRGetVar('remoteDesktopXSquares'),
				'nbSquaresY'=>cfRGetVar('remoteDesktopYSquares'),
				'path'=>cfAppResourceDir(),
				'forceUpdate'=>(int)$forceUpdate,
				'saveFormats'=>$compression,
				'jpgQuality'=>$jpgQuality,
				'inputs'=>$inputs
			));
		}
	}

	return $updateString;
}

/**
 * @desc return array of JS keycode to VB keycodes for "special" keys
 * @return array : keys : JS keycode, values : V keycodes
 */
function specialKeys(){
	return array(
		8=>'{BACKSPACE}',
		19=>'{BREAK}',
		20=>'{CAPSLOCK}',
		46=>'{DELETE}',
		35=>'{END}',
		13=>'{ENTER}',
		27=>'{ESC}',
		480=>'{HELP}',
		36=>'{HOME}',
		45=>'{INSERT}',
		40=>'{DOWN}',
		37=>'{LEFT}',
		38=>'{UP}',
		39=>'{RIGHT}',
		34=>'{PGDN}',
		33=>'{PGUP}',
		144=>'{NUMLOCK}',
		44=>'{PRTSC}',
		145=>'{SCROLLLOCK}',
		9=>'{TAB}',
		112=>'{F1}',
		113=>'{F2}',
		114=>'{F3}',
		115=>'{F4}',
		116=>'{F5}',
		117=>'{F6}',
		118=>'{F7}',
		119=>'{F8}',
		120=>'{F9}',
		121=>'{F10}',
		122=>'{F11}',
		123=>'{F12}',
		124=>'{F13}',
		125=>'{F14}',
		126=>'{F15}',
		127=>'{F16}',
		91=>'{WINL}',
		92=>'{WINR}',
		998=>'{COPY}',
		999=>'{PASTE}');
}

/**
 * @desc Replace a value between 0 & 255 by it's character
 *
 * @param integer $charCode
 * @return string
 */
function rDAsciiFix($charCode){
	return chr($charCode);
	static $fix=array(
		'8859-1'=>'                                  - ',
		'8859-2'=>'                                 A?LLSSTZ\[shc]Za?lls?stz?zRALCCEEDNNORUUTralcceeddnnoruut?'
	);
	static $lngCodes=array('fr'=>'8859-1','pl'=>'ISO-8859-2');

	if($charCode<128) return chr($charCode);

	return $fix[$lngCodes[cfGGetVar('language')]][$charCode-128];

	return $fixed;
}

/**
 * @desc return 1 if keypress fix must be applied for key up & down detection
 *
 */
function rDKeypressFix(){
	if(cfGetBrowser()=='opera') return 1;
	return 0;
}

/**
 * @desc Return an array of width & height of all monitors
 *
 * @return array(monitorNb=>array("width"=>width,"height"=>height),...)
 */
function rDGetMonitors(){
	static $monitors=array();
	if(count($monitors)) return $monitors;

	if(function_exists('customGetMonitors')) $monitorsString=customGetMonitors(); else $monitorsString=cfServerSendRequest('rDeskGetMonitors');

	if($monitorsString) foreach (explode('/',$monitorsString) as $mon){
		list($size,$position)=explode(':',$mon);
		list($w,$h)=explode('x',$size);
		list($l,$t)=explode('x',$position);
		$monitors[]=array('width'=>$w,'height'=>$h,'left'=>$l,'top'=>$t);
	}

	return $monitors;
}

/**
 * @desc Process async commands passed in POST parameters
 * 		seq: sequential number of request, used to prevent processing twice a request
 * 		mixedEvents: string of passed keyboard / mouse events
 */
function rDProcessAsyncCommands(){
	if(!cfIsAsync()) return;

	$actions='';

	$refreshDesktop=false; // If set to true, update screenshot squares

	// Asynchronous header
	cfAsyncHeader();

	/**
	 * Check sequential request number to avoid double processing of a request
	 */
	if(isset($_POST['seq'])){
		// Already processed
		if($_POST['seq']<=cfRGetVar('seq')){
			if(cfMGetVar('debug')) echo cfAsyncXMLJSaction('dbgw("REPEAT '.$_POST['seq'].' : '.(isset($_POST['mixedEvents'])?$_POST['mixedEvents']:'').'")');
			echo cfAsyncJSaction('asOK()');
			die(cfAsyncFooter());
		}
		// Increment last processed request sequential number
		cfRSetVar('seq',$_POST['seq']);
	}

	/**
	 * Multiple keystrokes and/or clipboard commands sent
	 */
	if(isset($_POST['mixedEvents'])){
		$mixedEvents=str_replace('*weezoamp*','&',$_POST['mixedEvents']);

		// Refresh desktop (except if client requested not to because image is not yet loaded
		if(!@$_POST['noScreenUpdate']) $refreshDesktop=true;

		// change zoom type ('adapt' or 'restore')
		if(isset($_POST['zoomType']) && ($_POST['zoomType']=='adapt' || $_POST['zoomType']=='restore')){
			cfRSetVar('adaptZoom',($_POST['zoomType']=='adapt'));
			rDUpdateZoom((int)@$_POST['winInnerWidth'],(int)@$_POST['winInnerHeight']);
			$refreshDesktop=true; // Force desktop refresh even if image is currently being loaded
		}

		// detect (and remove from string) get server clipboard request(s)
		$requestClipboard=0;
		$mixedEvents=str_replace('/cg/','/',$mixedEvents,$requestClipboard);
		if(substr($mixedEvents,0,3)=='cg/'){
			$requestClipboard=1;
			$mixedEvents=substr($mixedEvents,3);
		}
		if(substr($mixedEvents,-3,3)=='/cg'){
			$requestClipboard=1;
			$mixedEvents=substr($mixedEvents,0,strlen($mixedEvents)-3);
		}
		if($mixedEvents=='cg'){
			$requestClipboard=1;
			$mixedEvents='';
		}

		// Get server clipboard and send to client
		if($requestClipboard && !cfRGetVar('readOnly') && function_exists('customGetClipboardContent')){
			$clipboard=customGetClipboardContent();
			if(!$clipboard) $clipboard=cfCaption('clipboardNoData');
			if(substr($clipboard,0,5)=='text/'){
				$clipboard=str_replace('<','&lt;',cfUTF8Encode(substr($clipboard,5)));
				$clipboard=str_replace(chr(13),'!weezobr!',str_replace(chr(10),'!weezobr!',str_replace(chr(13).chr(10),'!weezobr!',str_replace(chr(10).chr(13),'!weezobr!',$clipboard))));
				$clipboard=str_replace('"','\\"',str_replace('\\','\\\\',$clipboard));
				if(cfGetBrowser()=='ie') echo cfAsyncXMLJSaction('pasteInLocalClipboardIE("'.$clipboard.'");');
				else echo cfAsyncXMLJSaction('pasteInLocalClipboard("'.$clipboard.'");');
			}
			if(substr($clipboard,0,5)=='file/'){
				//$clipboardFiles=explode('/',substr($clipboard,5));
				cfRSetVar('clipboardFiles',$clipboard);
				echo cfAsyncXMLJSaction('downloadClipboardFiles();');
			}
		}

		// paste clipboard on server
		if((substr($mixedEvents,0,2)=='cp' || strpos($mixedEvents,'/cp')) && !cfRGetVar('readOnly') && function_exists('customSetClipboardContent')) {
			if(substr($mixedEvents,0,2)=='cp') $pos=2; else $pos=strpos($mixedEvents,'/cp')+3;
			$mixedEvents=substr($mixedEvents,$pos);

			// Other events may be placed after clipboard request (mmv...)
			$otherEvents='';
			if(strpos($mixedEvents,'/')) {
				$otherEvents=substr($mixedEvents,strpos($mixedEvents,'/')+1);
				$mixedEvents=substr($mixedEvents,0,strpos($mixedEvents,'/'));
			}

			// Decode sent data
			$clipboard=str_replace('*weezoslash*','/',$mixedEvents);

			// Convert using system codepage (more accurate on special chars)
			if(cfGGetVar('systemCodepage') && is_numeric(cfGGetVar('systemCodepage'))) $clipboard=@iconv('UTF-8','WINDOWS-'.cfGGetVar('systemCodepage'),$clipboard);
			// If system codepage not set, use regular utf8 decoding
			else $clipboard=cfUTF8Decode($clipboard,true,true,false,99999999999);

			// Insert text in clipboard
			if(customSetClipboardContent($clipboard)){
				// And paste (send ctrl-v)
				$mixedEvents='kd;17;c/kd;86;c/ku;86;c/ku;17;/'.$otherEvents;
				echo cfAsyncXMLJSaction('blink("clipboardOutBt",3);');
			}
			// Error setting data: don't send ctrl-v
			else {
				$mixedEvents=$otherEvents;
				echo cfAsyncXMLJSaction('blink("clipboardOutBt",9);');
			}
		}

		// set application actions
		$mixedEvents=str_replace(chr(13),'',$mixedEvents);// internal bug WA: \n are sent alongside with {ENTER}


		/**
		 * Send mouse and keyboard inputs to computer
		 */
		if(!cfRGetVar('readOnly')) rDSendInputs(cfUTF8Decode($mixedEvents,true,false,false,false));
	}

	// Update client window width & height, adapt zoom if necessary
	if(($winInnerWidth=@$_POST['winInnerWidth']) && ($winInnerHeight=@$_POST['winInnerHeight'])) rDUpdateZoom((int)$winInnerWidth,(int)$winInnerHeight);

	// Client orientation (mobiles)
	$o=@$_POST['orientation'];
	if($o==90||$o==-90) cfRSetVar('orientation','landscape');
	elseif($o==180||$o==0) cfRSetVar('orientation','portrait');
	else cfRSetVar('orientation',false);

	// Toggle inputbox display
	if(isset($_POST['displayInputBox'])) cfRSetVar('displayInputBox', $_POST['displayInputBox']);


	/**
	 * Refresh squares
	 */
	if($refreshDesktop){
		// Update squares
		$updateString=rDScreenUpdate(isset($_POST['forceWholeScreenUpdate']));

		// Refresh whole screen
		if($updateString=='refreshScreen') echo cfAsyncXMLJSaction('wl.goURL()');
		// Refresh only updated squares
		else {
			echo cfAsyncXMLJSaction('updateSquares("'.$updateString.'");');
		 	if(cfMGetVar('debug')) echo cfAsyncXMLJSaction('winMe.setTitle("'.$_POST['seq'].' - OK");');
		}
	}

	// Reset client-size state
	echo cfAsyncJSaction('asOK()');
	die(cfAsyncFooter());
}

/**
 * @desc Process Sync commands
 */
function rDProcessSyncCommands(){

	// Browser couldn't initialize canvas: reload with squares
	if(isset($_GET['unsupportedCanvas'])) cfRSetVar('screenUpdateMethod','squares');

	if(!isset($_POST['data3'])) return;
	$action=$_POST['data3'];

	// Clipboard file(s) download
	if($action=='downloadClipboardFiles' && !cfRGetVar('readOnly')){
		if(cfRGetVar('clipboardFiles')){
			$clipboardFiles=explode('/',substr(cfRGetVar('clipboardFiles'),5));
			if(count($clipboardFiles)){
				require_once(INCLUDE_DIR.'explorerFunctions.php');
				// Single file download
				if(count($clipboardFiles)==1 && is_file($clipboardFiles[0])){
					if(!cfIsGeneralProtected($clipboardFiles[0])) efForceDownload($clipboardFiles[0]);
				}
				// Zipped files download
				elseif(count($clipboardFiles)>0) {
					if(count($clipboardFiles)==1) $label=basename($clipboardFiles[0]); else $label=cfCaption('genClipboard');
					unset($clipboardFiles);
					foreach (explode('/',substr(cfRGetVar('clipboardFiles'),5)) as $value) $clipboardFiles[basename($value)]=$value;
					efZipDownload($clipboardFiles, $label.'.zip', cfUGetVar('downloadSpeedLimit'));
				}
			}
		}
	}
	// Process zoom change POST
		// data3: "adapt" or "restore"
		// data4: document width
		// data5: document height
	// Or screen selection
		// data2: screen number
		// data3: "setScreen"
		// data4: document width
		// data5: document height

	// Screen selection
	if($action=='setScreen') {
		$screensResolution=rDGetMonitors();

		if(!isset($screensResolution[$_POST['data2']-1])) die('Incorrect screen number: '.$_POST['data2']);
		cfRSetVar('screenCurrent',$screenCurrent=$_POST['data2']);
		cfRSetVar('screenWidth',$screenWidth=$screensResolution[$screenCurrent-1]['width']);
		cfRSetVar('screenHeight',$screenHeight=$screensResolution[$screenCurrent-1]['height']);

		// If adapted zoom, simulate zoom change request to recompute new zoom factor
		if(cfRGetVar('adaptZoom')) $action='adapt'; else $action='restore';
	}

	// 1/1 zoom
	if($action=='restore') {
		rDUpdateZoom((int)$_POST['data4'],(int)$_POST['data5']);
		cfRSetVar('adaptZoom',false);
	}
	// Stretched zoom
	elseif ($action=='adapt'){
		rDUpdateZoom((int)$_POST['data4'],(int)$_POST['data5']);
		cfRSetVar('adaptZoom',true);
	}
}

/**
 * @desc Send keyboard and mouse events to computer
 *
 * @param string $eventsString
 */
function rDSendInputs($eventsString){
	// Linux/OSX: used to ensure $refreshDesktop is set to true, to transmit inputs to UI with the screen refresh command (as inputs are not processed by DLL like for windows)
	global $refreshDesktop;

	$loggedEvents=array(); // Debug string
	$downKeys=array(); // List of down keys keycodes, used to release some keys at the end of command
	$dontLetDownKeys=array(91,92,16,17,18); // List of keys that must be released at the end of command

	$modifiersRestored=false; // True once artifically released modifiers have been re-pressed (if necessary)
	$modifiers=cfRGetVar('modifiersDown');
	$mod='';

	$isLocalhost=($_SERVER['REMOTE_ADDR']==='127.0.0.1');

	$monitors=rDGetMonitors();
	$offsetX=$monitors[cfRGetVar('screenCurrent')-1]['left'];
	$offsetY=$monitors[cfRGetVar('screenCurrent')-1]['top'];

	foreach (explode('/',$eventsString) as $event){

		$eventType=substr($event,0,1);
		$event=substr($event,1);

		if(!$eventType) continue;

		switch($eventType){
			// Mouse events.
			// Format: [mld|mlu|mmv|mlc|mrc|mrd|mru];x;y;[+][%][^] (where + is present if shift is down, % if alt is down and ^ if ctrl is down
			case 'm':
				list($action,$x,$y,$mod)=explode(';',$event);
				if($action!=='mv') {$loggedEvents[]=$event; $mouseEventsSent=true;}

				if($action!=='mv') cfDebugSingle($action.' '.($x+$offsetX).'x'.($y+$offsetY));

				if(!$isLocalhost || !RD_DISABLE_LOCALHOST) {
					// Restore modifiers state (from last request)
					if(!$modifiersRestored){
						if(isset($modifiers['a']) && strpos($mod,'a')!==false) rDSendKeyboardEvent(18,false,false,false,true);
						if(isset($modifiers['c']) && strpos($mod,'c')!==false) rDSendKeyboardEvent(17,false,false,false,true);
						if(isset($modifiers['s']) && strpos($mod,'s')!==false) {rDSendKeyboardEvent(161,false,false,false,true); rDSendKeyboardEvent(16,false,false,false,true);}
						$modifiersRestored=true;
					}

					// Send mouse event
					rDSendMouseEvent($action,$x+$offsetX,$y+$offsetY);
				}
				else{
					if(file_exists(cfAppBinDir().'/rDeskDbg.exe')) cfExecFork(cfAppBinDir().'/rDeskDbg.exe','SW_SHOW_NORMAL',$x.' '.$y);
				}
				break;

			// Keyboard events
			// Format: k[u|d|s];keycode;[|a|c|s] (http://msdn.microsoft.com/en-us/library/dd375731(v=VS.85).aspx for keycodes)
			case 'k':
			case 'c': // Clipboard paste
				if($eventsString=='c' && substr($event,0,1)!='p') continue; // Ensure only "P"aste is processed for clipboard
				$loggedEvents[]=$event;

				// Event is a string of characters (from mobile keyboard)
				if(substr($event,0,1)=='s' || substr($event,0,1)=='p'){
					cfDebugSingle('string ('.substr($event,0,1).'): '.substr($event,1));

					if(!$isLocalhost || !RD_DISABLE_LOCALHOST) rDSendStringEvent(substr($event,1));
				}
				// Event is an ascii or an iso character code
				elseif(substr($event,0,1)=='a'){
					list($action,$kc,$mod)=explode(';',$event);
					$action=substr($action,1);
					cfDebugSingle('ascii key ('.substr($event,0,1).'): '.$action.' kc='.rDAsciiFix($kc));

					if(!$isLocalhost || !RD_DISABLE_LOCALHOST) customSendKeyboardString(rDAsciiFix($kc),($action=='d')?1:0);
				}
				// Event is a single key up/down (+ modifiers)
				else{
					list($action,$kc,$mod)=explode(';',$event);
					$keyDown=($action==='d');
					cfDebugSingle('single key ('.substr($event,0,1).'): '.$action.' kc='.$kc);

					// Send keycode + alt + ctrl + shift + (true if key pressed, false if released)
					if(!$isLocalhost || !RD_DISABLE_LOCALHOST) {
						// Restore modifiers state (from last request)
						if(!$modifiersRestored){
							if(isset($modifiers['a']) && strpos($mod,'a')!==false) rDSendKeyboardEvent(18,false,false,false,true);
							if(isset($modifiers['c']) && strpos($mod,'c')!==false) rDSendKeyboardEvent(17,false,false,false,true);
							if(isset($modifiers['s']) && strpos($mod,'s')!==false) {rDSendKeyboardEvent(16,false,false,false,true);rDSendKeyboardEvent(161,false,false,false,true);}
							$modifiersRestored=true;
						}
						// Store down keys
						if($keyDown) $downKeys[$kc]=1; else unset($downKeys[$kc]);

						// Send keyboard event
						rDSendKeyboardEvent($kc,(strpos($mod,'a')!==false),(strpos($mod,'c')!==false),(strpos($mod,'s')!==false),$keyDown);
						if($kc==16) rDSendKeyboardEvent(161,false,false,false,$keyDown); // double shift down with right-shift
					}
				}
		}

		// Wait a little bit between 2 inputs (only if not processed by app)
		if(function_exists('customSendMouseEvent')) usleep(500);
	}

	$modifiers=array();

	// Release some keys (like Windows keys, alt ctrl and shift), to prevent weird behaviour on server
	foreach ($dontLetDownKeys as $kc) if(isset($downKeys[$kc])){
		unset($downKeys[$kc]);
		if($kc==16) $modifiers['s']=true;
		if($kc==17) $modifiers['c']=true;
		if($kc==18) $modifiers['a']=true;
		rDSendKeyboardEvent($kc,false,false,false,false);
	}

	// Store down shift/alt/ctrl keys (only if previously stored ones have been processed, i.e an event has been received)
	if($modifiersRestored) cfRSetVar('modifiersDown',$modifiers);

	//if(count($loggedEvents)) file_put_contents('C:/P1/keys.txt',$eventsString);
	//if(count($loggedEvents)) cfDebugSingle($loggedEvents);
	//if(count($loggedEvents) && cfIsAsync()) foreach ($loggedEvents as $le) echo cfAsyncJSaction('cfDbg("'.$le.'<br>",1)');

	// If any mouse event has been sent, wait a little before refreshing screen (only if not processed by app)
	if(isset($mouseEventsSent) && function_exists('customSendMouseEvent')) usleep(50000);
}

/**
 * @desc Update client-side window size and zoom
 *
 * @param int $clientWidth
 * @param int $clientHeight
 */
function rDUpdateZoom($clientWidth, $clientHeight){
	$w=$clientWidth;$h=$clientHeight;
	if(!is_numeric($w) || !is_numeric($h) || $w<=0) return;

	// Store client window width & height (used if screen is sync refreshed)
	cfRSetVar('clientWidth',$w); cfRSetVar('clientHeight',$h);

	// Compute zoom
	if(cfRGetVar('adaptZoom')){
		// iPhone method: compute server-side zoom from client-side viewport dimensions ($clientWidth, $clientHeight)
		if(cfRGetVar('screenFixedSquares')){
			$deviceWidth=cfGGetVar('userScreenWidth');
			$deviceHeight=cfGGetVar('userScreenHeight');
			// Set device width and height according to device orientation
			if(cfRGetVar('orientation')==='landscape' && $deviceWidth<$deviceHeight) cfSwap($deviceWidth,$deviceHeight);
			elseif(cfRGetVar('orientation')==='portrait' && $deviceWidth>$deviceHeight) cfSwap($deviceWidth,$deviceHeight);

			$zoom=$deviceWidth/$clientWidth;
			//cfDebugSingle('device w:'.$deviceWidth.' area w:'.$clientWidth.' => '.$zoom);
		}
		// Browser or android method: compute zoom from client-side area dimensions (iframe dimensions for browsers, or faked screen.width & height for android)
		else{
			// Compute zoom
			if($h==0 || cfRGetVar('screenWidth')/cfRGetVar('screenHeight')>=$w/$h) $zoom=@round($w/cfRGetVar('screenWidth'),3);
			else $zoom=round($h/cfRGetVar('screenHeight'),3);

			// Android: recompute faked screen dimensions
			if(cfRGetVar('fixedArea')){
				cfRSetVar('fixedAreaWidth',floor($zoom*cfRGetVar('screenWidth')));
				cfRSetVar('fixedAreaHeight',floor($zoom*cfRGetVar('screenHeight')));
//if(!cfIsAsync()) cfDbg('new zoom: '.$zoom.' new dims:'.cfRGetVar('fixedAreaWidth').'x'.cfRGetVar('fixedAreaHeight'));
			}
		}
		cfRSetVar('zoom',min(1,$zoom));
	}
	else cfRSetVar('zoom',1);
}

/**
 * @desc Send a square .jpg image. Parameters are in $_GET
 */
function rDSendSquareImage(){
	wSession_write_close(false);
	$_ENV['syncDebugCancel']=1;

	if(!is_numeric($_GET['img'])) die();
	$fName=cfAppResourceDir().'/sc'.$_GET['img'];
	$comp=cfRGetVar('compression');

	// Auto size selection
	if($comp=='auto'){
		$ext=(@filesize($fName.'.png')<@filesize($fName.'.jpg'))?'png':'jpg';
	}
	// jpg
	elseif($comp=='jpg') $ext='jpg';
	// png
	elseif($comp=='png') $ext='png';
	// gif
	elseif($comp=='gif') $ext='gif';

	$image=@file_get_contents($fName.'.'.$ext);

	// content type & length & no cache headers
	WHeaders::contentType($ext);
	header('Content-Length: '.strlen($image));
	if(stripos($_SERVER['HTTP_USER_AGENT'],'webkit')===false) WHeaders::noCache(); // Webkit memory leak bug workaround (http://code.google.com/p/chromium/issues/detail?id=36142)
	WHeaders::noCache(); // Webkit memory leak bug workaround (http://code.google.com/p/chromium/issues/detail?id=36142)

	// send file
	//cfStreamFile($fName.'.'.$ext,false);
	die($image);
	exit;
}

/**
 * @desc Send a square .jpg image. Parameters are in $_GET
 */
function rDSendMaskImage(){
	wSession_write_close(false);
	$_ENV['syncDebugCancel']=1;

	$fName=cfAppResourceDir().'/sc1';
	$comp=cfRGetVar('compression');

	// Auto size selection
	if($comp=='pngjpg'){$ext=(@filesize($fName.'.png')<@filesize($fName.'.jpg'))?'png':'jpg';}
	// jpg
	elseif($comp=='jpg') $ext='jpg';
	// png
	elseif($comp=='png') $ext='png';
	// gif
	elseif($comp=='gif') $ext='gif';


	// Image & no cache headers
	WHeaders::contentType($ext);
	if(stripos($_SERVER['HTTP_USER_AGENT'],'webkit')===false) WHeaders::noCache(); // Webkit memory leak bug workaround (http://code.google.com/p/chromium/issues/detail?id=36142)

	// send file
	//cfStreamFile($fName.'.'.$ext,false);
	die(file_get_contents($fName.'.'.$ext));

	exit;
}

/**
 * @desc Display an auto-updating page to wait for server-side confirmation of access to read-only shared desktop
 *
 */
function rDWaitForActivationPage(){
	// Async check of activation state
	if(cfIsAsync()){
		cfAsyncHeader();
		if(cfGGetVar('readOnlyActivated')==='forbidden') echo cfAsyncJSaction('setForbidden()'); // If access has been granted, reload page so remote desktop is displayed
		elseif(cfGGetVar('readOnlyActivated')===true) echo cfAsyncJSaction('wl.goURL()'); // If access has been granted, reload page so remote desktop is displayed
		die(cfAsyncFooter());
	}
	include_once(INCLUDE_DIR.'outputFunctions.php');

	// Wait page
	cfInsertHEAD();
	echo '<div id="loadMask">';
	echo outImage(outIcon('loading'),false,'id="loadingIcon"','position:absolute;top:50%;left:50%;margin:-8px 0 0 -8px');
	echo '<div style="position:absolute;top:50%;left:50%;visibility:hidden;margin-top:-30px" id="forbiddenAccess">';
	echo outImage(outIcon('forbiddenBig'),false,false,'margin-left:-24px').'<br><br>';
	echo '<div style="position:absolute;left:-75px;width:150px;text-align:center;font-weight:bold;margin-top:0.2em">'.cfCaption('loginForbiddenAccess').'</div>';
	echo '</div>';
	echo '</div>';
	?>
<script type="text/javascript">
function setForbidden(){
	dgi("forbiddenAccess").style.visibility="visible";
	dgi("loadingIcon").style.display="none";
	clearInterval(aC);
}
var aC=setInterval(function(){wl.asr.send({'activationCheck':1})},2000);
</script>
	<?php
	exit;
}

/*
 * Verify access rights (note: access rights are also checked in ./index.php (to display menu before error page)
*/
if(!cfUGetVar('administrator') && !@$_ENV['readOnlyDesktop']) {
	require_once(INCLUDE_DIR.'outputFunctions.php');
	outDisplayErrorPage(cfcaption('errorRemoteAdministrationNotAuthorized'));
}



/*
 ***************************************************************************************************************************
 * Process images access
 ***************************************************************************************************************************
 */
if(isset($_GET['img'])) rDSendSquareImage();
if(isset($_GET['mask'])) rDSendMaskImage();


/**
 * Init resource
 */
initResource();

// Read-only desktop: wait for server-side activation
if(cfRGetVar('readOnly') && (isset($_POST['activationCheck']) || (cfGGetVar('readOnlyActivated')!==true && !cfRGetVar('noHostConfirmation')))) rDWaitForActivationPage();

// Process asynchronous commands (keyboard/mouse inputs, screen update requests, clipboard request, misc. toggles
rDProcessAsyncCommands();

// Synchronous commands (zoom change, screen change, file download clipboard)
rDProcessSyncCommands();

// Full screen update screen
rDSyncScreenUpdate(true);

require_once(INCLUDE_DIR.'outputFunctions.php');

/*
 ***************************************************************************************************************************
 * Display page
 ***************************************************************************************************************************
 */
cfInsertHEAD(false,array('winNoBar'=>1));

$sqx=cfRGetVar('remoteDesktopXSquares'); $sqy=cfRGetVar('remoteDesktopYSquares');

// Insert CSS stuff
?>
<style type="text/css">
.hOut{position:absolute;top:-2000px;left:-2000px}
.sqi{display:block;position:absolute;width:100%;height:100%}
</style>
<?php

// Insert javascript variables
?>
<script language="javascript" type="text/javascript">
var debug=<?php echo (cfMGetVar('debug'))?'1':'0';?>,
readOnly=<?php echo (cfRGetVar('readOnly'))?'1':'0';?>,
seq=<?php echo cfRGetVar('seq');?>,
altClick=<?php echo (cfRGetVar('altClick'))?'1':'0';?>, altClickMenuShown=0,
keypressFix=<?php if(rDKeypressFix()) echo '{"list":{}}'; else echo '0'?>,
squaresNb=<?php echo cfRGetVar('remoteDesktopXSquares')*cfRGetVar('remoteDesktopYSquares');?>,
squaresX=<?php echo cfRGetVar('remoteDesktopXSquares')?>,
squaresY=<?php echo cfRGetVar('remoteDesktopYSquares')?>,
fixedSquares=<?php echo (int)cfRGetVar('screenFixedSquares')?>,
fixedArea=<?php echo (int)cfRGetVar('fixedArea')?>,
<?php
// Android fixed stuff
if(cfRGetVar('fixedArea')){
	echo 'fixedAreaWidth='.cfRGetVar('fixedAreaWidth').',';
	echo 'fixedAreaHeight='.cfRGetVar('fixedAreaHeight').',';
	echo 'clientWidth='.cfGGetVar('userScreenWidth').',';
	echo 'clientHeight='.cfGGetVar('userScreenHeight').',';
	//cfDbg(cfRGetVar('fixedAreaWidth').'x'.cfRGetVar('fixedAreaHeight'));
}
?>
browserOS="<?php echo cfBGetVar('OS');?>",
monW=<?php echo $screenWidth; ?>,monH=<?php echo $screenHeight;?>,
adaptZoom=<?php echo (int)cfRGetVar('adaptZoom');?>,
zoom=<?php echo cfRGetVar('zoom');?>,
maskSquaresSide=<?php echo cfRGetVar('maskSquaresSide');?>;
<?php echo cfJSRVar('screenUpdateMethod');?>;
<?php if(cfGetBrowser()=='ie') echo 'var wsbi=W.setInterval("watchScrollBars()",200);'; ?>
var displayInputBox=<?php echo (int)(cfRGetVar('displayInputBox'));?>;
<?php
// Special keycodes: deprecated?
// echo 'var sk=[];';foreach (specialKeys() as $code=>$value) echo 'sk['.$code.']="'.$value."\";\n";
?>
<?php echo cfJSRVar('dblClickTime');?>;
<?php echo cfJSRVar('refreshAfterEvent');?>;
<?php echo cfJSRVar('refreshAfterNoEvent');?>;
<?php echo cfJSCaptionVar('clipboardNoData');?>;
</script>
<?php

// Insert remote desktop javascript
echo cfScriptLink('remoteDesktop.js');
?>
</head>
<?php

if(!$screenWidth) die('Error accessing screen');


/******************************************************************************
 * Construct desktop
 *****************************************************************************/

if(cfRGetVar('screenFixedSquares')){
	?>
<body style="position:relative;margin:0;padding:0;width:<?php echo $screenWidth;?>px;height:<?php echo $screenHeight;?>px;overflow:hidden" onload="setTimeout('init()',100)">
<div id="squares" style="background:#444;width:<?php echo $screenWidth;?>px;height:<?php echo $screenHeight;?>px;">
<?php
	echo cfScriptLink('wz_dragdrop.js');
}
else{
	?>
<body style="background:url('bg.gif'); margin:0px;padding:0px; width:<?php echo floor($screenWidth*cfRGetVar('zoom'));?>px;height:<?php echo floor($screenHeight*cfRGetVar('zoom'));?>px;overflow:auto" onload="setTimeout('init()',100)">
<?php
echo cfScriptLink('wz_dragdrop.js');
echo '<div id="squares" style="overflow:hidden;display:block;padding:0; margin:0; border:0; position:absolute; width:'.(round(cfRGetVar('zoom')*$screenWidth)).'px;height:'.(round(cfRGetVar('zoom')*$screenHeight)).'px">';
}


?>
<!-- mouse drag mask -->
<div id="mask" title="" alt="" style="display:block; position:absolute;z-index:1000;top:0px; left:0px; width:<?php echo round(cfRGetVar('zoom')*$screenWidth).'px; height:'.round(cfRGetVar('zoom')*$screenHeight).'px;'.((cfGetBrowser()=='ie')?'background:black; filter:Alpha(opacity=1, style=0)"':'');?>" onmouseover="showMask()" oncontextmenu="return false">
</div>
<?php

$zoom=floor(1000*cfRGetVar('zoom'))/1000;
$bw=floor(cfRGetVar('zoom')*$screenWidth/$sqx);
$bh=floor(cfRGetVar('zoom')*$screenHeight/$sqy);

/******************************************************************************
 * Display squares
 *****************************************************************************/
if(cfRGetVar('screenUpdateMethod')=='squares'){
	for($i=0;$i<$sqx;$i++) {
		for($o=0;$o<$sqy;$o++) echo '<img  id="sq'.($i+$o*$sqx).'" class="sqi">'."\n";
	}
}

/******************************************************************************
 * Display canvas
 *****************************************************************************/
else if(cfRGetVar('screenUpdateMethod')=='canvas'){
	if(cfRGetVar('screenFixedSquares'))
		echo '<canvas id="maskCanvas" style="position:absolute;top:0;left:0;width:'.($screenWidth).'px;height:'.($screenHeight).'px" width="'.($screenWidth).'" height="'.($screenHeight).'"></canvas>';
	else
		echo '<canvas id="maskCanvas" style="position:absolute;top:0;left:0;width:'.($zoom*$screenWidth).'px;height:'.($zoom*$screenHeight).'px" width="'.($zoom*$screenWidth).'" height="'.($zoom*$screenHeight).'"></canvas>';
}

?>
<!-- selection mask -->
<div id="select" style="display:none; position:absolute; <?php if(cfGetBrowser()=='ie') echo 'border:1px solid #013A54; background:#6BB4D6; filter:Alpha(opacity=30, style=0)'; else echo 'border:1px solid #6BB4D6; background:url(\''.outIcon('sel.png').'\')';?>"></div>
<div id="mouseIcons" style="z-index:0"></div>

<?php
// Invisible key input
if(!cfRGetVar('altClick')){
	if(cfGetBrowser()=='safari')
		echo '<div style="position:absolute;max-width:10px;max-height:10px;overflow:hidden" id="keys2div"><textarea id="keys2" style="width:10px;height:10px;overflow:hidden;opacity:0.01" size="1"></textarea></div>';
	else
		echo '<div style="position:absolute;max-width:1px;max-height:1px;overflow:hidden" id="keys2div"><textarea id="keys2" style="width:1px;height:1px;overflow:hidden" size="1"></textarea></div>';
}

// key output Status box
echo '<div id="nStatus" style="border:1px solid #777; background:url(\'ibbg.png\');padding: 4px 7px 4px 7px; position:absolute; top:10px; color:black; left:-1000px; display:none;"></div>';

echo '</div>';





/******************************************************************************
 * UI Stuff...
 *****************************************************************************/



/**
 * Reduced controlBox
 */
if(!cfRGetVar('altClick')){
	echo '<div id="controlBox" style="border:2px solid #777; background:url(\'ibbg.png\'); display:block;padding: 0.4em; position:absolute; top:10px; color:black; font-weight:bold; white-space:nowrap; left:'.((cfRGetVar('displayInputBox'))?'-1000':'50').'px" onmouseover="fadeMask()">';
	echo outButton('', 'javascript:toggleInputBox()',outIcon('keybUnfold'));
	echo outImage(outIcon('ib_unfold'),false,false,'position:absolute; top:2px');

	// Zoom buttons
	if(cfRGetVar('adaptZoom'))
		echo outButton('','javascript:changeZoomType(\'restore\')',outIcon('zoomOut'),cfCaption('rDeskRestoreZoom'),false,'style="margin-left:10px"');
	else
		echo outButton('','javascript:changeZoomType(\'adapt\')',outIcon('zoomIn'),cfCaption('rDeskAdaptZoom'),false,'style="margin-right:10px"');

	// Screen selection buttons
	for($i=1;$i<=$screenNb;$i++){
		if($i<4) echo outButton('','javascript:setScreen('.$i.')',outIcon('screen'.(($i==$screenCurrent)?'S':'').$i));
		else echo outButton($i,'javascript:setScreen('.$i.')');
	}

	// Fullscreen button
	echo outButton('',"javascript:$('squares').e.toggleFullscreen(1)",outIcon('fscreen'));

	if(!cfRGetVar('readOnly')){
		// Clipboad buttons
		echo outButton('','javascript:sendClipboard()',outIcon('clipboardOut'),cfCaption('clipboardSend'),false,'style="margin-left:10px" name="clipboardOutBt"');
		echo outButton('','javascript:requestClipboard()',outIcon('clipboardIn'),cfCaption('clipboardRequest'),false,'style="margin-right:10px" name="clipboardInBt"');

		// Config form button
		echo outResourceConfigBt(false);

		// Help button
		echo outHelpButton('helpRDesk');
	}

	// Debug button
	if(cfRGetVar('debug')) echo outButton('TD','javascript:toggleDebug()');

	echo '<b id="controlBoxLE"></b>';
	echo '</div>';

	/**
	 * Maximized controlBox (inputBox)
	 */
	echo '<div id="inputBox" style="border:2px solid #777; background:url(\'ibbg.png\'); padding: 0.4em; position:absolute; top:10px; color:black; font-weight:bold; left:'.((cfRGetVar('displayInputBox'))?'50':'-1000').'px;white-space:nowrap" onmouseover="fadeMask()" onclick="inputBoxFocused=true" ondblclick="inputBoxFocused=true" oncontextmenu="return true;">';

	echo outButton('', 'javascript:toggleInputBox()',outIcon('keybFold'));
	echo outImage(outIcon('ib_fold'),false,false,'position:absolute; top:2px');

	// Zoom buttons
	if(cfRGetVar('adaptZoom'))
		echo outButton('','javascript:changeZoomType(\'restore\')',outIcon('zoomOut'),cfCaption('rDeskRestoreZoom'),false,'style="margin-left:15px"');
	else
		echo outButton('','javascript:changeZoomType(\'adapt\')',outIcon('zoomIn'),cfCaption('rDeskAdaptZoom'),false,'style="margin-right:15px"');

	// Fullscreen button
	echo outButton('',"javascript:$('squares').e.toggleFullscreen(1)",outIcon('fscreen'));

	// Screen selection buttons
	if($screenNb>1) for($i=1;$i<=$screenNb;$i++){
		if($i<4) echo outButton('','javascript:setScreen('.$i.')',outIcon('screen'.(($i==$screenCurrent)?'S':'').$i));
		else echo outButton($i,'javascript:setScreen('.$i.')');
	}

	if(!cfRGetVar('readOnly')){
		// Clipboad buttons
		echo outButton('','javascript:sendClipboard()',outIcon('clipboardOut'),cfCaption('clipboardSend'),false,'style="margin-left:10px" name="clipboardOutBt"');
		echo outButton('','javascript:requestClipboard()',outIcon('clipboardIn'),cfCaption('clipboardRequest'),false,'style="margin-right:10px" name="clipboardInBt"');

		// Config form button
		echo outResourceConfigBt(false);

		// Help button
		echo outHelpButton('helpRDesk');
		echo '<br>';

		// Send keys input
		echo '<div style="margin-left:0.5em;margin-top:4px">'.cfCaption('rDeskSendKeys').'</div>';
		echo '<input class="textInput" id="keys" type="text" value="" style="vertical-align:middle; margin-left:0.5em;margin-right:1em;" '.((cfGetBrowser()=='ie')?'onkeyup="keyUp()"':'').' onfocus="inputBoxFocused=true">';
		echo '<select id="specialKeySelect" size="1" style="vertical-align:middle;" onChange="insertKey()" onfocus="inputBoxFocused=true">';
		echo '<option value="Cancel">'.cfCaption('genCancel').'</option>';
		echo '<option value="^">CTRL</option>';
		echo '<option value="+">SHIFT</option>';
		echo '<option value="%">ALT</option>';
		echo '<option value="^%{DEL}">CTRL-ALT-SUPPR</option>';
		foreach (specialKeys() as $values) echo '<option value="'.$values.'">'.$values.'</option>';
		echo '</select>';
		echo outButton('','javascript:sendKeys()',outIcon('go'),cfCaption('genSend'),false);
		//echo outButton('','javascript:void(dgi(\'keys\').value=\'\')',outIcon('eraser'),cfCaption('genClear'));
		echo '<b id="inputBoxLE"></b>';
	}
	echo '</div>';

	/**
	 * Clipboard exchange forms
	 */
	echo outDivFrame('frame1','id="clipboardBox" onmouseover="fadeMask()"','position:absolute; width:510px;left:-1000px;top:-50px;');
	echo outFrameHeaderTable('frame1Header','<span id="clipboardCaptionRequest">'.cfCaption('clipboardRequest').'</span><span id="clipboardCaptionSend">'.cfCaption('clipboardSend').'</span>',outButton('','javascript:closeClipboardBox();',outIcon('close'),cfCaption('genClose')));
	echo '<div id="clipboardCaptionPaste">'.cfCaption('clipboardPaste').'</div>';
	echo '<center><textarea id="clipboardText" cols="53" rows="10" onfocus="clipboardBoxFocused=true" onBlur="clipboardBoxFocused=false" style="width:100%"></textarea></center>';
	echo '<a href="#" class="button" id="clipboardSendBt" style="display:none; width:0px;height:0px;position:absolute;left:-100px"></a>';
	echo outButton(cfCaption('genSend'),'javascript:sendClipboard(true);',outIcon('clipBoardOut'),false,'clipboardSendBt');
	echo '</div>';
}

outInsertStandardComForm($_SERVER['PHP_SELF'],false,false,false,false,false,false,true);


/*******************************************************************************
 * Alternate click menu
 ******************************************************************************/
if(cfRGetVar('altClick') && !cfRGetVar('readOnly')){
?>
<a name="acBt" id="acDbl" class="hOut" href="javascript:altClickAction('Dbl')"><img alt="Dbl Click" src="<?php echo outIcon('acDbl');?>"></a>
<a name="acBt" id="acDrag" class="hOut" href="javascript:altClickAction('Drag')"><img alt="Drag" src="<?php echo outIcon('acDrag');?>"></a>
<a name="acBt" id="acLc" class="hOut" href="javascript:altClickAction('Lc')"><img alt="Click" src="<?php echo outIcon('mouseL');?>"></a><br>
<a name="acBt" id="acRc" class="hOut" href="javascript:altClickAction('Rc')"><img alt="Right-click" src="<?php echo outIcon('mouseR');?>"></a>
<a name="acBt" id="acMv" class="hOut" href="javascript:altClickAction('Mv')"><img alt="Move" src="<?php echo outIcon('acMove');?>"></a>
<a name="acBt" id="acCancel" class="hOut" href="javascript:altClickAction('Cancel')"><img alt="Cancel" src="<?php echo outIcon('acCancel');?>"></a>
<a name="acBt" id="acKb" class="hOut" href="javascript:altClickAction('Kb')"><img alt="Cancel" src="<?php echo outIcon('acKeyboard');?>"></a>
<a name="acBt" id="acHome" class="hOut" href="javascript:altClickAction('Home')"><img alt="Home" src="<?php echo outIcon('home');?>"></a>
<?php
	// Android zoom in/out buttons
	if(cfRGetVar('fixedArea')){
		?><a name="acBt" id="acZoomIn" class="hOut" href="javascript:zoomFix('in')"><img alt="ZoomIn" src="<?php echo outIcon('fZoomIn');?>"></a><?php
		?><a name="acBt" id="acZoomOut" class="hOut" href="javascript:zoomFix('out')"><img alt="ZoomOut" src="<?php echo outIcon('fZoomOut');?>"></a><?php
	}

	// Screen selection buttons
	for($i=1;$i<=$screenNb;$i++) if($i!=$screenCurrent){
		?><a name="acBt" id="acScreen<?php echo $i;?>" class="hOut" href="javascript:setScreen(<?php echo $i;?>)"><img alt="S<?php echo $i;?>" src="<?php echo outIcon('screen'.$i);?>"></a><?php
	}
?>
<div id="acText" style="text-align:justify;position:absolute;top:-2000px;z-index:1001">
<?php
if(cfBGetVar('OS')=='Android' && !cfRGetVar('readonly')){
Aze	// Mobile text input: set larger size for text input to avoid automatic browser zoom
	?>
<textarea id="acTxtInput" type="text" style="position:absolute;left:50%;width:3px;height:100px;font-size:60px;padding:0;overflow:hidden;<?php echo outCssOpacity(1);?>" autocorrect="off" autocapitalize="off" autocomplete="off" onblur="this.hasFocus=0;setTimeout('keybHideMenu(1)',250)" onfocus="this.hasFocus=1" onkeyup="keybKU(W.event,this)"></textarea>
	<?php
}
elseif(!cfRGetVar('readonly')){
	?>
<textarea id="acTxtInput" type="text" style="position:absolute;left:50%;width:3px;height:3px;font-size:1px;padding:0;overflow:hidden" autocorrect="off" autocapitalize="off" autocomplete="off" onblur="this.hasFocus=0;setTimeout('keybHideMenu(1)',250)" onfocus="this.hasFocus=1" onkeyup="keybKU(W.event,this)"></textarea>
	<?php
}

$icoDir=cfAppDocRoot().'/res/'.WEnv::res('type').'/'.WEnv::res('subType').'/';

//foreach (specialKeys() as $k=>$v) echo $icoDir.strtolower(str_replace(array('{','}'),'',$v)).'.png<br>';
foreach (specialKeys() as $k=>$v) {
	$key=strtolower(str_replace(array('{','}'),'',$v));
	if(is_file($icoDir.'k'.($key).'.png')) {
		echo '<a name="keybBt" class="acBt" href="javascript:keybAction(\'{'.strtoupper($key).'}\')"><img alt="" src="k'.$key.'.png"></a>';
	}
}

?>

<div style="display:none">
<input type="checkbox" id="acCtrl" style="display:none"><a href="javascript:altClickCheck('acCtrl')">Ctrl</a>
<input type="checkbox" id="acAlt" style="display:none"><a href="javascript:altClickCheck('acAlt')">Alt</a>
<input type="checkbox" id="acShift" style="display:none"><a href="javascript:altClickCheck('acShift')">Shift</a>

<select id="acKeySelect" size="1" style="width:30%;overflow:hidden" onchange="if(!this.value) return;dgi('acTxtInput').value+=this.value;this.value=''">
<option value="">-</option>
<option value="^">CTRL</option>
<option value="+">SHIFT</option>
<option value="%">ALT</option>
<option value="^%{DEL}">CTRL-ALT-SUPPR</option>
<?php foreach (specialKeys() as $values) echo '<option value="'.$values.'">'.$values.'</option>'; ?>
</select>

</div>

</div>
<img id="target" class="hOut" src="<?php echo outIcon('target');?>">
<?php
}

?>
<script language="javascript" type="text/javascript">
var screenWidth=<?php echo $screenWidth;?>, screenHeight=<?php echo $screenHeight;?>,imgBuff=[];
var imgCache={'l':(new Image()),'r':(new Image()),'w':(new Image())};
imgCache.l.src='<?php echo outIcon('mouseL');?>';
imgCache.r.src='<?php echo outIcon('mouseR');?>';
imgCache.w.src='<?php echo outIcon('mouseW');?>';
<?php

// Draggable items
cfDragAddItem('MouseIcons');
if(cfRGetVar('altClick')){
	cfDragAddItem('altClickDiv');
}
else{
	cfDragAddItem('mask','+ELASTIC+SCROLL+RESET_Z');
	cfDragAddItem('controlBox','+CURSOR_MOVE+SCROLL');
	cfDragAddItem('inputBox','+CURSOR_MOVE+SCROLL');
	cfDragAddItem('clipboardBox','+CURSOR_MOVE+SCROLL');
}
echo cfDragRegisterItems('+CURSOR_MOVE+SCROLL',false);
if(cfIsMobile()) echo 'D.ontouchmove=function(){};';
?>
if(dd.elements.inputBox) dd.elements.inputBox.maximizeZ();
if(dd.elements.controlBox) dd.elements.controlBox.maximizeZ();
dragMinTimeForPreDrop=-1; // call preDrop even if mask drag time is 0
<?php
// Keep focussing on keys2 input
if(!cfRGetVar('altClick')) echo 'resumeFocus(1);';
?>
showMask();
if(winMe.addControls) {
	winMe.addControls('<?php

	/*******************************************************************************
	 * window bar controls
	 ******************************************************************************/
$output='';

// Zoom buttons
$output.=outButton('',"javascript:dgi('zoomRestore').style.display='none';dgi('zoomFull').style.display='';winIF(this).changeZoomType('restore')",outIcon('zoomOut'),str_replace("'",'`',cfCaption('rDeskRestoreZoom')),'zoomRestore','style="margin-left:15px'.((cfRGetVar('adaptZoom'))?'':';display:none').'"');
$output.=outButton('',"javascript:dgi('zoomFull').style.display='none';dgi('zoomRestore').style.display='';winIF(this).changeZoomType('adapt')",outIcon('zoomIn'),str_replace("'",'`',cfCaption('rDeskAdaptZoom')),'zoomFull','style="margin-right:15px'.((cfRGetVar('adaptZoom'))?';display:none':'').'"');

// Screen selection buttons
if($screenNb>1) for($i=1;$i<=$screenNb;$i++){
	if($i<4) $output.=outButton('','javascript:winIF(this).setScreen('.$i.')',outIcon('screen'.(($i==$screenCurrent)?'S':'').$i));
	else $output.=outButton($i,'javascript:winIF(this).setScreen('.$i.')');
}


// Fullscreen button
$output.=outButton('',"javascript:$('squares').e.toggleFullscreen(1);",outIcon('fscreen'));

// Action controls
if(!cfRGetVar('readOnly') && !cfRGetVar('altClick')){
	// Clipboad buttons
	$output.=outButton('','javascript:winIF(this).sendClipboard()',outIcon('clipboardOut'),cfCaption('clipboardSend'),false,'style="margin-left:10px" name="clipboardOutBt"');
	$output.=outButton('','javascript:winIF(this).requestClipboard()',outIcon('clipboardIn'),cfCaption('clipboardRequest'),false,'style="margin-right:10px" name="clipboardInBt"');

	// Config form button
	$cfb=str_replace("javascript:","javascript:winIF(this).",outResourceConfigBt(false));
	$cfb=str_replace('onclick="configureResource()','onclick="winIF(this).configureResource()',$cfb);
	$cfb=substr($cfb,0,strpos($cfb,'<scr'));
	$output.=$cfb;

	// Send keys input
	$output.='<input class="textInput" id="keys" type="text" value="" style="vertical-align:middle; margin-left:0.5em;margin-right:1em;" '.((cfGetBrowser()=='ie')?'onkeyup="keyUp()"':'').' onfocus="winIF(this).focusKeys()">';
	$output.='<select id="specialKeySelect" size="1" style="vertical-align:middle;" onChange="winIF(this).insertKey()" onfocus="winIF(this).inputBoxFocused=true">';
	$output.='<option value="Cancel">'.cfCaption('genCancel').'</option>';
	$output.='<option value="^">CTRL</option>';
	$output.='<option value="+">SHIFT</option>';
	$output.='<option value="%">ALT</option>';
	$output.='<option value="^%{DEL}">CTRL-ALT-SUPPR</option>';
	//foreach (specialKeys() as $values) $output.='<option value="'.$values.'">'.$values.'</option>';
	$output.='</select>';
	// Send keys button
	$output.=outButton('','javascript:winIF(this).sendKeys()',outIcon('go'),cfCaption('genSend'),false);
	// Clear keys input
	//$output.=outButton('','javascript:winIF(this).clearKeys()',outIcon('eraser'),cfCaption('genClear'));
	$output.='<b id="inputBoxLE"></b>';
}
$output=str_replace("'","\\'",str_replace("\n",'',str_replace("\r",'',$output)));
$output=str_replace('winIF(this)','winIF(\\\'\'+winMe.id+\'\\\')',$output);
echo $output;

?>');
	$('controlBox').e.remove();try{dd.elements.controlBox.del()}catch(e){}
	$('inputBox').e.remove(); try{dd.elements.inputBox.del()}catch(e){}
}
</script>
</body>
</html>