<?php
/**
 * Resource configuration base script
 *
 * - Verify access rights to resource configuration
 * - Load resource configuration file into an array ($resourceData)
 * - prepare HTML page (Head, body, css...)
 * - include resource configuration php script
 *
 * This script is either included by accounts.php (if accessed from a browser) or directly called by embeded browser of application
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

require_once(INCLUDE_DIR.'outputFunctions.php');
require_once(INCLUDE_DIR.'resourceConfigFunctions.php');

// Resource data directory (as cfAppResourceDir() function doesn't work properly in administration context
$resourceDataDir;

/*
 * Verify access rights
 */
if($_SERVER['REMOTE_ADDR']!='127.0.0.1'){
	if(cfUGetVar('administrator')!=true || !cfGGetVar('remoteAdministrationAuthorized')) outDisplayErrorPage(cfcaption('errorRemoteAdministrationNotAuthorized'));
}

/**************************************************************************************************
 * Advanced configuration
 **************************************************************************************************/
if((isset($_GET['resourceConfigFilename']) && $_GET['resourceConfigFilename']=='advancedConfig') || (isset($_POST['resourceConfigFilename']) && $_POST['resourceConfigFilename']=='advancedConfig')){
	$_ENV['configurationEnvironment']='application';
	$fileName='advancedConfig';
	// Insert page header
	if(!cfIsAsync() && !isset($_GET['noHead']) && !isset($_POST['noHead'])){
		cfInsertHEAD(true,array('icon'=>'configAdvanced','title'=>cfCaption('configAdvanced')));
		echo "<body>\n";
		echo cfScriptLink('wz_dragdrop.js');
	}
	$_ENV['buttonLessConfig']=false;
	require('advancedConfig.php');
	exit;
}



/**************************************************************************************************
 * Resource configuration
 **************************************************************************************************/
if(isset($_GET['config']) || isset($_GET['resourceConfigFilename']) || isset($_POST['config']) || isset($_POST['resourceConfigFilename']) || (isset($_SESSION['activeResourceId']) && cfRGetVar('configuringResource'))) {

	// Configuration called from web browser
	if(isset($_GET['config']) || isset($_POST['config']) || (isset($_SESSION['activeResourceId']) && cfRGetVar('configuringResource'))) {
		$_ENV['configurationEnvironment']='browser';
		if(isset($_GET['config'])) $fileName=$_GET['config'];
		elseif(isset($_POST['config'])) $fileName=$_POST['config'];
		else $fileName=cfRGetVar('configuringResource');
	}
	// Configuration called from IU resource tab
	if(isset($_GET['resourceConfigFilename']) || isset($_POST['resourceConfigFilename'])) {
		$_ENV['configurationEnvironment']='application';
		if(isset($_GET['resourceConfigFilename'])) $fileName=$_GET['resourceConfigFilename']; else $fileName=$_POST['resourceConfigFilename'];
		$_ENV['resourceConfigFilename']=$fileName;
	}

	// Buttonless config
	$_ENV['buttonLessConfig']=cfIsInApp();

	// Check and parse resource configuration file
	if(!($resId=array_search($fileName,cfMGetVar('weezoResourcesList'))) || !($resourceData=cfMGetVar('weezoResData'.$resId)) || (!isset($resourceData['type']) || !isset($resourceData['subType']) || !isset($resourceData['name']))) {
		// If problem reading configuration file
		echo '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"><HTML><HEAD>';
		echo '<meta http-equiv="content-type" content="text/html; charset=utf-8"><TITLE></TITLE>'."\n";

		if(file_exists(cfAppDataDir().'/general.ini') && ($applicationIni=cfParse_ini_file(cfAppDataDir().'/general.ini')) && isset($applicationIni['skin'])){
			echo '<link rel="stylesheet" type="text/css" href="/themes/skins/'.$applicationIni['skin'].'/theme.css">'."\n";
		}
		echo "</HEAD>\n";
		outDisplayErrorPage(cfCaption('adminResourceBadCfgFile'),cfCaption('genError'),true);
		exit();
	}

	/**
	 * Define a resource object for the configured resource
	 */
	$resourceObject=new WResConfig($resId);

	$_ENV['resourceDirectory']=cfAppDataDir().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.cfFileWithoutExtension($fileName);
	// Store read resource data to check wether it has changed or not when saving with rcWriteResourceFile ($originalResourceData passed as global var)
	$originalResourceData=$resourceData;

	// resource configuration file OK
	$standaloneConfig=false; // $standaloneConfig is true if configuration script doesn't need (want...) to be wrapped in common HTML stuff. $standaloneConfig is red from describer file (see below)
	$configStartsSession=false; // $configStartsSession is true if config script does "wSession_start()". $configStartsSession is red from describer file (see below)

	// check for descriptor file
	$resourceDefinition=cfMGetVar('weezoResourceDefinition'.strtolower($resourceData['type'].$resourceData['subType']));
	if(isset($resourceDefinition['describer'])) {
		$describerData=$resourceDefinition['describer'];
		if(@$describerData['standaloneConfig'] && @$describerData['configFile']) $standaloneConfig=true;
		if(@$describerData['configStartsSession']) $configStartsSession=true;
	}
	// Set resource data directory
	$resourceDataDir=cfAppDataDir().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.cfFileWithoutExtension($fileName);

	/**************************************************************************************************
	 * Standalone configuration: redirect
	 **************************************************************************************************/
	if($standaloneConfig) {
		// Seek config script
		$src='/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/';
		if(isset($describerData['configFile'])) $src.=$describerData['configFile'];
		elseif (is_file($tmp=cfAppDocRoot().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/config.php')) $src.=$tmp;
		else $src.=cfAppDocRoot().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/index.php';

		// Build args
		$args=array('standaloneConfig'=>1,'resourceConfigFilename'=>$fileName,'resourceConfigScript'=>true);
		if(isset($_GET['weezoKey'])) $args['weezoKey']=$_GET['weezoKey'];
		if(@$describerData['configStartsSession']) $args['configStartsSession']='true';
		if(count($args)) $src.='?'.http_build_query($args,null,'&');

		// Redirect
		header('Location: '.$src);
		exit;
	}

	/**************************************************************************************************
	 * COMMON POST COMMANDS
	 **************************************************************************************************/

	// Async thumbnails cleaning
	if(isset($_POST['clearThumbnails'])){
		// Unlink thumbnails
		rcThumbnailsClear();

		// Return response
		cfAsyncHeader();
		echo cfAsyncXMLJSaction('wl.mask.hide()');
		cfAsyncFooter();
	}








/**************************************************************************************************
 * Insert HTML wrapping (If standaloneConfig is not set to "true" in describer file)
 **************************************************************************************************/
	if(cfIsInApp() && !$standaloneConfig && !cfIsAsync()){
		ob_start(); // Buffer page for smoother display

		cfInsertHEAD(false);
		?>
<script type="text/javascript">
D.onmousemove=function(event){return (W.event && W.event.srcElement && (W.event.srcElement.tagName=='INPUT'||W.event.srcElement.tagName=='TEXTAREA'))}
wl.asr.extraArgs={'resourceConfigFilename':'<?php echo $fileName;?>'} // Add this param to ensure async requests work
</script>
</head>
<body id="uiResourceConfigBody">
		<?php

		// Drag'n drop scripts
		echo cfScriptLink('wz_dragdrop.js'); // disable text selection (don't kno why...)
		?><form style="position:absolute;top:-100px" name="deselForm"><input type="text" name="desel" value="a">a</input></form><?php
	}

	if(cfGGetVar('UIDebug') && cfIsInApp() && !cfIsAsync()) echo outButtonSmall(false,"javascript:void(prompt('url','".cfHostName().$_SERVER['REQUEST_URI'].'&WSESSID='.$_COOKIE['WSESSID']."'))",outIcon('bugs'),false,false,'style="position:absolute;left:50%"');

	// Remote administration configuration
	if(!cfIsInApp() && !$standaloneConfig){
		echo outDivFrame('frame1');
		echo outFrameHeaderTable('frame1Header',cfCaption('adminResourceConfiguration', cfUTF8Encode($resourceData['name'])),outButton(cfCaption('genDelete'),'javascript:deleteRes()',outIcon('del')));
	}
	elseif(cfIsInApp() && !$standaloneConfig && !cfIsAsync()) {
		echo '<div class="frame1">';
	}

	//
	// include configuration php script
	//
	$includedByResourceConfig=true;
	$limitedConfig=false;

	// If configuration scripts starts session by itself, close current session
	if($configStartsSession) {wSession_write_close(); /*unset($_SESSION);*/}

	// If async update, don't send generated page to output: buffer response then discard it
	// Remote admin sets rcFalseAsyncResponse to prevent this as it catches sync response to asynchronously put it in html node
	if(cfIsAsync() && !isset($_ENV['rcFalseAsyncResponse'])) ob_start();

	// Includes
	require_once(INCLUDE_DIR.'resourceConfigFunctions.php');
	require_once(INCLUDE_DIR.'outputFunctions.php');

	// Include configuration script
	set_include_path(get_include_path().PATH_SEPARATOR.str_replace('/','\\',cfAppDocRoot()).'\\'.$resourceData['type'].'\\'.$resourceData['subType']);



	if(isset($describerData['configFile']))
		$incFile=cfAppDocRoot().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$describerData['configFile'];
	else
		$incFile=cfAppDocRoot().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/config.php';

	try{
		if(is_file($incFile))
			$inc=include_once($incFile);
		else
			$inc=false;
	}
	catch (Exception $e){}

	// No configuration file: display only name configuration control
	if($inc===false){
		rcInsertScriptAndForm();

		$limitedConfig=rcConfigRights();
		if($limitedConfig) rcInitLimitedConfig();

		// Resource icon & name
		if(!$limitedConfig){rcInsertIconName($resourceData); echo '<br/>';}
		echo '<div style="font-weight:bold;margin-top:16em;margin-bottom:16em;text-align:center">'.cfCaption('resNoConfigNoFile').'</div>';
		// Save / cancel buttons
		echo rcButtonSaveCancel();
		echo '</form>';
		// Save parameters
		rcWriteResourceFile($resourceData, $fileName);
	}

	// Asynchronous resource update
	if(cfIsAsync() && !isset($_ENV['rcFalseAsyncResponse'])){
		ob_clean();
		cfAsyncHeader();
		if(isset($_ENV['rcAsyncResponse'])) echo $_ENV['rcAsyncResponse'];
		echo cfAsyncFooter(false);
	}

	// Close HTML stuff
	if(!$standaloneConfig && !cfIsAsync()){
		echo '</div>';
		if(cfIsInApp()) {
			if(cfGGetVar('UIDebug')) echo '<textarea style="width:100%">'.cfRGetVar('id')."\n".str_replace('/','\\',$_ENV['resourceDirectory'])."\n\n".cfHostName().$_SERVER['REQUEST_URI'].'</textarea>';
			//echo cfDragRegisterItems(false);
			echo '</BODY></HTML>'."\n";
			ob_end_flush();
		}
		elseif(!cfIsInApp()) echo '</div>'."\n";
	}
}
?>