<?php
/**
 * Multimedia
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/*
// ... and redirect to Video resource
require(cfAppDocRoot().'/res/explorer/Video/index.php');
exit;
*/
require(INCLUDE_DIR.'explorerFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

/*
 ***************************************************************************************************************************
 * Initialize resource
 ***************************************************************************************************************************
 */
function initVideoExplorerData2(){
	if(!cfRGetVar('resourceParametersInitialized')){

		cfRSetVar('resourceLayout','useInterval');
		cfRSetVar('useHTTPXmlRequest',1);

		cfRInitVar('extensionFilter',false); // array of extensions of displayed files. Set to false if no filter on file extension

		// Allowed file types
		if(strtolower(basename(dirname($_SERVER['SCRIPT_FILENAME'])))=='video')
			cfRSetVar('fileTypeFilter',array('video')); // All "multimedia" formats
		else
			cfRSetVar('fileTypeFilter',array('image','audio','video')); // Multimedia files only

		cfRInitVar('efSortBy','name'); cfRSetVar('efSortOrder','asc');// Use sort options

		cfRSetVar('efEditMode',false);// Startup comments edit mode (false:view, true:edit)
		cfRSetVar('thumbnailsIndex','0'); // index of first displayed thumbnail
		cfRSetVar('activePreview',0); // index of preview image

		// Number of items per page
		cfRInitVar('itemsPerPage',30);

		// Maximum flash output width (backward compatibility, as config defines flashOutputSize)
		cfRInitVar('flashOutputSizeMax',cfRGetVar('flashOutputSize'));
		// Maximum flash output quality (backward compatibility, as config defines flashOutputQuality)
		cfRInitVar('flashOutputQualityMax',cfRGetVar('flashOutputQuality'));

		// iPhone specific
		if(cfBGetVar('name')=='iPhone') {
			$w=2.4*cfRGetVar('thumbnailMaxWidth')+4*((cfRGetVar('thumbnailBorderFrame'))?24:0);
			cfRSetVar('meta','<meta name="viewport" content="width='.floor($w).'">');
		}

		// No-download browsers
		if(!cfBGetVar('download')){
			cfRSetVar('downloadShortcut',false);
			cfRSetVar('multipleDownloadAllowed',false);
			cfRSetVar('uploadAllowed',false);
		}

		// Wii specific parameters
		if(cfIsWII()){
			if(!cfRGetVar('reencodedVideo')){
				cfRSetVar('reencodedVideo',true);
				cfRSetVar('flashStream',true);
			}
			cfRSetVar('flashOutputSize',400);
			cfRSetVar('flashOutputQuality','medium');
		}

		if(cfRGetVar('resourceConfigured')) cfRSetVar('resourceParametersInitialized',true); // indicate that parameters have been initialized
		else $errorMessageCritical=cfCaption('configErrorNotConfigured');
	}


	// Init treeview
	global $tree;
	if(cfRGetVar('treeViewAvailable')){
		cfRSetVar('treeViewDisplayed',true);
		$tree=new treeViewFolder(cfRGetVar('treeViewAvailable'));
		$tree=treeFolderLoad(cfRGetVar('treeViewAvailable'));
		//$tree->syncSelectPath=true;
	}
	else cfRSetVar('treeViewDisplayed',false);
}

// Init common explorer data
efInit();

// Init file explorer data
initVideoExplorerData2();

/**
 * If mobile device, redirect to mobile explorer
 */
if(cfIsMobile()) {require(INCLUDE_DIR.'mobileExplorerFunctions.php');mefExplorer();exit;}

// Init treeView
if(cfRGetVar('treeViewAvailable')){
	$tree=new treeViewFolder(cfRGetVar('treeViewAvailable'));
	$tree=treeFolderLoad(cfRGetVar('treeViewAvailable'));
}

/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */
efProcessPOST();

/*
 ***************************************************************************************************************************
 * Get files and subfolders list, generate thumbnails if needed
 ***************************************************************************************************************************
 */
efDirUpdate();

// If using list view or icon view and async directory selection through treeView, update files view and exit
efFilesListAsyncUpdate();

/*
 ***************************************************************************************************************************
 * Insert HEAD, JS functions and form
 ***************************************************************************************************************************
 */
efInsertExplorerHeadAndBody();

/*
 ***************************************************************************************************************************
 * Start of resource specific section
 ***************************************************************************************************************************
 */

$tnS=cfRGetVar('thumbnailsSizeFactor')==0;
$bw=$tnS?12:24;
$totalWidth=$bw*2+efThumbnailsMaxWidth();
?>
<style type="text/css">
.title{display:block;width:<?php echo $totalWidth;?>;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}
.frame{vertical-align:top; background: black;border:1px solid #303030}
.tna{padding-top:<?php echo (($tnS)?6:14);?>px; padding-bottom:<?php echo (($tnS)?6:14);?>px}
.pl{height:100%;vertical-align:top;border-right:1px solid #111}
.pr{height:100%;vertical-align:top;border-left:1px solid #111}
.photoFrame{vertical-align:middle; background: #FFFFFF; border-left:1px solid #E0E0E0; border-right: 1px solid #B0B0B0; border-bottom:1px solid #B0B0B0; border-top: 1px solid #E0E0E0;width:auto}
fieldset{border:0px;padding:0;display:inline;vertical-align:middle;margin:0.5em;text-align:center}
.cdFrame{border:1px solid #AAA;padding:2;padding-left:12;background:url(<?php echo outIcon('cdB');?>) repeat-y #444}
.cdFrame img{border:1px solid #555}
</style>
<?php
echo outDivFrame('frame1',false,'height:auto !important;height:98%;min-height:98%');



/*
 ***************************************************************************************************************************
 * Frame header
 ***************************************************************************************************************************
 */


// Files layout selector and thumbnails size selector
if(!cfRGetVar('filesLayoutLocked')) $l=efFilesListSelector(array('resourceIcon'=>outIcon('displayVideo'))).'&nbsp;&nbsp;&nbsp;'.outThumbnailsSizeButtons(true); else $l="";
$r='';

// Sort controls
$r.=outFileSortControls(1);

// Thumbnails interval selection controls
$r.='&nbsp;&nbsp;&nbsp;&nbsp;'.outExplorerThumbnailsSelector();

// Search files button
if(cfRGetVar('searchFiles')) $r.='&nbsp;'.outButton(false,'javascript:searchFiles()',outIcon('search'),cfCaption('genSearch'));

echo outFrameHeaderTable('frame1Header',$l,' '.$r,'id="frame1HeaderTop"');



/*
 ***************************************************************************************************************************
 * Action panel (multiple downloads, image upload, navigation)
 ***************************************************************************************************************************
 */
if(efActionPanelDisplayed()) {
	echo outTableTransparent().'<tr>';
	efActionPanel('left',false,array());
	echo '<td id="leftTD" style="vertical-align:top;width:auto">';
}

/*
 ***************************************************************************************************************************
 * Thumbnails area
 ***************************************************************************************************************************
 */

echo outTableTransparent('frame1','height:100%','id="thumbnailsFrame"');
echo '<colgroup>';


// Thumbnails width
$thumbnailsDimension=$totalWidth;

// If no video file present in current directory
if($_ENV['weezoResEfFiles']['nbFiles']==0){
	echo '<tr><td style="vertical-align:top"><div id="noFileDiv" class="frame2" style="text-align:center;height:100%;min-height:100px;margin:0;overflow:auto"><span style="position:relative;top:49%">'.cfCaption('explorerNoVideoHere').'</div></td></tr>'."\n";
}

// List or icons view
else{
	/*
	$drag=(cfBGetVar('drag') && cfRGetVar('multipleDownloadAllowed')); // Draggable item
	$tnSF=cfRGetVar('thumbnailsSizeFactor'); // Size factor (0 smallest, 1 medium, 2 big)
	$tnI=(int)cfRGetVar('thumbnailsIndex'); // 1st displayed thumbnail index in list of files
	$iPP=(int)cfRGetVar('itemsPerPage'); if(!$iPP) $iPP=9999; // Number of items per page
	$bf=cfRGetVar('thumbnailBorderFrame'); // Video has border frame
	$pl=($tnSF)?'pl.gif':'pl0.gif'; // Video border frame background image
	$pr=($tnSF)?'pr.jpg':'pr0.gif'; // Video border frame background image
	$ds=cfRGetVar('downloadShortcut'); // Display download shortcut
	$cd=cfTGetVar('compactDisplay')||$tnSF==0; // Compact display (file name)
	$ttFI=cfHGetVar('tooltipFileInfo'); // Tooltip file info
	*/
	echo '<tr><td style="vertical-align:top"><div id="noFileDiv" class="frame2" style="overflow:auto;height:100%">';
	echo '</div></td></tr>';
}

echo '</table>';

if(efActionPanelDisplayed()) echo '</td></tr></table>';


/*
 ***************************************************************************************************************************
 * JS layout generation script
 ***************************************************************************************************************************
 */
?>
</div>
</div>
<script type="text/javascript">
function filesLayoutResource(files,tnw){
	var tW,pL,pR,bgLR,tnh=Math.floor(tnw*3/4),o='<div id="filesTable" style="text-align:left">';

	// Small
	if(!thumbnailsSizeFactor) {
		var pL='pl0.gif',pR='pr0.gif',tW=tnw+34,bgLR='bgl0.gif',m3uImg="<?php echo outIcon('fi/med/icoM3U');?>"
	}
	// Medium / big
	else{
		pL='pl.gif';pR='pr.jpg',tW=tnw+58,bgLR='bgl.gif',m3uImg="<?php echo outIcon('fi/big/icoM3U');?>";
	}
	var pPW=5+5*thumbnailsSizeFactor, cdBP=7*(1+thumbnailsSizeFactor)
	<?php // preload css background image, or it would be loaded after everything else ?>
	o+='<img src="<?php echo dirname($_SERVER['PHP_SELF']);?>/'+bgLR+'" style="position:absolute;left:-100px">';

	for(var i=0;i<files.length;i++){
		f=fL.file(i);

		u='<span style="position:relative">';

		u+='<fieldset id="'+((f.type=='d')?'d':'f')+i+'" onclick="view(\'id'+i+'\')" oncontextmenu="fL.cm(this,event)" onmouseover="tooltip(this,fL.tt)">';
		u+='<div class="fIC" id="id'+i+'" style="width:'+tW+'px">'+f.name+'</div>';
		u+='<div id="_id'+i+'" style="position:relative;width:'+tW+'px" <?php if(!cfIsWII()){?> onmousedown="fL.sDF(this,\'\',event)"<?php } ?>><center>';


		// Video
		if(<?php if(cfRGetVar('fileTypeFilter')==array('video')) echo '1||';?>f.mime=='v'){
			u+='<table class="frame" cellspacing="0" cellpadding="0"><tr>';

			if(!f.tnUrl) continue;
			u+='<td class="pl" style="background:url('+bgLR+') repeat-y"><img src="'+pL+'" alt=""></td><td class="tna">';
			u+='<img class="img" src="'+f.tnUrl+'" style="width:'+f.tnW+'px;height:'+f.tnH+'px'
				+';padding-top:'+(Math.floor((tnh-f.tnH)/2))+'px;padding-bottom:'+(Math.floor((tnh-f.tnH)/2))+'px;left:'+(Math.floor((tnw-f.tnW)/2))+'px'
				+';visibility:hidden" onload="fade(this)">';
			u+='</td><td class="pr" style="background:url('+bgLR+') repeat-y"><img src="'+pR+'" alt=""></td></tr></table>';

		}
		// Image
		else if(<?php if(cfRGetVar('fileTypeFilter')==array('image')) echo '1||';?>f.mime=='i'){
			u+='<div class="photoFrame" style="vertical-align:middle;width:'+(f.tnW+pPW-4)+'px;padding:'+pPW+'px">';
			u+='<img class="img" src="'+f.tnUrl+'" style="width:'+f.tnW+'px;height:'+f.tnH+'px;top:'+(Math.floor((tnh-f.tnH)/2))+'px;left:'+(Math.floor((tnw-f.tnW)/2))+'px;visibility:hidden" onload="fade(this)">';
			u+='</div>';
		}
		// Audio
		else{
			if(f.name.substring(f.name.length-3).toLowerCase()=='m3u'){
				u+='<img src="'+m3uImg+'" alt="M3U">';
			}
			else{
			u+='<div class="cdFrame" style="padding-left:'+cdBP+'px;width:'+f.tnW+'px;background-position:'+(cdBP-30)+'px 0px">';
			u+='<img class="" src="'+f.tnUrl+'" style="width:'+f.tnW+'px;height:'+f.tnH+'px;top:'+(Math.floor((tnh-f.tnH)/2))+'px;left:'+(Math.floor((tnw-f.tnW)/2))+'px;visibility:hidden" onload="fade(this)">';
			u+='</div>';
			}
		}
		u+='</center></div></fieldset>'

		// Rating
		u+=wl.ratingItem(i,f.rating,!thumbnailsSizeFactor,{'center':1,'extraStyle':'left:50%;bottom:-'+(20+5*thumbnailsSizeFactor+f.tnH/2)+'px;position:absolute'})

		u+='</span>';
		o+=u;
	}
	o+='</div>';
	return o;
}
</script>
<?php	echo cfDragRegisterItems(); ?>
<script type="text/javascript">
<?php if(cfRGetVar('treeViewDisplayed')){ ?> var treeViewFrameID="<?php echo $tree->HTMLDivID();?>"; <?php } ?>

var viewMaximized=<?php echo (int)cfConstant('MAXIMIZE_VIDEOS_VIEW'); ?>,
fLviewOnClick=1;

function winResized(w,h){
	alignBottom(dgi("thumbnailsFrame"),winMe,10);
	if(dgi("noFileDiv")) alignBottom(dgi("noFileDiv"),winMe,10);
	if(W.treeViewFrameID && dgi('actionsPanel').style.display!='none') alignBottom(dgi(treeViewFrameID),dgi("thumbnailsFrame"),0);
	dd.recalc();
}
function setMFResizable(){dd.elements[treeViewFrameID].warm();}
function setMFUnresizable(){dd.elements[treeViewFrameID].freeze();}
winResized();
</script>
<?php
efFilesList('noFileDiv',array('noSortControls'=>1));
?>
</BODY>
</HTML>